@@ -1,5 +1,5 @@
 /* Management of historical information about an Xconq game.
-   Copyright (C) 1992-1997 Stanley T. Shebs.
+   Copyright (C) 1992-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,20 +7,19 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern int find_event_type PARAMS ((Obj *sym));
-extern int update_total_hist_lines PARAMS ((Side *side));
-extern int build_hist_contents PARAMS ((Side *side, int n, HistEvent **histcontents,
-					int numvishistlines));
+extern int find_event_type(Obj *sym);
+extern int update_total_hist_lines(Side *side);
+extern int build_hist_contents(Side *side, int n, HistEvent **histcontents,
+			       int numvishistlines);
 
-int pattern_matches_event PARAMS ((Obj *pattern, HistEvent *hevt));
-void event_desc_from_list PARAMS ((Side *side, Obj *lis, HistEvent *hevt,
-				   char *buf));
+int pattern_matches_event(Obj *pattern, HistEvent *hevt);
+void event_desc_from_list(Side *side, Obj *lis, HistEvent *hevt, char *buf);
 
-static void notify_event PARAMS ((Side *side, HistEvent *hevt));
-static void play_event_movies PARAMS ((Side *side, HistEvent *hevt));
-static void rewrite_unit_references PARAMS ((int oldid, int newid));
-static void rewrite_unit_references_in_event PARAMS ((HistEvent *hevt,
-						      int oldid, int newid));
+static void notify_event(Side *side, HistEvent *hevt);
+static void play_event_movies(Side *side, HistEvent *hevt);
+static void rewrite_unit_references(int oldid, int newid);
+static void rewrite_unit_references_in_event(HistEvent *hevt, int oldid,
+					     int newid);
 
 extern int any_post_event_scores;
 extern int need_post_event_scores;
@@ -73,7 +72,7 @@ int statistics_wanted = TRUE;
 int statistics_dumped;
 
 void
-init_history()
+init_history(void)
 {
     /* The first "event" is just a marker. */
     history = create_historical_event(H_LOG_HEAD);
@@ -86,7 +85,7 @@ init_history()
 }
 
 void
-start_history()
+start_history(void)
 {
     /* Ignore multiple starts. */
     if (recording_events)
@@ -96,8 +95,7 @@ start_history()
 }
 
 HistEvent *
-create_historical_event(type)
-HistEventType type;
+create_historical_event(HistEventType type)
 {
     HistEvent *hevt = (HistEvent *) xmalloc(sizeof(HistEvent));
 
@@ -109,20 +107,16 @@ HistEventType type;
     return hevt;
 }
 
+/* Record a historical event. */
+
 HistEvent *
-#ifdef __STDC__
 record_event(HistEventType type, SideMask observers, ...)
-#else
-record_event(type, observers, d1, d2, d3, d4)
-HistEventType type;
-SideMask observers;
-int d1, d2, d3, d4;
-#endif
 {
     int i, val;
     char *descs;
     HistEvent *hevt;
     Side *side;
+    va_list ap;
 
     if (!recording_events)
       return NULL;
@@ -131,25 +125,16 @@ int d1, d2, d3, d4;
     hevt->enddate = g_turn();
     hevt->observers = observers;
     descs = hevtdefns[type].datadescs;
-#ifdef __STDC__
-    {
-	va_list ap;
 
-	va_start(ap, observers);
-	for (i = 0; descs[i] != '\0'; ++i) {
-	    if (i >= 4)
-	      run_error("hevt type %d has too many parameters", type);
-	    val = va_arg(ap, int);
-	    hevt->data[i] = val;
-	}
-	va_end(ap);
+    va_start(ap, observers);
+    for (i = 0; descs[i] != '\0'; ++i) {
+	if (i >= 4)
+	  run_error("hevt type %d has too many parameters", type);
+	val = va_arg(ap, int);
+	hevt->data[i] = val;
     }
-#else
-    hevt->data[0] = d1;
-    hevt->data[1] = d2;
-    hevt->data[2] = d3;
-    hevt->data[3] = d4;
-#endif
+    va_end(ap);
+
     /* Check on plausibility of event data. */
     for (i = 0; descs[i] != '\0'; ++i) {
 	val = hevt->data[i];
@@ -159,8 +144,11 @@ int d1, d2, d3, d4;
 	      run_warning("invalid side number %d in hist event", val);
 	    break;
 	  case 'U':
+	    /* Note that when validating unit id, the unit may be in
+	       the middle of the the process of becoming a past unit,
+	       so allowing finding a dead unit. */
 	    if (val != 0
-		&& find_unit(val) == NULL
+		&& find_unit_dead_or_alive(val) == NULL
 		&& find_past_unit(val) == NULL)
 	      run_warning("invalid unit/pastunit id %d in hist event", val);
 	    break;
@@ -178,11 +166,9 @@ int d1, d2, d3, d4;
     if (observers != NOSIDES) {
 	/* Let all the observers' interfaces look at this event. */
 	for_all_sides(side) {
-	    if (side_in_set(side, observers)) {
+	    if (side_in_set(side, observers) && active_display(side)) {
 		update_event_display(side, hevt, TRUE);
-		if (g_event_notices() != lispnil) {
-		    notify_event(side, hevt);
-		}
+		notify_event(side, hevt);
 		if (g_event_movies() != lispnil) {
 		    play_event_movies(side, hevt);
 		}
@@ -196,13 +182,12 @@ int d1, d2, d3, d4;
 }
 
 static void
-notify_event(side, hevt)
-Side *side;
-HistEvent *hevt;
+notify_event(Side *side, HistEvent *hevt)
 {
-    int found = FALSE;
     char abuf[BUFSIZE];
-    Obj *rest, *head, *pattern, *parms, *msgdesc, *text;
+    Obj *rest, *head, *pattern, *text;
+    Side *side2;
+    Unit *unit;
 
     for_all_list(g_event_notices(), rest) {
 	head = car(rest);
@@ -233,12 +218,61 @@ HistEvent *hevt;
 	    }
 	}
     }
+    /* If we didn't find anything special, still put out some generic
+       messages for important cases. */
+    switch (hevt->type) {
+      case H_SIDE_LOST:
+	if (hevt->data[0] == side_number(side)) {
+	    notify(side, "You lost!");
+	} else {
+	    notify(side, "%s lost!", side_desig(side_n(hevt->data[0])));
+	}
+	break;
+      case H_SIDE_WON:
+	if (hevt->data[0] == side_number(side)) {
+	    notify(side, "You won!");
+	} else {
+	    notify(side, "%s won!", side_desig(side_n(hevt->data[0])));
+	}
+	break;
+#if 0 /* this already has a special message */
+      case H_GAME_ENDED:
+	notify(side, "The game is over!");
+	break;
+#endif
+      case H_UNIT_COMPLETED:
+	side2 = side_n(hevt->data[0]);
+	unit = find_unit(hevt->data[1]);
+	if (unit != NULL) {
+	    if (side2 == side) {
+		notify(side, "You completed %s.",
+		       unit_handle(side, unit));
+	    } else {
+		notify(side, "%s completed %s.",
+		       side_desig(side2), unit_handle(side2, unit));
+	    }
+	}
+	break;
+      case H_UNIT_CREATED:
+	side2 = side_n(hevt->data[0]);
+	unit = find_unit(hevt->data[1]);
+	if (unit != NULL) {
+	    if (side2 == side) {
+		notify(side, "You created %s.",
+		       unit_handle(side, unit));
+	    } else {
+		notify(side, "%s created %s.",
+		       side_desig(side2), unit_handle(side2, unit));
+	    }
+	}
+	break;
+      default:
+	break;
+    }
 }
 
 static void
-play_event_movies(side, hevt)
-Side *side;
-HistEvent *hevt;
+play_event_movies(Side *side, HistEvent *hevt)
 {
     int found = FALSE;
     char *soundname;
@@ -300,10 +334,12 @@ HistEvent *hevt;
     }
 }
 
+/* A unit's death requires some bookkeeping - we need to update the
+   history to use a past unit, plus record the loss for statistics
+   purposes. */
+
 void
-record_unit_death(unit, reason)
-Unit *unit;
-HistEventType reason;
+record_unit_death(Unit *unit, HistEventType reason)
 {
     enum loss_reasons lossreason;
     PastUnit *pastunit;
@@ -317,41 +353,47 @@ HistEventType reason;
     if (reason >= 0) {
 	switch (reason) {
 	  case H_UNIT_GARRISONED:
-	    record_event(reason, add_side_to_set(unit->side, NOSIDES), pastunit->id, tmphevtdata1);
+	    /* Garrison events mention the unit being garrisoned. */
+	    record_event(reason, add_side_to_set(unit->side, NOSIDES),
+			 pastunit->id, tmphevtdata1);
 	    break;
 	  default:
-	    record_event(reason, add_side_to_set(unit->side, NOSIDES), pastunit->id);
+	    record_event(reason, add_side_to_set(unit->side, NOSIDES),
+			 pastunit->id, -1);
 	    break;
 	}
     }
-    if (unit->side) {
-	switch (reason) {
-	  case H_UNIT_KILLED:
-	  case H_UNIT_WRECKED:
-	    lossreason = combat_loss;
-	    break;
-	  case H_UNIT_STARVED:
-	    lossreason = starvation_loss;
-	    break;
-	  case H_UNIT_DIED_IN_ACCIDENT:
-	  case H_UNIT_WRECKED_IN_ACCIDENT:
-	  case H_UNIT_VANISHED:
-	    lossreason = accident_loss;
-	    break;
-	  case H_UNIT_DISBANDED:
-	    lossreason = disband_loss;
-	    break;
-	  default:
-	    lossreason = other_loss;
-	    break;
-	}
-	count_loss(unit->side, unit->type, lossreason);
+    /* Reduce specific events to general categories of unit loss, used
+       for statistics display. */
+    /* Note that we track independent unit losses as well. */
+    switch (reason) {
+      case H_UNIT_KILLED:
+      case H_UNIT_WRECKED:
+	lossreason = combat_loss;
+	break;
+      case H_UNIT_STARVED:
+	lossreason = starvation_loss;
+	break;
+      case H_UNIT_DIED_IN_ACCIDENT:
+      case H_UNIT_WRECKED_IN_ACCIDENT:
+      case H_UNIT_VANISHED:
+	lossreason = accident_loss;
+	break;
+      case H_UNIT_DISBANDED:
+	lossreason = disband_loss;
+	break;
+      default:
+	lossreason = other_loss;
+	break;
     }
+    count_loss(unit->side, unit->type, lossreason);
 }
 
+/* Create a past unit corresponding to the given unit, replace unit
+   references in the history. */
+
 PastUnit *
-change_unit_to_past_unit(unit)
-Unit *unit;
+change_unit_to_past_unit(Unit *unit)
 {
     PastUnit *pastunit;
 
@@ -365,10 +407,8 @@ Unit *unit;
 }
 
 void
-record_unit_side_change(unit, newside, reason, agent)
-Unit *unit, *agent;
-Side *newside;
-HistEventType reason;
+record_unit_side_change(Unit *unit, Side *newside, HistEventType reason,
+			Unit *agent)
 {
     int capture;
     enum loss_reasons lossreason;
@@ -401,10 +441,11 @@ HistEventType reason;
     count_gain(newside, unit->type, gainreason);
 }
 
+/* Record a unit's name change by creating a past unit with the
+   previous name. */
+
 void
-record_unit_name_change(unit, newname)
-Unit *unit;
-char *newname;
+record_unit_name_change(Unit *unit, char *newname)
 {
     PastUnit *pastunit;
 
@@ -413,28 +454,24 @@ char *newname;
 }
 
 void
-count_gain(side, u, reason)
-Side *side;
-int u;
-enum gain_reasons reason;
+count_gain(Side *side, int u, enum gain_reasons reason)
 {
     if (side)
       ++(side->gaincounts[num_gain_reasons * u + reason]);
 }
 
 void
-count_loss(side, u, reason)
-Side *side;
-int u;
-enum loss_reasons reason;
+count_loss(Side *side, int u, enum loss_reasons reason)
 {
     if (side)
       ++(side->losscounts[num_loss_reasons * u + reason]);
 }
 
+/* Create a past unit with the given type and for the unit with
+   the given id. */
+
 PastUnit *
-create_past_unit(type, id)
-int type, id;
+create_past_unit(int type, int id)
 {
     PastUnit *pastunit = (PastUnit *) xmalloc(sizeof(PastUnit));
 
@@ -455,8 +492,7 @@ int type, id;
 }
 
 PastUnit *
-find_past_unit(n)
-int n;
+find_past_unit(int n)
 {
     PastUnit *pastunit;
 
@@ -467,9 +503,11 @@ int n;
     return NULL;
 }
 
+/* Scan through the history, changing matching unit/pastunit
+   references into past unit references. */
+
 static void
-rewrite_unit_references(oldid, newid)
-int oldid, newid;
+rewrite_unit_references(int oldid, int newid)
 {
     HistEvent *hevt;
 
@@ -480,16 +518,14 @@ int oldid, newid;
 }
 
 static void
-rewrite_unit_references_in_event(hevt, oldid, newid)
-HistEvent *hevt;
-int oldid, newid;
+rewrite_unit_references_in_event(HistEvent *hevt, int oldid, int newid)
 {
     int i;
     char *descs;
 
     descs = hevtdefns[hevt->type].datadescs;
-    /* Scan through the data description, looking for
-       values that are references to actual units. */
+    /* Scan through the data description, looking for values that are
+       references to actual units. */
     for (i = 0; descs[i] != '\0'; ++i) {
 	if (descs[i] == 'U' && hevt->data[i] == oldid)
 	  hevt->data[i] = newid;
@@ -499,7 +535,7 @@ int oldid, newid;
 /* Indicate that history is no longer being recorded. */
 
 void
-end_history()
+end_history(void)
 {
     record_event(H_LOG_ENDED, ALLSIDES);
     recording_events = FALSE;
@@ -510,10 +546,7 @@ end_history()
    separate lines whenever the date is different. */
 
 HistEvent *
-get_nth_history_line(side, n, nextevt)
-Side *side;
-int n;
-HistEvent **nextevt;
+get_nth_history_line(Side *side, int n, HistEvent **nextevt)
 {
     int i = 0;
     HistEvent *hevt, *hevtprev = NULL;
@@ -548,8 +581,7 @@ HistEvent **nextevt;
 /* Count the total number of lines that can be in the history. */
 
 int
-update_total_hist_lines(side)
-Side *side;
+update_total_hist_lines(Side *side)
 {
     int nlines;
     HistEvent *hevt, *hevtprev = NULL;
@@ -572,10 +604,8 @@ Side *side;
    appear. */
 
 int
-build_hist_contents(side, n, histcontents, numvishistlines)
-Side *side;
-int n, numvishistlines;
-HistEvent **histcontents;
+build_hist_contents(Side *side, int n, HistEvent **histcontents,
+		    int numvishistlines)
 {
     int numcontents;
     HistEvent *hevt, *nexthevt;
@@ -611,7 +641,7 @@ HistEvent **histcontents;
    all to a file. */
 
 void
-dump_statistics()
+dump_statistics(void)
 {
     char *fname;
     Side *side;
@@ -624,7 +654,7 @@ dump_statistics()
        (for instance, the user might have cancelled). */
     if (empty_string(fname))
       return;
-    fp = fopen(fname, "w");
+    fp = open_file(fname, "w");
     if (fp != NULL) {
 	if (1 /* records exist */) {
 	    write_side_results(fp, NULL);
@@ -648,13 +678,13 @@ dump_statistics()
     }
 }
 
-/* Short, unreadable, but greppable listing of past unit.  Primarily useful
-   for debugging and warnings.  We use several buffers and rotate between
-   them so we can call this more than once in a single printf. */
+/* Short, unreadable, but greppable listing of past unit.  Primarily
+   useful for debugging and warnings.  We use several buffers and
+   rotate between them so we can call this more than once in a single
+   printf. */
 
 char *
-past_unit_desig(pastunit)
-PastUnit *pastunit;
+past_unit_desig(PastUnit *pastunit)
 {
     char *shortbuf;
 
@@ -685,9 +715,7 @@ PastUnit *pastunit;
 /* Would be faster to stash these, but enough difference to care? */
 
 int
-total_gain(side, u)
-Side *side;
-int u;
+total_gain(Side *side, int u)
 {
     int i, total = 0;
 
@@ -697,9 +725,7 @@ int u;
 }
 
 int
-total_loss(side, u)
-Side *side;
-int u;
+total_loss(Side *side, int u)
 {
     int i, total = 0;
 
