@@ -886,6 +886,8 @@ set_ai_for_player(int n, char *aitype)
     Player *player;
 
     player = assignments[n].player;
+    if (player == NULL)
+        return;
     if (empty_string(aitype) || find_ai_type(aitype) > 0) {
 	player->aitypename = aitype;
     } else {
