@@ -1,5 +1,5 @@
 /* Scores and scoring in Xconq.
-   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,17 +15,17 @@ any later version.  See the file COPYING.  */
    "the Star of Satyria enters Noblum".  The other is the accumulation
    of a numeric score.  A scorekeeper may include both, so it can (for
    instance) end the game the instant that a player's score reaches
-   100, or 100 more than another player, or whatever.  */
+   100, or 100 more than another player, or whatever.
+
+   The indepside should not participate in scoring, so the iterations
+   in this file generally iterate only over the "real" sides. */
 
 #include "conq.h"
-extern char *escaped_symbol PARAMS ((char *str));
-extern char *escaped_string PARAMS ((char *str));
 #include "kernel.h"
 
-char *basic_player_name PARAMS ((Player *player));
-int read_scorefile PARAMS ((void));
-int interp_score_record PARAMS ((Obj *form));
-char *get_scores PARAMS ((Side *side));
+static int read_scorefile(void);
+static int interp_score_record(Obj *form);
+static char *basic_player_name(Player *player);
 
 typedef struct a_score_record {
     char *gamename;
@@ -47,8 +47,9 @@ typedef struct a_score_record {
 #define for_all_score_records(sr)  \
   for ((sr) = records; (sr) != NULL; (sr) = (sr)->next)
 
-static void eval_sk_last_side_wins PARAMS ((Scorekeeper *sk));
-static void eval_sk_last_alliance_wins PARAMS ((Scorekeeper *sk));
+static void eval_sk_last_side_wins(Scorekeeper *sk);
+static void eval_sk_last_alliance_wins(Scorekeeper *sk);
+static void score_variant_desc(ScoreRecord *sr, char *varbuf);
 
 /* The head of the list of scorekeepers. */
 
@@ -93,12 +94,16 @@ int any_turn_specific_scores;
 
 static int num_sides_originally;
 
+static int only_checking;
+
+static int we_have_a_winner;
+
 ScoreRecord *records;
 
 /* Clear out any possible scorekeepers. */
 
 void
-init_scorekeepers()
+init_scorekeepers(void)
 {
     scorekeepers = last_scorekeeper = NULL;
     numscorekeepers = 0;
@@ -111,7 +116,7 @@ init_scorekeepers()
 }
 
 Scorekeeper *
-create_scorekeeper()
+create_scorekeeper(void)
 {
     Scorekeeper *sk = (Scorekeeper *) xmalloc(sizeof(Scorekeeper));
 
@@ -142,8 +147,7 @@ create_scorekeeper()
 }
 
 Scorekeeper *
-find_scorekeeper(id)
-int id;
+find_scorekeeper(int id)
 {
     Scorekeeper *sk;
 
@@ -158,7 +162,7 @@ int id;
    must happen after all scorekeepers have been defined. */
 
 void
-init_scores()
+init_scores(void)
 {
     int score;
     Side *side;
@@ -232,7 +236,7 @@ init_scores()
    side moves anything. */
 
 void
-check_pre_turn_scores()
+check_pre_turn_scores(void)
 {
     Side *side;
     Scorekeeper *sk;
@@ -256,7 +260,7 @@ check_pre_turn_scores()
 /* Test all the scorekeepers that should be run only at the end of a turn. */
 
 void
-check_post_turn_scores()
+check_post_turn_scores(void)
 {
     Side *side;
     Scorekeeper *sk;
@@ -289,7 +293,7 @@ check_post_turn_scores()
 }
 
 void
-check_post_action_scores()
+check_post_action_scores(void)
 {
     Side *side;
     Scorekeeper *sk;
@@ -314,7 +318,7 @@ check_post_action_scores()
 }
 
 void
-check_post_event_scores()
+check_post_event_scores(void)
 {
     Side *side;
     Scorekeeper *sk;
@@ -342,18 +346,13 @@ check_post_event_scores()
    on the given side.  This can be expensive to run. */
 
 void
-run_scorekeeper(side, sk)
-Side *side;
-Scorekeeper *sk;
+run_scorekeeper(Side *side, Scorekeeper *sk)
 {
     eval_sk_form(side, sk, sk->body);
 }
 
 int
-eval_sk_form(side, sk, form)
-Side *side;
-Scorekeeper *sk;
-Obj *form;
+eval_sk_form(Side *side, Scorekeeper *sk, Obj *form)
 {
     int val;
     char *formtype;
@@ -396,15 +395,16 @@ Obj *form;
 	  }
 	  break;
 	case K_WIN:
-	  if (side->ingame)
+	  if (side->ingame && !only_checking)
 	    side_wins(side, sk->id);
 	  break;
 	case K_LOSE:
-	  if (side->ingame)
+	  if (side->ingame && !only_checking)
 	    side_loses(side, NULL, sk->id);
 	  break;
 	case K_END:
-	  all_sides_draw();
+	  if (!only_checking)
+	    all_sides_draw();
 	  break;
 	case K_SET:
 	  if (sk->scorenum >= 0) {
@@ -447,9 +447,7 @@ Obj *form;
 }
 
 int
-sum_property(side, form)
-Side *side;
-Obj *form;
+sum_property(Side *side, Obj *form)
 {
     int sum = 0, u, typevec[MAXUTYPES];
     Obj *sidelist = lispnil, *typelist = lispnil, *prop = lispnil;
@@ -493,8 +491,7 @@ Obj *form;
 /* Return the point value of a particular unit. */
 
 int
-point_value(unit)
-Unit *unit;
+point_value(Unit *unit)
 {
     /* Incomplete units are always worthless. */
     if (unit->cp > 0 && !completed(unit))
@@ -507,8 +504,7 @@ Unit *unit;
 /* Return the sum of a side's units' point values. */
 
 int
-side_point_value(side)
-Side *side;
+side_point_value(Side *side)
 {
     Unit *unit;
 
@@ -530,10 +526,7 @@ Side *side;
 /* Evaluate a given form with respect to the given scorekeeper and side. */
 
 int
-eval_sk_test(side, sk, form)
-Side *side;
-Scorekeeper *sk;
-Obj *form;
+eval_sk_test(Side *side, Scorekeeper *sk, Obj *form)
 {
     char *formtype;
     Obj *arg1 = lispnil, *arg2 = lispnil, *rest;
@@ -588,8 +581,7 @@ Obj *form;
 }
 
 static void
-eval_sk_last_side_wins(sk)
-Scorekeeper *sk;
+eval_sk_last_side_wins(Scorekeeper *sk)
 {
     Side *side2, *winner = NULL;
     int numleft = 0, points;
@@ -602,7 +594,16 @@ Scorekeeper *sk;
 		Dprintf("%s has %d points worth of units\n",
 			side_desig(side2), points);
 		if (points == 0) {
+		  if (!only_checking)
 		    side_loses(side2, NULL, sk->id);
+		} else if (side2 == indepside
+			   && !side_has_ai(side2)
+			   && !side_has_display(side2)) {
+		    /* The independent side should not be counted
+		       among the sides left (unless it has somebody
+		       running it), otherwise the but-I-already-won!
+		       player has to scour the map looking for
+		       remaining inert independents. */
 		} else {
 		    ++numleft;
 		    winner = side2;
@@ -610,14 +611,15 @@ Scorekeeper *sk;
 	    }
 	}
 	if (numleft == 1) {
+	  we_have_a_winner = TRUE;
+	  if (!only_checking)
 	    side_wins(winner, sk->id);
 	}
     }
 }
 
 static void
-eval_sk_last_alliance_wins(sk)
-Scorekeeper *sk;
+eval_sk_last_alliance_wins(Scorekeeper *sk)
 {
     Side *side2, *side3, *winner = NULL;
     int numleft = 0, sidepoints[MAXSIDES+1], alliancepoints[MAXSIDES+1];
@@ -648,7 +650,12 @@ Scorekeeper *sk;
 	for_all_sides(side2) {
 	    if (side2->ingame) {
 		if (alliancepoints[side2->id] == 0) {
-		    side_loses(side2, NULL, sk->id);
+		    if (!only_checking)
+		      side_loses(side2, NULL, sk->id);
+		} else if (side2 == indepside
+			   && !side_has_ai(side2)
+			   && !side_has_display(side2)) {
+		    /* See comment in eval_sk_last_side_wins. */
 		} else {
 		    ++numleft;
 		    winner = side2;
@@ -667,10 +674,12 @@ Scorekeeper *sk;
 	    }
 	    /* If so, everybody in the alliance wins equally. */
 	    if (alliancewins) {
+		we_have_a_winner = TRUE;
 		for_all_sides(side2) {
 		    if (side2->ingame
 			&& (side2 == winner || trusted_side(winner, side2))) {
-			side_wins(side2, sk->id);
+			if (!only_checking)
+			  side_wins(side2, sk->id);
 		    }
 		}
 	    }
@@ -681,9 +690,7 @@ Scorekeeper *sk;
 /* Implement the effects of a side winning. */
 
 void
-side_wins(side, why)
-Side *side;
-int why;
+side_wins(Side *side, int why)
 {
     /* Nothing happens to the side's units or people. */
     side->status = 1;
@@ -696,13 +703,15 @@ int why;
 /* Implement the effects of a side losing. */
 
 void
-side_loses(side, side2, why)
-Side *side, *side2;
-int why;
+side_loses(Side *side, Side *side2, int why)
 {
     int x, y, s, s2, ux, uy, changed;
     Unit *unit;
 
+    /* The independents cannot lose by definition even if they
+    are down to zero points (unless played by a human). */
+    if (side == indepside && !side_has_display(side))
+      return;
     /* These should not happen, but a stupid AI might try, so just
        and clear the mistaken side. */
     if (side == side2) {
@@ -713,6 +722,10 @@ int why;
 	run_warning("losing to side not in game, ignoring side");
 	side2 = NULL;
     }
+    /* If there's a controlling side, it gets everything. */
+    if (side->controlled_by != NULL && side->controlled_by->ingame) {
+	side2 = side->controlled_by;
+    }
     if (side2 != NULL) {
 	/* Dispose of all of a side's units. */
 	for_all_units(unit) {
@@ -747,25 +760,58 @@ int why;
 	    }
 	}
 	/* Reset view coverage everywhere. */
-	if (!all_see_all) {
+	if (!side->see_all) {
 	    for_all_cells(x, y) {
 		set_cover(side, x, y, 0);
 	    }
 	}
     }
+    
+    /* Advanced units (cities) should not disappear - make them independent. */
+    else {
+	for_all_units(unit) {
+	    if (unit->side == side && u_advanced(unit->type)) {
+		if (in_play(unit)) {
+		    ux = unit->x;  uy = unit->y;
+		    change_unit_side(unit, indepside, H_SIDE_LOST, NULL);
+		    /* Everybody gets to see this change. */
+		    all_see_cell(ux, uy);
+		} else {
+		    /* Even out-of-play units need to have their side set. */
+		    set_unit_side(unit, indepside);
+		}
+	    }
+	}
+    }
+    
     /* Add the mark of shame itself. */
     side->status = -1;
     remove_side_from_game(side);
     /* Record the event after the side is removed, so we don't get infinite
        recursion if there is an event-triggered scorekeeper. */
     record_event(H_SIDE_LOST, ALLSIDES, side_number(side), why);
+    /* As a special case, look at post-turn scorekeepers to see if
+       sides should end their turns early because the game is over. */
+    only_checking = TRUE;
+    check_post_turn_scores();
+    only_checking = FALSE;
+    if (we_have_a_winner) {
+	for_all_sides(side2) {
+	    if (side2->ingame) {
+		finish_turn(side2);
+	    }
+	}
+    }
+    /* When the remaining sides' turns are finished, the end-of-turn
+       processing will commence, and it includes the real run of
+       post-turn scorekeeper checking. */
 }
 
 /* Implement a draw.  Note that unlike winning or losing, the draw
    applies to all sides currently in the game. */
 
 void
-all_sides_draw()
+all_sides_draw(void)
 {
     SideMask drew;
     Side *side;
@@ -775,11 +821,11 @@ all_sides_draw()
     	if (side->ingame) {
 	    side->status = 0;
 	    remove_side_from_game(side);
-	    add_side_to_set(side, drew);
+	    drew = add_side_to_set(side, drew);
 	}
     }
     /* Now that all sides are out, safe to record events to that effect
-       (no possibility of triggering post-even scorekeepers). */
+       (no possibility of triggering post-event scorekeepers). */
     for_all_sides(side) {
     	if (side_in_set(side, drew)) {
 	    record_event(H_SIDE_WITHDREW, ALLSIDES, side_number(side));
@@ -787,11 +833,9 @@ all_sides_draw()
     }
 }
 
-/* Record the outcome of the game into the scorefile. */
-
-char *
-basic_player_name(player)
-Player *player;
+/* (should move this to nlang.c?) */
+static char *
+basic_player_name(Player *player)
 {
     char *playername;
 
@@ -807,8 +851,10 @@ Player *player;
     return playername;
 }
 
+/* Record the outcome of the game into the scorefile. */
+
 void
-record_into_scorefile()
+record_into_scorefile(void)
 {
     int i;
     int any_advantage_variation = FALSE, adv, adv2, advantage;
@@ -834,7 +880,11 @@ record_into_scorefile()
     if (!empty_string(g_scorefile_name()))
       filename = g_scorefile_name();
     fp = open_scorefile_for_writing(filename);
-    if (fp != NULL) {
+    if (fp == NULL) {
+	run_warning("%s cannot be opened for writing, will not record score",
+		    filename);
+	/* (should provide some sort of retry here) */
+    } else {
 	fprintf(fp, "(g %s",
 		/* (should record this for comparison when displaying
                    scores) */
@@ -857,22 +907,39 @@ record_into_scorefile()
 		var = &(variants[i]);
 		varname = c_string(var->id);
 		fprintf(fp, " (");
-		/* Encode the common variants by number, for compactness. */
-		if (keyword_code(varname) == K_WORLD_SEEN) {
+		/* Encode the common variants by number, for
+		   compactness.  Note that once assigned, these
+		   numbers can never change. */
+		switch (keyword_code(varname)) {
+		  case K_WORLD_SEEN:
 		    fprintf(fp, "1 %d", var->intvalue);
-		} else if (keyword_code(varname) == K_SEE_ALL) {
+		    break;
+		  case K_SEE_ALL:
 		    fprintf(fp, "2 %d", var->intvalue);
-		} else if (keyword_code(varname) == K_SEQUENTIAL) {
+		    break;
+		  case K_SEQUENTIAL:
 		    fprintf(fp, "3 %d", var->intvalue);
-		} else if (keyword_code(varname) == K_WORLD_SIZE) {
-		    fprintf(fp, "4 %d %d %d",
+		    break;
+		  case K_PEOPLE:
+		    fprintf(fp, "4 %d", var->intvalue);
+		    break;
+		  case K_ECONOMY:
+		    fprintf(fp, "5 %d", var->intvalue);
+		    break;
+		  case K_SUPPLY:
+		    fprintf(fp, "6 %d", var->intvalue);
+		    break;
+		  case K_WORLD_SIZE:
+		    fprintf(fp, "11 %d %d %d",
 			    area.width, area.height, world.circumference);
-		} else if (keyword_code(varname) == K_REAL_TIME) {
-		    fprintf(fp, "5 %d %d %d",
+		    break;
+		  case K_REAL_TIME:
+		    fprintf(fp, "12 %d %d %d",
 			    g_rt_for_game(),
 			    g_rt_per_side(),
 			    g_rt_per_turn());
-		} else {
+		    break;
+		  default:
 		    fprintf(fp, "%s", escaped_symbol(varname));
 		    if (var->hasintvalue) {
 			fprintf(fp, " %d", var->intvalue);
@@ -892,8 +959,11 @@ record_into_scorefile()
 	for_all_sides(side) {
 	    /* Only record info about sides that actually participated. */
 	    if (side->everingame) {
-		fprintf(fp, " (");
 		playername = basic_player_name(side->player);
+		/* If a side has no player, don't record it. */
+		if (empty_string(playername))
+		  continue;
+		fprintf(fp, " (");
 		fprintf(fp, "%s", escaped_symbol(playername));
 		fprintf(fp, " %s",
 			(side_won(side)
@@ -901,7 +971,7 @@ record_into_scorefile()
 			 : (side_lost(side)
 			    ? "lost"
 			    : "drew")));
-		/* (should write info about ai helpers) */
+		/* (should write info about ai helping human players) */
 		if (any_advantage_variation) {
 		    /* (should have cached as "actual advantage"?) */
 		    advantage = max(1, side->advantage);
@@ -928,8 +998,8 @@ record_into_scorefile()
     }
 }
 
-int
-read_scorefile()
+static int
+read_scorefile(void)
 {
     int startlineno = 1, endlineno = 1;
     int numrecs;
@@ -943,6 +1013,10 @@ read_scorefile()
       filename = g_scorefile_name();
     fp = open_scorefile_for_reading(filename);
     if (fp != NULL) {
+        /* Note that we clear all existing score records.  This is because
+	   the score file may be shared and thus getting multiple updates. */
+	/* (should free any existing records) */
+	records = NULL;
 	numrecs = 0;
 	/* Read everything in the file. */
 	while ((form = read_form(fp, &startlineno, &endlineno)) != lispeof) {
@@ -960,9 +1034,8 @@ read_scorefile()
     return FALSE;
 }
 
-int
-interp_score_record(form)
-Obj *form;
+static int
+interp_score_record(Obj *form)
 {
     char *propname;
     Obj *props, *prop;
@@ -1006,17 +1079,17 @@ Obj *form;
     return FALSE;
 }
 
-static void score_variant_desc PARAMS ((ScoreRecord *sr, char *varbuf));
+/* Collect scorefile contents and format them.  This routine starts from
+   a fresh set of records each time, and allocates a new formatting
+   buffer each time, so should be called sparingly. */
 
 char *
-get_scores(side)
-Side *side;
+get_scores(Side *side)
 {
-    int wins, losses, draws, plays, allplays, i;
-    char *buf, buf2[BUFSIZE], sdadvbuf[20], varbuf[BUFSIZE];
-    char *thisgame, *playername, *sdnamestr, *sdfatestr;
-    Obj *sds, *sd, *sdname, *sdfate, *more, *more1, *restvarset, *varset;
-    Variant *variants, *tmpvar, *var;
+    int wins, losses, draws, plays, allplays;
+    char *buf, sdadvbuf[20], varbuf[BUFSIZE];
+    char *thisgame, *thisgametitle, *playername, *sdnamestr, *sdfatestr;
+    Obj *sds, *sd, *sdname, *sdfate, *more, *more1;
     ScoreRecord *sr;
 
     thisgame = (mainmodule->origmodulename
@@ -1028,7 +1101,11 @@ Side *side;
     if (records == NULL)
       return "No scores available.\n";
     buf = xmalloc(5000);
-    sprintf(buf, "Scores for \"%s\":\n", thisgame);
+    thisgametitle = thisgame;
+    if (!empty_string(mainmodule->title))
+      thisgametitle = mainmodule->title;
+    sprintf(buf, "Scores for %s", thisgametitle);
+    strcat(buf, ":\n");
     playername = NULL;
     if (side != NULL) {
 	playername = basic_player_name(side->player);
@@ -1141,21 +1218,21 @@ Side *side;
     return buf;
 }
 
-/* Given a score record, report any variants that were specified and different from
-   the current module's defaults. */
+/* Given a score record, report any variants that were specified and
+   different from the current module's defaults. */
 
 static void
-score_variant_desc(sr, varbuf)
-ScoreRecord *sr;
-char *varbuf;
+score_variant_desc(ScoreRecord *sr, char *varbuf)
 {
-    int i, hasdflt, dfltval, num, val;
+    int i, hasdflt, dfltval, val;
     Obj *restvarset, *varset, *varsetname, *rslt;
     Variant *variants, *tmpvar, *var;
 
     varbuf[0] = '\0';
     if (sr->varsets != lispnil) {
-	variants = (mainmodule->origvariants ? mainmodule->origvariants : mainmodule->variants);
+	variants = (mainmodule->origvariants
+		    ? mainmodule->origvariants
+		    : mainmodule->variants);
 	for_all_list(sr->varsets, restvarset) {
 	    varset = car(restvarset);
 	    if (consp(varset)) {
@@ -1173,9 +1250,18 @@ char *varbuf;
 			varsetname = intern_symbol(keyword_name(K_SEQUENTIAL));
 			break;
 		      case 4:
-			varsetname = intern_symbol(keyword_name(K_WORLD_SIZE));
+			varsetname = intern_symbol(keyword_name(K_PEOPLE));
 			break;
 		      case 5:
+			varsetname = intern_symbol(keyword_name(K_ECONOMY));
+			break;
+		      case 6:
+			varsetname = intern_symbol(keyword_name(K_SUPPLY));
+			break;
+		      case 11:
+			varsetname = intern_symbol(keyword_name(K_WORLD_SIZE));
+			break;
+		      case 12:
 			varsetname = intern_symbol(keyword_name(K_REAL_TIME));
 			break;
 		      default:
@@ -1204,11 +1290,14 @@ char *varbuf;
 		    hasdflt = TRUE;
 		}
 		val = 54321;
+		/* The numeric value is the second element in the variant
+		   setting. */
 		if (numberp(cadr(varset)) && cddr(varset) == lispnil)
 		  val = c_number(cadr(varset));
+		/* Handle variants with several values. */
 		if (keyword_code(c_string(varsetname)) == K_WORLD_SIZE) {
 		    if (!hasdflt || !equal(rslt, cdr(varset)))
-		      tprintf(varbuf, " %s=%dx%dw%d", c_string(varsetname),
+		      tprintf(varbuf, " %s=%dx%dW%d", c_string(varsetname),
 			      c_number(cadr(varset)), c_number(caddr(varset)),
 			      c_number(cadr(cddr(varset))));
 		} else if (keyword_code(c_string(varsetname)) == K_REAL_TIME) {
@@ -1229,8 +1318,7 @@ char *varbuf;
    to win somehow, or if it can just goof off. */
 
 int
-should_try_to_win(side)
-Side *side;
+should_try_to_win(Side *side)
 {
     Scorekeeper *sk;
 
