@@ -1,5 +1,5 @@
 /* Units in Xconq.
-   Copyright (C) 1986-1989, 1991-1997 Stanley T. Shebs.
+   Copyright (C) 1986-1989, 1991-2000 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,11 +7,10 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern void interp_utype_list PARAMS ((short *arr, Obj *lis));
+extern int type_can_sit_on_conn(int u, int x, int y);
 #include "kernel.h"
-extern int num_sides_allowed PARAMS ((int u));
-extern void wreck_unit PARAMS ((Unit *unit));
-extern int side_owns_occupant PARAMS ((Side *side, Unit *unit));
+extern int num_sides_allowed(int u);
+extern int side_owns_occupant(Side *side, Unit *unit);
 
 /* This is not a limit, just sets initial allocation of unit objects
    and how many additional to get if more are needed.  This can be
@@ -22,13 +21,13 @@ extern int side_owns_occupant PARAMS ((Side *side, Unit *unit));
 #define INITMAXUNITS 200
 #endif
 
-static void allocate_unit_block PARAMS ((void));
-static int compare_units PARAMS ((Unit *unit1, Unit *unit2));
-static int compare_units_by_keys PARAMS ((const void *e1, const void *e2));
+static void allocate_unit_block(void);
+static int compare_units(Unit *unit1, Unit *unit2);
+static int compare_units_by_keys(const void *e1, const void *e2);
 
-static void leave_cell_aux PARAMS ((Unit *unit, Unit *oldtransport));
+static void leave_cell_aux(Unit *unit, Unit *oldtransport);
 
-static void change_cell_aux PARAMS ((Unit *unit, int x0, int y0, int x, int y));
+static void change_cell_aux(Unit *unit, int x0, int y0, int x, int y);
 
 /* The list of available units. */
 
@@ -58,6 +57,8 @@ static char *utypenamen;
 
 char **shortestnames = NULL;
 
+int longest_shortest;
+
 char **shortestgnames = NULL;
 
 char *actorstatebuf = NULL;
@@ -76,6 +77,11 @@ int *numlivebytype;
 
 /* The next number to use for a unit id. */
 
+/* Starting the unit->id list at 1 will cause crashes in
+   side_number(unit->side) when it is fed the ghost unit with id = 0,
+   which seems to exist even though it does not have a valid side. */
+/* However, this should be found and fixed - id == 0 means "invalid unit". */
+
 int nextid = 1;
 
 /* A cache of completenesses. */
@@ -90,6 +96,16 @@ int recent_dead_flushed = TRUE;
 
 static Side *tmpside_classtest;
 
+/* Advanced unit support. */
+
+short default_size = 1;		/* Default abstract size of unit */
+short default_usedcells = 0;	/* Number of cells being used by the unit */
+short default_maxcells = 1;	/* Default max number of cells that unit can use */
+short default_curadvance = -1;	/* Default advance being researched by new unit */
+long default_population = 1;	/* Default size of unit's population */
+
+extern void free_used_cells(Unit *unit);
+
 /* Grab a block of unit objects to work with. */
 
 static void
@@ -181,8 +197,8 @@ int type, makebrains;
     /* At ground level. */
     newunit->z = 0;
     /* Units default to being independent. */
-    newunit->side = NULL;
-    newunit->origside = NULL;
+    newunit->side = indepside;
+    newunit->origside = indepside;
     /* Create at max hp, let others reduce if necessary. */
     newunit->hp = newunit->hp2 = u_hp(type);
     /* Create fully functional, let other routines set incompleteness. */
@@ -196,6 +212,14 @@ int type, makebrains;
     /* Always zero out all the supply values. */
     for_all_material_types(m)
       newunit->supply[m] = 0;
+
+    /* Allocate cache for units last production. */
+    if (nummtypes > 0)
+        newunit->production = (short *) xmalloc(nummtypes * sizeof(short));
+    /* Always zero out all the production values. */
+    for_all_material_types(m)
+       newunit->production[m] = 0;
+
     /* Will allocate tooling state when actually needed. */
     newunit->tooling = NULL;
     /* Will allocate opinions when actually needed. */
@@ -221,6 +245,19 @@ int type, makebrains;
     newunit->prevx = newunit->prevy = -1;
     newunit->transport_id = lispnil;
     newunit->aihook = NULL;
+
+    /* Advanced unit support. */
+    newunit->size = default_size;
+    newunit->reach = u_reach(type);
+    newunit->usedcells = default_usedcells;
+    newunit->maxcells = default_maxcells;
+    newunit->population = default_population;
+    newunit->curadvance = default_curadvance;
+    newunit->autobuild = FALSE;		
+    newunit->autoresearch = FALSE;
+    newunit->autoplan = FALSE;
+    newunit->cp_stash = 0;
+
     /* Add to the global unit counts. */
     ++numunits;
     ++numliveunits;
@@ -283,7 +320,7 @@ int flagacp;
 	  memset((char *) unit->act, 0, sizeof(ActorState));
 	/* Indicate that the action points have not been set. */
 	if (flagacp)
-	  unit->act->acp = u_acp_min(unit->type) - 1;
+	  unit->act->acp = unit->act->initacp = u_acp_min(unit->type) - 1;
 	/* Flag the action as undefined. */
 	unit->act->nextaction.type = ACTION_NONE;
     } else {
@@ -294,8 +331,7 @@ int flagacp;
 }
 
 void
-init_unit_extras(unit)
-Unit *unit;
+init_unit_extras(Unit *unit)
 {
     if (unit->extras == NULL) {
 	unit->extras = (UnitExtras *) xmalloc(sizeof(UnitExtras));
@@ -315,9 +351,7 @@ Unit *unit;
 /* Changing a unit's type has many effects. */
 
 void
-change_unit_type(unit, newtype, reason)
-Unit *unit;
-int newtype, reason;
+change_unit_type(Unit *unit, int newtype, int reason)
 {
     int oldtype = unit->type, oldhp = unit->hp;
     PastUnit *pastunit;
@@ -348,20 +382,41 @@ int newtype, reason;
     /* We might have to change sides as a result. */
     /* (should modify per-side counts) */
     if (!type_allowed_on_side(newtype, unit->side)) {
-    	if (type_allowed_on_side(newtype, NULL)) {
+    	if (type_allowed_on_side(newtype, indepside)) {
 	    /* Unit becomes independent. */
-	    change_unit_side(unit, NULL, reason, NULL);
+	    change_unit_side(unit, indepside, reason, NULL);
 	} else {
+	    /* (what to do? search for an allowed side?) */
 	    run_warning("Leaving unit on disallowed side");
 	}
     }
     /* Unit will always need a new number. */
     assign_unit_number(unit);
     init_unit_opinions(unit, numsides);
+    /* Redo the supply numbers. */
+    /* (should have separate control, so don't get cheated of old supply) */
+    init_supply(unit);
     /* This clears the unit's acp - desirable? */
     init_unit_actorstate(unit, FALSE);
     init_unit_plan(unit);
     unit->aihook = NULL;
+
+    /* Advanced unit support, but only if we're becoming an advanced type
+       and we weren't before. */
+    /* (should share with unit creation) */
+    if (u_advanced(newtype) && !u_advanced(oldtype)) {
+	unit->size = default_size;
+	unit->reach = u_reach(unit->type);
+	unit->usedcells = default_usedcells;
+	unit->maxcells = default_maxcells;
+	unit->population = default_population;
+	unit->curadvance = default_curadvance;
+	unit->autobuild = FALSE;		
+	unit->autoresearch = FALSE;
+	unit->autoplan = FALSE;
+	unit->cp_stash = 0;
+    }
+
     /* Increment viewing coverage. */
     cover_area(unit->side, unit, unit->transport, -1, -1, unit->x, unit->y);
     /* If vision range is 0, allow glimpses of adjacent cell terrain.
@@ -377,8 +432,7 @@ int newtype, reason;
 }
 
 int
-max_builds(u)
-int u;
+max_builds(int u)
 {
     int u2;
 
@@ -390,9 +444,7 @@ int u;
 }
 
 int
-side_owns_occupant(side, unit)
-Side *side;
-Unit *unit;
+side_owns_occupant(Side *side, Unit *unit)
 {
     int suboccs = FALSE;
     Unit *occ;
@@ -421,9 +473,7 @@ Unit *unit;
    used. */
 
 int
-enter_cell(unit, x, y)
-Unit *unit;
-int x, y;
+enter_cell(Unit *unit, int x, int y)
 {
 #ifdef DEBUGGING
     /* Not necessarily an error, but indicative of bugs elsewhere. */
@@ -451,6 +501,7 @@ int x, y;
     enter_cell_aux(unit, x, y);
     /* Inevitable side-effect of appearing in the new location. */
     all_see_occupy(unit, x, y, TRUE);
+    kick_out_enemy_users(unit->side, x, y);
     return TRUE;
 }
 
@@ -459,9 +510,7 @@ int x, y;
 /* (should eventually account for variable-size units) */
 
 int
-can_occupy_cell(unit, x, y)
-Unit *unit;
-int x, y;
+can_occupy_cell(Unit *unit, int x, int y)
 {
     int u = unit->type, u2, u3, t = terrain_at(x, y), numthistype = 0;
     int fullness = 0, tcap, utcap, numtypes[MAXUTYPES];
@@ -493,8 +542,7 @@ int x, y;
 }
 
 int
-type_can_occupy_cell(u, x, y)
-int u, x, y;
+type_can_occupy_cell(int u, int x, int y)
 {
     int t = terrain_at(x, y), u2, u3, numthistype = 0, fullness = 0;
     int tcap, utcap, numtypes[MAXUTYPES];
@@ -524,11 +572,10 @@ int u, x, y;
 }
 
 /* Similar, but don't count a specific given unit when calculating. */
+/* (should share with can_occupy_cell, make unit3 be optional arg?) */
 
 int
-can_occupy_cell_without(unit, x, y, unit3)
-Unit *unit, *unit3;
-int x, y;
+can_occupy_cell_without(Unit *unit, int x, int y, Unit *unit3)
 {
     int u = unit->type, u2, u3, t = terrain_at(x, y), numthistype = 0;
     int fullness = 0, tcap, utcap, numtypes[MAXUTYPES];
@@ -562,9 +609,7 @@ int x, y;
 }
 
 int
-type_can_occupy_cell_without(u, x, y, unit3)
-int u, x, y;
-Unit *unit3;
+type_can_occupy_cell_without(int u, int x, int y, Unit *unit3)
 {
     int t = terrain_at(x, y), u2, u3, numthistype = 0, fullness = 0;
     int tcap, utcap, numtypes[MAXUTYPES];
@@ -597,12 +642,10 @@ Unit *unit3;
 
 /* Test whether the given position has a connection that the unit may use. */
 
-int can_occupy_conn_1 PARAMS ((Unit *unit, int nx, int ny, int c));
+int can_occupy_conn_1(Unit *unit, int nx, int ny, int c);
 
 int
-can_occupy_conn(unit, nx, ny, nz)
-Unit *unit;
-int nx, ny, nz;
+can_occupy_conn(Unit *unit, int nx, int ny, int nz)
 {
     int c;
 
@@ -616,9 +659,8 @@ int nx, ny, nz;
 	return can_occupy_conn_1(unit, nx, ny, c);
     } else {
 	/* Test each connection type to see if it will work. */
-	for_all_terrain_types(c) {
-	    if (t_is_connection(c)
-		&& aux_terrain_defined(c)
+	for_all_connection_types(c) {
+	    if (aux_terrain_defined(c)
 		&& can_occupy_conn_1(unit, nx, ny, c))
 	      return TRUE;
 	}
@@ -627,9 +669,7 @@ int nx, ny, nz;
 }
 
 int
-can_occupy_conn_1(unit, nx, ny, c)
-Unit *unit;
-int nx, ny, c;
+can_occupy_conn_1(Unit *unit, int nx, int ny, int c)
 {
     int u, u2, tcap, utcap, fullness, numthistype, numtypes[MAXUTYPES];
     Unit *unit2;
@@ -664,13 +704,30 @@ int nx, ny, c;
     return (fullness + ut_size(u, c) <= tcap);
 }
 
+/* Return TRUE if there is any connection terrain at the given
+   location that the given type of unit may sit on, even if
+   the cell terrain is hostile. */
+   
+int
+type_can_sit_on_conn(int u, int x, int y)
+{
+    int c;
+
+    for_all_connection_types(c) {
+	if (aux_terrain_defined(c)
+	    && any_connections_at(x, y, c)
+	    && !ut_vanishes_on(u, c)
+	    && !ut_wrecks_on(u, c))
+	  return TRUE;
+    }
+    return FALSE;
+}
+
 /* Recursive helper to update everybody's position.  This should be one of
    two routines that modify actual unit positions (leaving is the other). */
 
 void
-enter_cell_aux(unit, x, y)
-Unit *unit;
-int x, y;
+enter_cell_aux(Unit *unit, int x, int y)
 {
     int u = unit->type;
     Unit *occ;
@@ -715,6 +772,8 @@ Unit *transport, *unit;
 {
     int u = unit->type, u2 = transport->type, u3, o;
     int numthistype = 0, numalltypes = 0, occvolume = 0;
+    int numfacilities = 0;
+    int nummobiles = 0;
     int ucap, uucap;
     int numtypes[MAXUTYPES];
     Unit *occ;
@@ -741,6 +800,10 @@ Unit *transport, *unit;
 	++numtypes[occ->type];
 	if (o == u)
 	  ++numthistype;
+	if (u_facility(o))
+	  ++numfacilities;
+	if (mobile(o))
+	  ++nummobiles;
 	/* Only count against fullness if exclusive capacity exceeded. */
 	if (numtypes[o] > uu_capacity_x(u2, o)) {
 	    occvolume += uu_size(o, u2);
@@ -753,6 +816,16 @@ Unit *transport, *unit;
     if (uu_occ_max(u2, u) >= 0
         && numthistype + 1 - uucap > uu_occ_max(u2, u))
       return FALSE;
+	/* Check upper limit on number of facilities. */
+	if (u_facility_total_max(u2) >= 0
+	    && u_facility(u)
+	    && numfacilities + 1 > u_facility_total_max(u2))
+		return FALSE;
+	/* Check upper limit on number of mobiles. */
+	if (u_mobile_total_max(u2) >= 0
+	    && mobile(u)
+	    && nummobiles + 1 > u_mobile_total_max(u2))
+		return FALSE;
     /* Check upper limit on count of occupants of all types. */
     if (u_occ_total_max(u2) >= 0
         && numalltypes + 1 > u_occ_total_max(u2))
@@ -770,6 +843,8 @@ Unit *transport;
 {
     int u2 = transport->type, u3, o;
     int numthistype = 0, numalltypes = 0, occvolume = 0;
+    int numfacilities = 0;
+    int nummobiles = 0;
     int ucap, uucap;
     int numtypes[MAXUTYPES];
     Unit *occ;
@@ -791,6 +866,10 @@ Unit *transport;
 	++numtypes[o];
 	if (o == u)
 	  ++numthistype;
+	if (u_facility(o))
+	  ++numfacilities;
+	if (mobile(o))
+	  ++nummobiles;
 	/* Only count against fullness if exclusive capacity exceeded. */
 	if (numtypes[o] > uu_capacity_x(u2, o)) {
 	    occvolume += uu_size(o, u2);
@@ -803,6 +882,16 @@ Unit *transport;
     if (uu_occ_max(u2, u) >= 0
         && numthistype + 1 - uucap > uu_occ_max(u2, u))
       return FALSE;
+	/* Check upper limit on number of facilities. */
+	if (u_facility_total_max(u2) >= 0
+	    && u_facility(u)
+	    && numfacilities + 1 > u_facility_total_max(u2))
+		return FALSE;
+	/* Check upper limit on number of mobiles. */
+	if (u_mobile_total_max(u2) >= 0
+	    && mobile(u)
+	    && nummobiles + 1 > u_mobile_total_max(u2))
+		return FALSE;
     /* Check upper limit on count of occupants of all types. */
     if (u_occ_total_max(u2) >= 0
         && numalltypes + 1 > u_occ_total_max(u2))
@@ -842,6 +931,14 @@ Unit *unit, *transport;
     int u = unit->type, ustack, u2stack;
     Unit *unit2, *topunit, *prevunit = NULL, *nextunit = NULL;
 
+    /* Don't enter transport if there is no room. */
+    if (!can_carry(transport, unit)) {
+	/* Disband the unit if it is incomplete. */
+    	if (!completed(unit))
+	    	notify(unit->side, "%s is full. New unit disbanded on creation.", transport->name);			
+    		kill_unit(unit, H_UNIT_DISBANDED);
+	return;
+     }	
     if (unit == transport) {
     	run_error("Unit is trying to enter itself");
     }
@@ -996,9 +1093,7 @@ int x, y;
 }
 
 void
-set_unit_position(unit, x, y, z)
-Unit *unit;
-int x, y;
+set_unit_position(Unit *unit, int x, int y, int z)
 {
     int u, t, tmpz;
 
@@ -1006,7 +1101,7 @@ int x, y;
     unit->x = x;  unit->y = y;  unit->z = z;
     /* Constrain the altitude according to terrain if nonzero. */
     if (unit->z != 0) {
-	if (unit->z & 1 == 0) {
+	if ((unit->z & 1) == 0) {
 	    u = unit->type;
 	    t = terrain_at(x, y);
 	    tmpz = unit->z / 2;
@@ -1020,9 +1115,7 @@ int x, y;
 }
 
 static void
-change_cell_aux(unit, x0, y0, x, y)
-Unit *unit;
-int x0, y0, x, y;
+change_cell_aux(Unit *unit, int x0, int y0, int x, int y)
 {
     int u = unit->type;
     Unit *occ;
@@ -1103,26 +1196,29 @@ Unit *unit;
     unit->nexthere = NULL;
 }
 
+/* Add only the terrain types of adjacent cells to the unit's side's
+   view, don't show units or anything else. */
+
 void
-glimpse_adjacent_terrain(unit)
-Unit *unit;
+glimpse_adjacent_terrain(Unit *unit)
 {
     int x = unit->x, y = unit->y, dir, x1, y1;
     Side *side = unit->side;
 
     if (u_vision_range(unit->type) == 0
 	&& unit->transport == NULL
-	&& !all_see_all
+	&& !g_see_all()
 	&& !g_terrain_seen()
 	&& side != NULL) {
 	for_all_directions(dir) {
-		if (point_in_dir(x, y, dir, &x1, &y1)) {
-		    if (terrain_view(side, x1, y1) == UNSEEN) {
-			set_terrain_view(side, x1, y1,
-					 buildtview(terrain_at(x1, y1)));
-			update_cell_display(side, x1, y1, TRUE);
-		    }
+	    if (point_in_dir(x, y, dir, &x1, &y1)) {
+		if (terrain_view(side, x1, y1) == UNSEEN) {
+		    set_terrain_view(side, x1, y1,
+				     buildtview(terrain_at(x1, y1)));
+		    update_cell_display(side, x1, y1,
+					UPDATE_ALWAYS | UPDATE_ADJ);
 		}
+	    }
 	}
     }
 }
@@ -1223,19 +1319,12 @@ Unit *unit, *occ;
    to another.  This is a common internal routine, so no messages here. */
 
 void
-change_unit_side(unit, newside, reason, agent)
-Unit *unit, *agent;
-Side *newside;
-int reason;
+change_unit_side(Unit *unit, Side *newside, int reason, Unit *agent)
 {
     int ux = unit->x, uy = unit->y;
     Side *oldside = unit->side;
     Unit *occ;
 
-    if (newside == indepside) {
-	run_warning("non-null indepside");
-	newside = NULL;
-    }
     if (oldside == newside)
       return;
     /* Fail if the unit may not be on the new side. */
@@ -1258,6 +1347,9 @@ int reason;
 	cover_area(unit->side, unit, unit->transport, ux, uy, -1, -1);
 	/* Actually set the side slot of the unit here. */
 	set_unit_side(unit, newside);
+	/* Always redo the unit's number. */
+	unit->number = 0;
+	assign_unit_number(unit);
 	/* Cover it for the new side now. */
 	cover_area(unit->side, unit, unit->transport, -1, -1, ux, uy);
 	/* A freebie for the unit's previous side. */
@@ -1277,15 +1369,27 @@ int reason;
    on the given side. */
 
 int
-unit_allowed_on_side(unit, side)
-Unit *unit;
-Side *side;
+unit_allowed_on_side(Unit *unit, Side *side)
 {
     int u;
 
     if (unit == NULL)
       return FALSE;
     u = unit->type;
+    return new_unit_allowed_on_side(u, side);
+}
+
+/* It is crucial to pass the unit type instead of the unit itself as
+   argument in cases where the test must be applied before a new unit
+   is created, to prevent the appearance of independent ghost units
+   that never got the correct side set. Fortunately, there is no need
+   to pass a real unit since the test only uses the type. */
+
+int
+new_unit_allowed_on_side(int u, Side *side)
+{
+    int u2, sum;
+
     /* Test general limitations on the type. */
     if (!type_allowed_on_side(u, side))
       return FALSE;
@@ -1295,8 +1399,6 @@ Side *side;
 	  return FALSE;
     }
     if (g_units_per_side_max() >= 0) {
-	int u2, sum;
-
 	sum = 0;
 	for_all_unit_types(u2) {
 	    sum += side->numunits[u2];
@@ -1308,8 +1410,7 @@ Side *side;
 }
 
 int
-test_class_membership(leaf)
-Obj *leaf;
+test_class_membership(Obj *leaf)
 {
     char *sclass;
 
@@ -1329,29 +1430,70 @@ Obj *leaf;
     }
 }
 
-int
-type_allowed_on_side(u, side)
-int u;
-Side *side;
-{
-    if (side == NULL)
-      side = indepside;
-    if (side->uavail == NULL) {
-	int u2;
+/* Inability to build a unit should not preclude its capture
+   etc. Separate calls to has_advance_to_build have therefore been
+   introduced everywhere in the code. */
 
+int
+type_allowed_on_side(int u, Side *side)
+{
+    int u2;
+
+    if (side->uavail == NULL) {
 	side->uavail = (short *) xmalloc(numutypes * sizeof(short));
 	for_all_unit_types(u2) {
 	    tmpside_classtest = side;
 	    side->uavail[u2] =
-	      eval_boolean_expression(u_possible_sides(u2), test_class_membership, TRUE);
+	      eval_boolean_expression(u_possible_sides(u2),
+				      test_class_membership, TRUE);
 	}
     }
     return side->uavail[u];
 }
 
 int
-num_sides_allowed(u)
-int u;
+type_ever_available(int u, Side *side)
+{
+    int u2;
+    Unit *unit;
+
+    if (!type_allowed_on_side(u, side))
+      return FALSE;
+
+    for_all_side_units(side, unit) {
+	if (unit->type == u)
+	  return TRUE;
+    }
+    for_all_unit_types(u2) {
+	if (uu_acp_to_create(u2, u) > 0
+	    && type_allowed_on_side(u2, side))
+	  return TRUE;
+    }
+    /* (should add tests for capture etc) */
+    return FALSE;
+}
+
+/* TRUE if side has advance to build this unit. */
+
+int
+has_advance_to_build(Side *side, int u)
+{
+    int a;
+
+    for_all_advance_types(a) {
+	if (ua_needed_to_build(u, a) && !has_advance(side, a)) {
+	    return FALSE;
+	}
+    }
+    /* Account for an obsoleting advance. */
+    a = u_obsolete(u);
+    if (a != NONATYPE && has_advance(side, a))
+      return FALSE;
+    return TRUE;
+}
+
+int
+num_sides_allowed(int u)
 {
     int rslt;
     Side *side;
@@ -1366,12 +1508,8 @@ int u;
 }
 
 int
-unit_trusts_unit(unit1, unit2)
-Unit *unit1, *unit2;
+unit_trusts_unit(Unit *unit1, Unit *unit2)
 {
-    if (unit1->side == NULL || unit2->side == NULL)
-      return FALSE;
-    /* (should eventually be possible for some indeps to trust) */
     return (unit1->side == unit2->side
 	    || trusted_side(unit1->side, unit2->side));
 }
@@ -1384,66 +1522,65 @@ Unit *unit1, *unit2;
    side's units, in which case we just want to relink, don't care about
    testing whether the type is allowed or not. */
 
-int
-set_unit_side(unit, side)
-Unit *unit;
-Side *side;
+void
+set_unit_side(Unit *unit, Side *side)
 {
     int u = unit->type;
     Side *oldside, *newside;
 
-    if (side == indepside)
-      side = NULL;
-    if (unit->side != side) {
-	/* Subtract from the counts for the ex-side. */
-	oldside = (unit->side ? unit->side : indepside);
-	if (oldside->numunits)
-	  --(oldside->numunits[u]);
-	if (oldside->numlive && in_play(unit))
-	  --(oldside->numlive[u]);
-	/* Set the unit's slot. */
-	/* Note that indep units have a NULL side, even though there
+    /* Might not have anything to do. */
+    if (unit->side == side)
+      return;
+    /* Subtract from the counts for the ex-side. */
+    oldside = unit->side;
+    if (oldside->numunits)
+      --(oldside->numunits[u]);
+    if (oldside->numlive && in_play(unit))
+      --(oldside->numlive[u]);
+    /* Set the unit's slot. */
+    /* Note that indep units have a NULL side, even though there
 	   is an actual side object for independents. */
-	unit->side = side;
-	/* Make sure this unit is off anybody else's list. */
-	delete_unit(unit);
-	newside = (side ? side : indepside);
-	insert_unit(newside->unithead, unit);
-	/* Add to counts for the side. */
-	if (newside->numunits)
-	  ++(newside->numunits[u]);
-	if (newside->numlive && in_play(unit))
-	  ++(newside->numlive[u]);
-	/* Invalidate both sides' point value caches. */
-	oldside->point_value_valid = FALSE;
-	newside->point_value_valid = FALSE;
-	/* Bump the tech level if owning this type helps. */
-	if (side != NULL
-	    && side->tech[u] < u_tech_from_ownership(u)) {
-	    side->tech[u] = u_tech_from_ownership(u);
-	    /* (should update any displays of tech - how to ensure?) */
-	}
+    unit->side = side;
+    /* Make sure this unit is off anybody else's list. */
+    delete_unit(unit);
+    newside = side;
+    insert_unit(newside->unithead, unit);
+    /* Add to counts for the side. */
+    if (newside->numunits)
+      ++(newside->numunits[u]);
+    if (newside->numlive && in_play(unit))
+      ++(newside->numlive[u]);
+    /* Invalidate both sides' point value caches. */
+    oldside->point_value_valid = FALSE;
+    newside->point_value_valid = FALSE;
+    /* Bump the tech level if owning this type helps. */
+    if (side != NULL
+	&& side->tech[u] < u_tech_from_ownership(u)) {
+	side->tech[u] = u_tech_from_ownership(u);
+	/* (should update any displays of tech - how to ensure?) */
+    }
+    /* Turn on automation for units that switch to being independent
+	or under AI control. Note that the reverse is NOT
+	true. Switching from AI to human control will therefore leave
+	the unit on automatic. */
+    if (side == indepside || !side_has_display(side) || side_has_ai(side)) {
+	unit->autoplan = TRUE;
+	unit->autoresearch = TRUE;
+	unit->autobuild = TRUE;
     }
-    return TRUE;
 }
 
-int
-set_unit_origside(unit, side)
-Unit *unit;
-Side *side;
+/* The origside is more of a historical record or cache, doesn't need
+   the elaboration that unit side change does. */
+
+void
+set_unit_origside(Unit *unit, Side *side)
 {
-    if (unit->origside != side) {
-	/* Set the unit's slot. */
-	unit->origside = side;
-    }
-    return TRUE;
+    unit->origside = side;
 }
 
 void
-set_unit_name(side, unit, newname)
-Side *side;
-Unit *unit;
-char *newname;
+set_unit_name(Side *side, Unit *unit, char *newname)
 {
     /* Always turn 0-length names into NULL. */
     if (newname != NULL && strlen(newname) == 0)
@@ -1462,6 +1599,24 @@ char *newname;
     /* (should also send to any other side directly viewing this unit!) */
 }
 
+/* Given an amount to add, add it to the unit's hp, being careful
+   to check the limits.  Expect the caller to update the unit's
+   display, for instance there may be a health bar to change. */
+
+void
+add_to_unit_hp(Unit *unit, int hp)
+{
+    int hpmax;
+
+    unit->hp += hp;
+    hpmax = u_hp(unit->type);
+    if (unit->hp > hpmax)
+      unit->hp = hpmax;
+    unit->hp2 += hp;
+    if (unit->hp2 > hpmax)
+      unit->hp2 = hpmax;
+}
+
 void
 change_morale(unit, sign, morchange)
 Unit *unit;
@@ -1602,7 +1757,7 @@ int reason;
     /* Maybe enter the loss into the historical record. */
     if (reason >= 0)
       record_unit_death(unit, reason);
-    remove_unit_from_vector((unit->side ? unit->side : indepside)->actionvector, unit, -1);
+    remove_unit_from_vector(unit->side->actionvector, unit, -1);
     if (side != NULL) {
 	/* Invalidate the side's point value cache. */
 	side->point_value_valid = FALSE;
@@ -1613,6 +1768,11 @@ int reason;
 	if (alive(occ))
 	  kill_unit_aux(occ, reason);
     }
+
+    /* Advanced unit support. */
+    if (u_advanced(unit->type))
+    	free_used_cells(unit);
+    
     /* Update global counts. */
     --numliveunits;
     --(numlivebytype[u]);
@@ -1625,7 +1785,7 @@ int reason;
    Dead units stay on the dead unit list for each side until that side has had
    a chance to move.  Then they are finally flushed in a permanent fashion. */ 
 
-static void flush_one_unit PARAMS ((Unit *unit));
+static void flush_one_unit(Unit *unit);
 
 void
 flush_dead_units()
@@ -1866,9 +2026,15 @@ shortest_unique_name(u)
 int u;
 {
     char namebuf[BUFSIZE], *name1;
-    int u1, u2, i, allhavechars, firstuniq[MAXUTYPES], firstuniq1;
+    int u1, u2, i, len, allhavechars, firstuniq[MAXUTYPES], firstuniq1;
     int shortestdone[MAXUTYPES];
 
+	/* Don't try to allocate shortestnames before numutypes has been
+	 * defined. This will cause crashes later on as the program mistakenly
+	 * believes that all shortestnames[u] have been allocated just because 
+	 * shortestnames != NULL. */
+	if (numutypes == 0)
+		return NULL;
     if (shortestnames == NULL) {
 	shortestnames = (char **) xmalloc(numutypes * sizeof(char *));
 	/* First use game definition's single chars if possible. */
@@ -1922,10 +2088,14 @@ int u;
 	    }
 	    /* For any types where the unique short name is shorter
 	       than the seed name, truncate appropriately. */
+	    longest_shortest = 0;
 	    for_all_unit_types(u1) {
 		if (firstuniq[u1] + 1 < strlen(shortestnames[u1])) {
 		    (shortestnames[u1])[firstuniq[u1] + 1] = '\0';
 		}
+		len = strlen(shortestnames[u1]);
+		if (len > longest_shortest)
+		  longest_shortest = len;
 	    }
 	}
 	if (Debug) {
@@ -2468,50 +2638,60 @@ int u, s, x, y;
 
     if (!type_can_occupy_cell(u, x, y))
       return NULL;
+    side2 = side_n(s);
+    /* Check this BEFORE creating new unit! */
+    if (!new_unit_allowed_on_side(u, side2))
+      return NULL;
     newunit = create_unit(u, TRUE);
-    if (newunit != NULL) {
-	if (s != 0) {
-	    side2 = side_n(s);
-	    if (unit_allowed_on_side(newunit, side2)) {
-	    	set_unit_side(newunit, side2);
-	    	set_unit_origside(newunit, side2);
-	    	/* (should ensure that any changed counts are set correctly also) */
-	    }
-	}
-	init_supply(newunit);
-	if (can_occupy_cell(newunit, x, y)) {
-	    enter_cell(newunit, x, y);
-	} else {
-	    return NULL;
-	}
-	update_cell_display(side, x, y, TRUE);
-	update_unit_display(side, newunit, TRUE);
-	return newunit;
+    if (newunit == NULL)
+      return NULL;
+    if (s != 0) {
+	side2 = side_n(s);
+	set_unit_side(newunit, side2);
+	set_unit_origside(newunit, side2);
+	/* (should ensure that any changed counts are set correctly) */
+    }
+    init_supply(newunit);
+    if (can_occupy_cell(newunit, x, y)) {
+	enter_cell(newunit, x, y);
     } else {
+	/* (should undo creation of unit?) */
 	return NULL;
     }
+    update_cell_display(side, x, y, UPDATE_ALWAYS);
+    update_unit_display(side, newunit, TRUE);
+    return newunit;
 }
 
 /* Move a unit to a given location instantly, with all sides observing. */
 
 int
-designer_teleport(unit, x, y, other)
-Unit *unit, *other;
-int x, y;
+designer_teleport(Unit *unit, int x, int y, Unit *other)
 {
+    int oldx = unit->x, oldy = unit->y, rslt;
+    Side *side2;
+
     if (other != NULL && can_occupy(unit, other)) {
 	leave_cell(unit);
 	enter_transport(unit, other);
 	all_see_cell(x, y);
-	return TRUE;
+	rslt = TRUE;
     } else if (can_occupy_cell(unit, x, y)) {
-	leave_cell(unit);
-	enter_cell(unit, x, y);
-	all_see_cell(x, y);
-	return TRUE;
+	change_cell(unit, x, y);
+	rslt = TRUE;
     } else {
-	return FALSE;
+	rslt = FALSE;
     }
+    if (rslt) {
+	/* Provide accurate info on affected cells. */
+	for_all_sides(side2) {
+	    if (is_designer(side2)) {
+		see_exact(side2, oldx, oldy);
+		see_exact(side2, x, y);
+	    }
+	}
+    }
+    return rslt;
 }
 
 int
@@ -2539,4 +2719,3 @@ Unit *unit;
 }
 
 #endif /* DESIGNERS */
-
