@@ -7,7 +7,6 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-#include "kpublic.h"
 #include "macconq.h"
 
 #ifdef NEW_HEADERS
@@ -16,80 +15,86 @@ any later version.  See the file COPYING.  */
 #include <Picker.h>
 #endif
 
-// --- EXTERN DEFINED IN MACDRAW.C  ---------------------------------------------
+/* --- FUNCTIONS PROTOTYPES -------------------------------------------------*/
+
+extern void mac_init_cicn(Image *img);
+static int append_image_names(MenuHandle menu);
+static int append_color_names(MenuHandle menu);
+static int append_terrain_names(MenuHandle menu);
+static int append_imf_name(Obj *form, MenuHandle menu);
+static RGBColor get_color(char *name);					/* Get new color by name */
+static RGBColor pick_new_color(RGBColor inColor, char *name); 	/* Use color picker to get a new color */
+
+/* --- EXTERN DEFINED IN MACDRAW.C  -------------------------------------------*/
 
 extern Image **best_terrain_images;
 
-// --- FUNCTIONS PROTOTYPES ---------------------------------------------------
-
-static RGBColor get_color(char *name);					/* Get new color by name */
-static RGBColor pick_new_color(RGBColor inColor, char *name); 	/* Use color picker to get a new color */
-static void create_ppat_color(Str255 pname, RGBColor pColor);	/* Create a new color ppat */
-static void fill_in_empty_name(Side *side);					/* Fill in empty side name using nouns etc. */
-
-// --- DEFAULTS FOR NEW MAP PREFERENCES -----------------------------------------
+/* --- DEFAULTS FOR NEW MAP PREFERENCES ---------------------------------------*/
 
 int default_sidecolors = TRUE;		/* Use side colors in maps & lists */
 int default_iconmasks = TRUE;		/* Use masks for icons  */
 int default_boxmasks = TRUE;		/* Use masks for grouping boxes */
 int default_textmasks = FALSE;		/* Use masks for text */
 int default_featureborders = TRUE;	/* Draw feature (province) borders */
-int default_featurenames = TRUE;	/* Draw feature (province) names */
+int default_featurenames = TRUE;		/* Draw feature (province) names */
 int default_simple_borders = TRUE;	/* Do not draw borders to water or to empty lands */
 int default_shorelines = TRUE;		/* Draw shorelines in 3D relief */
-int default_draw_topline = TRUE;	/* Draw top line boxes in maps */
-int default_draw_topunit = TRUE;	/* Draw unit info boxes in maps */
+int default_draw_ai = TRUE;		/* Draw AI regions (if present). */
+int alert_256_colors = TRUE;		/* Suggest switch to 256 colors at start. */
+int default_draw_scrollbars = FALSE;	/* Draw scrollbars in maps. */
+int default_draw_cpanel = TRUE;		/* Draw control panel in maps. */
+int default_draw_emblems = TRUE;	/* Draw unit emblems in maps and lists. */
+int default_draw_topline = TRUE;		/* Draw top line boxes in maps */
+int default_draw_topunit = TRUE;		/* Draw unit info boxes in maps */
 int default_drawothermaps = TRUE;	/* Draw other maps as boxes */
-int default_erase_names = FALSE;	/* Draw extra rows to erase name of moving unit */
-int default_max_bufx = 600;			/* Horizontal scrolling buffers in offscreen graphics */
-int default_max_bufy = 400;			/* Vertical scrolling buffers in offscreen graphics */
+int default_erase_names = FALSE;		/* Draw extra rows to erase name of moving unit */
+int default_max_bufx = 600;		/* Horizontal scrolling buffers in offscreen graphics */
+int default_max_bufy = 400;		/* Vertical scrolling buffers in offscreen graphics */
 int default_optimize_fonts = TRUE;	/* Use optimized font sizes in map content */
+int default_blinking_curunit = TRUE;	/* Use blinking current unit instead of animated box */
+int default_selectionmasks = TRUE;	/* Use selection masks instead of selection boxes */
 int default_power = 5;				/* Default power of new map */
 
-// --- VARIOUS PREFERENCES -------------------------------------------------------
+/* --- VARIOUS PREFERENCES -----------------------------------------------------*/
 
 int use_colornames = TRUE;			/* Save and read side colors as colorschemes */
-int use_RGB_values = FALSE;			/* Save and read RGB colors in XCol resource */
+int use_RGB_values = FALSE;		/* Save and read RGB colors in XCol resource */
 int show_instructions = TRUE;		/* Show instructions window at start */
 int fullsize_map = TRUE;			/* Show full size map window at start */
 int unified_cicns = TRUE;			/* Use BitMap as mask for the PixMap */
 
-// --- COLOR SCHEME DEFAULTS -----------------------------------------------------
+/* --- COLOR SCHEME DEFAULTS ---------------------------------------------------*/
 
-int default_main_icon_color = 1;	/* Default number (1-3) of sideColor used for icons */ 
-int default_half_icon_color = 2;	/* Default number (1-3) of sideColor used for right half of icons */
-int default_icon_mask_color = 3;	/* Default number (1-3) of sideColor used for icon masks */
+int default_main_icon_color = 1;		/* Default number (1-3) of sideColor used for icons */ 
+int default_half_icon_color = 2;		/* Default number (1-3) of sideColor used for right half of icons */
+int default_icon_mask_color = 3;		/* Default number (1-3) of sideColor used for icon masks */
 int default_split_icon = TRUE;		/* Default use different colors for right and left halves of icons */
 		
-// --- COLORS, COLOR SCHEMES AND COLOR NAMES ----------------------------------------
+/* --- COLORS, COLOR SCHEMES AND COLOR NAMES --------------------------------------*/
+
+RGBColor blackcolor = {0, 0, 0};
+RGBColor graycolor = {32768, 32768, 32768};
 
 RGBColor *featColor;				/* Feature colors by RGB value */
 RGBColor *sideColor;				/* sideColor[0-15][1-3] are defined by default color schemes */
-									/* sideColor[][0] is not defined, and available for future use */
+							/* sideColor[][0] is not defined, and available for future use */
 
-// --- 68K COLORSCHEME BUG FIX -----------------------------------------------------
-
-extern char **current_colorscheme = 0;		/* Current color schemes by names (red,pink,blue) */
-									/* Unlike side->colorscheme defined also for null sides */	
-									/* Declared extern for use in macwins.c */
-
-//	char **colorscheme; 			/* Color schemes by name (red,pink,brown) etc */
-
-// ----------------------------------------------------------------------------
+char **current_colorscheme;		/* Current color schemes by names (red,pink,blue) */
+							/* Unlike side->colorscheme defined also for null sides */	
 
 char **default_featColorName;		/* Default feature colors by name (black etc) */
-char **default_colorscheme;			/* Default color schemes (red,pink,brown) etc */
+char **default_colorscheme;		/* Default color schemes (red,pink,brown) etc */
 
 char **featColorItemName;			/* Name of item for which featColor is set (Shorelines etc) */
-char **featColorName;				/* Name of feature color values (red, green etc) */
-char **sideColorName;				/* Name of side color values (red, green etc) */
+char **featColorName;			/* Name of feature color values (red, green etc) */
+char **sideColorName;			/* Name of side color values (red, green etc) */
 
 int *main_icon_color;				/* Number (1-3) of sideColor used for icons */ 
 int *half_icon_color;				/* Number (1-3) of sideColor used for right half of icons */
 int *icon_mask_color;				/* Number (1-3) of sideColor used for icon masks */
 int *split_icon;					/* Use two colors for left and right halves of icons if TRUE */
 
-// --- FUNCTIONS FOR BUILDING AND HANDLING COLORS AND COLOR SCHEMES -----------------------
+/* --- FUNCTIONS FOR BUILDING AND HANDLING COLORS AND COLOR SCHEMES ---------------------*/
 
 void 
 set_default_colors(void)			/* Default colors used in absence of saved preferences */
@@ -102,22 +107,8 @@ set_default_colors(void)			/* Default colors used in absence of saved preference
 	default_featColorName = (char **) xmalloc((FEATURES+1) * sizeof(char *));
 	sideColor = (RGBColor *) xmalloc((MAXSIDES+1) * 4 * sizeof(RGBColor));
 	sideColorName = (char **) xmalloc((MAXSIDES+1) * 4 * sizeof(char *));
-
-// --- 68K COLORSCHEME BUG FIX ----------------------------------------------------
-
 	current_colorscheme = (char **) xmalloc((MAXSIDES+1) * sizeof(char *)); 
-
-//	colorscheme = (char **) xmalloc((MAXSIDES+1) * sizeof(char *)); 
-
 	default_colorscheme = (char **) xmalloc((MAXSIDES+1) * sizeof(char *));
-
-//	for (i = 0; i <= MAXSIDES; i++) {
-//		side_n(i)->colorscheme = (char *) xmalloc(96);
-//		strcpy(side_n(i)->colorscheme,  "");
-//	}
-
-// ----------------------------------------------------------------------------
-
 	main_icon_color = (int *) xmalloc((MAXSIDES+1) * sizeof(int));
 	half_icon_color = (int *) xmalloc((MAXSIDES+1) * sizeof(int));
 	icon_mask_color = (int *) xmalloc((MAXSIDES+1) * sizeof(int));
@@ -139,27 +130,18 @@ set_default_colors(void)			/* Default colors used in absence of saved preference
 	default_colorscheme[13] = "light-pink,spring-green,midnight-blue";
 	default_colorscheme[14] = "dark-green,purple,aquamarine";
 	default_colorscheme[15] = "cyan,green-yellow,navy-blue";
-
-	default_featColorName[0] = "black";			/* Unused */
+	default_featColorName[0] = "dark-orange";		/* Window background outside map. */
 	default_featColorName[1] = "black";			/* Fore color */	
 	default_featColorName[2] = "pale-turquoise";	/* Mask color */
 	default_featColorName[3] = "midnight-blue";	/* Text color */
-	default_featColorName[4] = "dark-gray";			/* Grid color */
-	default_featColorName[5] = "black";			/* Unseen color */
-
-// --- DONT USE SAME DEFAULT COLORS FOR SHORELINES AND GRID ------------------------
-
-	default_featColorName[6] = "blue"; 		/* Shorelines */   // "mountains-3" also looks OK!
-
-//	default_featColorName[6] = "dark-gray";	/* Shorelines */
-
-// ---------------------------------------------------------------------
-
+	default_featColorName[4] = g_grid_color();
+	default_featColorName[5] = g_unseen_color();
+	default_featColorName[6] = g_shoreline_color();
 	default_featColorName[7] = "deep-pink";		/* Feature text and borders */
-	default_featColorName[8] = "red";			/* Frontlines */
-	default_featColorName[9] = "midnight-blue";	/* Meridians */
-	default_featColorName[10] = "saddle-brown";	/* Contour lines */
-
+	default_featColorName[8] = g_frontline_color();
+	default_featColorName[9] = g_meridian_color();
+	default_featColorName[10] = g_contour_color();
+	featColorItemName[0] = "Background";
 	featColorItemName[1] = "Fore color";	/* These names are used in the Colors menu */
 	featColorItemName[2] = "Mask color";
 	featColorItemName[3] = "Text color";
@@ -176,27 +158,11 @@ void
 set_default_side_colors(int e)
 {
 	/* First reset current colors to default colors */
-
-// --- 68K COLORSCHEME BUG FIX ----------------------------------------------------
-
 	side_n(e)->colorscheme = default_colorscheme[e];
-
-//	strcpy( side_n(e)->colorscheme, default_colorscheme[e]);
-
-// ----------------------------------------------------------------------------
-
 	/* Change the current colors */
 	set_side_colors(e);
 	/* Copy the result back to default colors */
-
-// --- 68K COLORSCHEME BUG FIX ----------------------------------------------------
-
 	default_colorscheme[e] = side_n(e)->colorscheme;
-
-//	strcpy(default_colorscheme[e], side_n(e)->colorscheme);
-
-// ---------------------------------------------------------------------------
-
 	force_overall_update();
 	save_preferences();
 }
@@ -231,28 +197,13 @@ init_side_colors(void)
 			set_sideColorName(i, 1, "undefined");
 			set_sideColorName(i, 2, "undefined");
 			set_sideColorName(i, 3, "undefined");
-
-// --- 68K COLORSCHEME BUG FIX ----------------------------------------------------
-
 			/* Use schemes defined in game file if present or else default schemes */
 			if (side_n(i) && side_n(i)->colorscheme)
-				current_colorscheme[i] = copy_string(side_n(i)->colorscheme);
-			else	current_colorscheme[i] = copy_string(default_colorscheme[i]);
+			  current_colorscheme[i] = copy_string(side_n(i)->colorscheme);
+			else
+			  current_colorscheme[i] = copy_string(default_colorscheme[i]);
 				
 			dissect_colorscheme(i);
-
-//			/* First load the default colorschemes */
-//			colorscheme[i] = default_colorscheme[i];
-
-//			/* Then overwrite them with schemes from game file */
-//			if (strcmp(side_n(i)->colorscheme, "") != 0) {
-//				colorscheme[i] = side_n(i)->colorscheme;
-//			}
-//			/* Dissect scheme into colors and color names */
-//			dissect_colorscheme(i, colorscheme[i]);
-
-// ----------------------------------------------------------------------------
-
 		}
 
 		/* Load color use parameters from their defaults values */
@@ -278,33 +229,19 @@ init_side_colors(void)
 		featColorName[i] = default_featColorName[i];
 		featColor[i] = get_color(featColorName[i]);
 	}
-
-// --- MOVED HERE FROM INIT_TERRAIN_IMAGES -----------------------------------------
-
+	/* This is now checked once and for all, instead of every time an area background is drawn. */
 	if (strcmp(gridcolor_name, unseencolor_name) == 0
-	    && empty_string(g_unseen_image_name()))
-		  grid_matches_unseen = TRUE;
-	blackcolor.red = blackcolor.green = blackcolor.blue = 0;
-
-// ---------------------------------------------------------------------------
-
+		|| g_terrain_seen() || (dside)->see_all)
+			grid_matches_unseen = TRUE;
 }
 
 /* This function is based on init_emblem from the X11 interface. */
 /* It also tests each name for validity and stores it in an array. */
 
-// --- 68K COLORSCHEME BUG FIX ----------------------------------------------------
-
 void
 dissect_colorscheme(int sidenum)
-
-//	dissect_colorscheme(int sidenum, char* colorscheme)
-
 {
 	char		*colorscheme = current_colorscheme[sidenum];
-	
-// ----------------------------------------------------------------------------
-
 	char	 	*s, *c;
 	char		cbuf[BUFSIZE];
 	int 		i, button;
@@ -351,107 +288,45 @@ get_color(char* name)
 	ImageFamily	*imf;
 	Image 		*img;
 	MacImage		*macimg;
-	short			button;
-	Str255		pname, filename;
-    	RGBColor 		newColor;
+	PixMapHandle 	pmhandle;
+	CTabHandle 	ctabhandle;
+	ColorSpec 		*ctdata;
 
-	/* Return without color if name is "undefined" */	
-	if (strcmp(name, "undefined") == 0) return;
+	/* Return black if name is "undefined" */	
+	if (strcmp(name, "undefined") == 0)
+	     return blackcolor;
 
-	/* Find (or create) the image family */
-	imf = get_imf(name);
-	mac_load_imf(imf);
+	/* Find (or create) the image family. */
+	imf = get_generic_images(name);
+
+	/* Needed only in the presence of mac-lib. */ 
+    	record_imf_get(imf);			
+
+	/* Get the 1x1 image (color). */
 	img = get_img(imf, 1, 1);
 
-	/* Choose the wanted color if it already exists */
-	if (img->hook) {
-		macimg = (MacImage *) img->hook;
-		newColor = macimg->color;
-		return newColor;
-	} else {
-		c2p(name, pname);
-		ParamText("\pColor ", pname, "\p does not exist. ", 
-				"\pDo you want to create a new color with that name?");
-		button = Alert(aTwoButtonAlert, NULL);
-		if (button == 1)	{			
-			/* Pick the new color */
-			newColor = pick_new_color(newColor, name);
-			ParamText("\pDo you wish to store color ", pname, 
-					 "\p in your Images file?", "\p");
-			button = Alert(aTwoButtonAlert, NULL);
-			if (button == 1) {
-				/* Create and store a new ppat color in the Images file */
-				create_ppat_color(pname, newColor);
-				/* Load the new ppat color into the image family */
-				mac_load_imf(imf);
-			}
-			return newColor;
-		/* Set name to "undefined" if no color was picked */
-		} else strcpy(name, "undefined");
-	}
-}
+	/* Needed only in the presence of mac-lib. */ 
+	mac_interp_imf(imf, img, 0);	
 
-/* 
+	/* Get the resulting mac image. */
+	macimg = (MacImage *) img->hook;
 
-This function creates a new ppat of the desired color, using the black ppat as a template. It might
-seem logical to just create a PixPat and save it as a ppat. However, this does not work since there
-is no easy way to make a ppat from a PixPat. GetPixPat converts ppats to PixPats, but there is no
-toolbox trap for the other direction! 
+	/* Return black if it has no colrpat. */
+	if (!macimg->colrpat) 
+	      return blackcolor;
 
-*/
-
-void
-create_ppat_color(Str255 pname, RGBColor pColor)
-{
-	short			ppatID, button;
-	Handle 		blackHandle, colorHandle;
-	CTabHandle 	ctabHandle;
-	Str255 		filename;
-
-	/* Get a handle to the black ppat in the Images file */
-	UseResFile(images_refnum);
-	if (ResError()) {
-		ParamText("\pCannot find your Images file! ", 
-				 "\pThe new color was not stored.", "\p", "\p");
-		button = Alert(aOneButtonAlert, NULL);
-		return;
-	} 
-	blackHandle = GetNamedResource('ppat', "\pblack");
+	/* Copy colrpat data into macimg->color. */
+	pmhandle = (*macimg->colrpat)->patMap;
+	ctabhandle = (*pmhandle)->pmTable;
+	ctdata = (ColorSpec *) &((*ctabhandle)->ctTable);
+	macimg->color.red   	= ctdata[0].rgb.red;
+	macimg->color.green 	= ctdata[0].rgb.green;
+	macimg->color.blue  	= ctdata[0].rgb.blue;
+	macimg->colordefined 	= TRUE;
 	
-	/* Get a new handle and copy the black ppat into it */
-	colorHandle = NewHandle(102);
-	HLock(colorHandle);
-	HLock(blackHandle);
-	memcpy(*colorHandle, *blackHandle, 102);
-	/* Disposing of blackHandle will cause a crash! */
-	HUnlock(blackHandle);
-
-	/* Create a single color ctab and store the new color inside */
-	ctabHandle = (CTabHandle) NewHandle(16);
-	HLock((Handle) ctabHandle);
-	(*ctabHandle)->ctFlags = 0;
-	(*ctabHandle)->ctSeed = GetCTSeed();
-	(*ctabHandle)->ctSize = 0;
-	(*ctabHandle)->ctTable->value = 0;
-	(*ctabHandle)->ctTable->rgb = pColor;
-
-	/* Copy the new ctab into the new handle */
-	memcpy(*colorHandle + 86, *ctabHandle, 16);
-	HUnlock((Handle) ctabHandle);
-	DisposHandle((Handle) ctabHandle);
-	
-	/* Add a new ppat resource using the new handle */
-	UseResFile(images_refnum);
-	ppatID = UniqueID('ppat');
-	AddResource(colorHandle, 'ppat', ppatID, pname);
-	UpdateResFile(images_refnum);
-	/* Disposing of colorHandle will cause a crash! */
-	HUnlock(colorHandle);
-
-	/* Run get_files to open all resource files, or we will get the "cannot find sound */
-	/* crunch" whine. Editing one file makes Xconq lose track of all other open files. */
-	get_files();
-}
+	/* Return the resulting color. */
+	return macimg->color;
+}	
 
 /* Given an initial color and a name, allow the user to pick a color
    for that name. */
@@ -483,7 +358,7 @@ pick_new_color(RGBColor inColor, char* name)
 	  return inColor;
 }
 
-// --- FUNCTIONS FOR READING AND WRITING XCOL RGB DATA --------------------------------
+/* --- FUNCTIONS FOR READING AND WRITING XCOL RGB DATA ------------------------------*/
 
 /* (should generify all this) */
 
@@ -607,7 +482,7 @@ write_xcol(void)
 	}
 }
 
-// --- COLOR MANAGING DIALOGS ----------------------------------------------------
+/* --- COLOR MANAGING DIALOGS --------------------------------------------------*/
 
 /* The following three dialogs are used by the Colors menu and also by Preferences : Colorschemes */
 
@@ -639,13 +514,6 @@ set_side_colors(int e)
 	Str255 pname;
 	DialogPtr win;
 
-	/* First check for the Images file */
-	n = GetResFileAttrs(images_refnum);
-	if (ResError()) {
-		ParamText("\pCannot find your lib-mac : Images file!", "\p", "\p", "\p");
-		button = Alert(aOneButtonAlert, NULL);
-		return;
-	} 
 	/* Then check for color support */
 	if (!hasColorQD) {
 		ParamText("\pSorry, your mac does not support colors.", "\p", "\p", "\p");
@@ -702,22 +570,12 @@ set_side_colors(int e)
 	c2p(tmpName3, pname);
 	SetIText(itemhandle, pname);
 
-	/* Load all available ppat colors into the popup menu */
+	/* Add "undefined" at the top of the popup menu. */
 	AppendMenu(popMenu, "\pundefined");
-	AppendMenu(popMenu, "\p(-");		/* Item separator */
-	n = CountResources('ppat');
-	for (i = 1; i <= n; ++i ) {
-		ppathandle = GetIndResource('ppat', i);
-		GetResInfo(ppathandle, NULL, NULL, pname);
-		/* Only use ppats that live in the Images file */
-		if (HomeResFile(ppathandle) != images_refnum)
-		  continue;
-		p2c(pname, cname);
-		/* Skip invalid names */
-		if (!valid_imf_name(cname))
-		  continue;
-		AppendMenu(popMenu, pname);
-	}
+	/* Add an item separator. */
+	AppendMenu(popMenu, "\p(-");
+	/* Then append available color and terrain names. */
+	append_image_names(popMenu);
 
 	/* Change port and start the main loop */
 	GetPort(&oldport);
@@ -876,12 +734,14 @@ set_side_colors(int e)
 		/* Plot 5 sample icons (if available) */
 		for (i = 0; i <= 4; i++) {
 			/* Skip now if we ran out of unit types */			
-			if (i > numutypes - 1) continue;
+			if (i > numutypes - 1)
+			     continue;
 			/* Get the best unit image from family */
 			uimg = best_image(uimages[i], 32, 32);
 			macuimg = (MacImage *) uimg->hook;
-			/* Get the cicn if it exists */
-			if (!macuimg->colricon) continue;
+			/* mac_init_cicn now always creates colricon. */
+			if (!macuimg->colricon)
+			     mac_init_cicn(uimg);
 			cicnhandle = (CIconHandle) macuimg->colricon;
 			/* Get pixmap, bitmap & maskmap from cicn and set srcrect */
 			bm = (*(cicnhandle))->iconBMap;
@@ -1154,13 +1014,6 @@ set_feature_color(int e)
 	Str255 pname;
 	DialogPtr win;
 
-	/* First check for the Images file */
-	n = GetResFileAttrs(images_refnum);
-	if (ResError()) {
-		ParamText("\pCannot find your lib-mac : Images file!", "\p", "\p", "\p");
-		button = Alert(aOneButtonAlert, NULL);
-		return;
-	} 
 	/* Then check for color support */
 	if (!hasColorQD) {
 		ParamText("\pSorry, your mac does not support colors.", "\p", "\p", "\p");
@@ -1181,21 +1034,9 @@ set_feature_color(int e)
 	c2p(tmpName, pname);
 	SetIText(itemhandle, pname);
 
-	/* Load all available ppat colors into the popup menu */
-	n = CountResources('ppat');
-	for (i = 1; i <= n; ++i ) {
-		ppathandle = GetIndResource('ppat', i);
-		GetResInfo(ppathandle, NULL, NULL, pname);
-		/* Only use ppats that live in the Images file */
-		if (HomeResFile(ppathandle) != images_refnum)
-		  continue;
-		p2c(pname, cname);
-		/* Skip invalid names */
-		if (!valid_imf_name(cname))
-		  continue;
-		AppendMenu(popMenu, pname);
-	}
-
+	/* Append available color and terrain names to the popup menu. */
+	append_image_names(popMenu);
+	
 	/* Change port and start the main loop */
 	GetPort(&oldport);
 	SetPort(win);
@@ -1290,9 +1131,9 @@ set_feature_color(int e)
 				featColor[e] = tmpColor;
 				featColorName[e] = copy_string(tmpName);
 
-				/* Set grid_matches_unseen */
+				/* This is now checked only once, instead of every time a map is redrawn. */
 				if (strcmp(gridcolor_name, unseencolor_name) == 0
-					 && empty_string(g_unseen_image_name())) {
+				    || g_terrain_seen() || (dside)->see_all) {
 		 	 		 grid_matches_unseen = TRUE;
 				} else grid_matches_unseen = FALSE;
 
@@ -1333,13 +1174,6 @@ set_terrain_color( Str255 tname)
 	Image *img;
 	DialogPtr win;
 
-	/* First check for the Images file */
-	n = GetResFileAttrs(images_refnum);
-	if (ResError()) {
-		ParamText("\pCannot find your lib-mac : Images file!", "\p", "\p", "\p");
-		button = Alert(aOneButtonAlert, NULL);
-		return;
-	} 
 	/* Then check for color support */
 	if (!hasColorQD) {
 		ParamText("\pSorry, your mac does not support colors.", "\p", "\p", "\p");
@@ -1372,21 +1206,9 @@ set_terrain_color( Str255 tname)
 		break;
 	}
 
-	/* Load all available ppat colors into the popup menu */
-	n = CountResources('ppat');
-	for (i = 1; i <= n; ++i ) {
-		ppathandle = GetIndResource('ppat', i);
-		GetResInfo(ppathandle, NULL, NULL, pname);
-		/* Only use ppats that live in the Images file */
-		if (HomeResFile(ppathandle) != images_refnum)
-		  continue;
-		p2c(pname, cname);
-		/* Skip invalid names */
-		if (!valid_imf_name(cname))
-		  continue;
-		AppendMenu(popMenu, pname);
-	}
-
+	/* Append available color and terrain names to the popup menu. */
+	append_image_names(popMenu);
+	
 	/* Change port and start the main loop */
 	GetPort(&oldport);
 	SetPort(win);
@@ -1482,9 +1304,11 @@ set_terrain_color( Str255 tname)
 
 			} else {	/* Update everything and exit the dialog */
 
-				/* Replace terrain images used for drawing */
+				/* Replace timages[e] with the new image family. */
+				timages[e] = imf;
+				/* Recompute its best images. */
 				for (i = 0; i < NUMPOWERS; ++i) {
-					img = best_image(imf, hws[i], hcs[i]);
+					img = best_image(timages[e], hws[i], hhs[i]);
 					if (img && e != -1) {
 						best_terrain_images[i * numttypes + e] = img;
 					}
@@ -1513,7 +1337,7 @@ set_terrain_color( Str255 tname)
 	update_all_map_windows();
 }
 
-// --- FUNCTIONS USED TO BUILD AND MAINTAIN MENUS -------------------------------------
+/* --- FUNCTIONS USED TO BUILD AND MAINTAIN MENUS -----------------------------------*/
 
 /* Insert side and feature names into the colors menu */
 
@@ -1528,88 +1352,43 @@ build_colors_menu(void)
 	/* First load side names at the top of the menu */
 	/* Side 0 (Independents) is already in the menu */
 	for (i = 1; i <= numsides; i++) {
-		if (!side_n(i)->name) {
-			fill_in_empty_name(side_n(i));
-		}
-		c2p(side_n(i)->name, pname);
+		sanitize_for_menu(shortest_side_title(side_n(i), tmpbuf), pname);
 		InsMenuItem(menu, pname, i);
 	}
 	/* Then load feature names below first separator */
-	for (i = 1; i <= FEATURES; i++) {
-		c2p(featColorItemName[i], pname);
-		InsMenuItem(menu, pname, numsides + 1 + i);
+	for (i = 0; i <= FEATURES; i++) {
+		sanitize_for_menu(featColorItemName[i], pname);
+		InsMenuItem(menu, pname, numsides + i + 2);
 	}
-
 	/* Then load terrain types below second separator */
 	for_all_terrain_types(t) {
-		/* Use image names if they exist */
-		if (!empty_string(t_image_name(t))) {
-			 sanitize_for_menu(t_image_name(t), pname);
-		/* Else use terrain names (may differ from image names */
-		} else
-		  sanitize_for_menu(t_type_name(t), pname);
-		InsMenuItem(menu, pname, numsides + FEATURES + 3 + t);
+		sanitize_for_menu(t_type_name(t), pname);
+		InsMenuItem(menu, pname, numsides + FEATURES + t + 4);
 	}
 }
 
-/* Use pluralnoun, noun, adjective, or as last resort side number to fill in an empty name */
-/* Some game modules come with nouns etc but not with side names! */
-
-void
-fill_in_empty_name(Side *side)
-{
-	Str255 pnumber;
-	char cnumber[32];
-
-	/* (should use copy_string?) */
-	if (!side->name) {
-		side->name = (char *) xmalloc(32);
-	}
-	if (!empty_string(side->name)) {
-		return;
-	} else if (!empty_string(side->pluralnoun)) {
-		strcpy(side->name, side->pluralnoun);
-	} else if (!empty_string(side->noun)) {
-		strcpy(side->name, side->noun);
-	} else if (!empty_string(side->adjective)) {
-		strcpy(side->name, side->adjective);
-	} else {
-		NumToString(side_number(side), pnumber);
-		p2c(pnumber, cnumber);		
-		strcpy(side->name, "Side ");
-		strcat(side->name, cnumber);   
-	}
-	
-}
-
 void
 build_sides_menu(void)
 {
 	MenuHandle menu;
-	Str255 pname, pnumber;
-	char cname[8], cnumber[4];
+	Str255 pname;
 	int i;
 
 	menu = GetMenu(mSidesPopup);
 	/* First load real side names for sides in play */
 	/* Side 0 (Independents) is already in the menu */
 	for (i = 1; i <= numsides; i++) {
-		if (!side_n(i)->name) {
-			fill_in_empty_name(side_n(i));
-		}
-		c2p(side_n(i)->name, pname);
-		InsMenuItem(menu, pname, i);
+		sanitize_for_menu(shortest_side_title(side_n(i), tmpbuf), pname);
+		AppendMenu(menu, pname);
 	}
 	/* Then load side numbers for remaining sides */
 	for (i = numsides + 1; i <= MAXSIDES; i++) {
-		NumToString(i, pnumber);
-		p2c(pnumber, cnumber);		
-		strcpy(cname, "Side ");
-		strcat(cname, cnumber);   
-		c2p(cname, pname);
-		InsMenuItem(menu, pname, i);
+		sprintf(tmpbuf, "Side %d", i);
+		sanitize_for_menu(tmpbuf, pname);
+		AppendMenu(menu, pname);
 	}
 }
+
 void
 build_features_menu(void)
 {
@@ -1619,13 +1398,128 @@ build_features_menu(void)
 
 	menu = GetMenu(mFeatPopup);
 	/* Load available feature names */
-	for (i = 1; i <= FEATURES; i++) {
-		c2p(featColorItemName[i], pname);
-		/* (i - 1) since this menu is empty */
-		InsMenuItem(menu, pname, i -1);
+	for (i = 0; i <= FEATURES; i++) {
+		sanitize_for_menu(featColorItemName[i], pname);
+		AppendMenu(menu, pname);
 	}
 }
 
+int
+append_image_names(MenuHandle menu)
+{
+	Handle	ppathandle;  
+	char 		cname[32];
+	Str255	pname;
+	int 		i, n, num = 0;
 
+	/* Use lib-mac Images file if available. */
+	n = GetResFileAttrs(images_refnum);
+	if (!ResError()) {
+		/* Load all available ppat colors into the popup menu. */
+		n = CountResources('ppat');
+		for (i = 1; i <= n; ++i ) {
+			ppathandle = GetIndResource('ppat', i);
+			GetResInfo(ppathandle, NULL, NULL, pname);
+			/* Only use ppats that live in the Images file. */
+			if (HomeResFile(ppathandle) != images_refnum)
+				continue;
+			p2c(pname, cname);
+			/* Skip invalid names. */
+			if (!valid_imf_name(cname))
+				continue;
+			AppendMenu(menu, pname);
+			++num;
+		}
 
+	/* Else use colors.imf and terrain.imf. */
+	} else {
+		/* First load all real colors. */
+		num += append_color_names(menu);
+		/* Then add an item separator. */
+		if (num)
+		    AppendMenu(menu, "\p(-");
+		/* Then load the terrain colors. */
+		num += append_terrain_names(menu);
+	}
+	return num;
+}
 
+int
+append_color_names(MenuHandle menu)
+{
+	int	num = 0, startlineno = 1, endlineno = 1;
+	LibraryPath *p;
+	Obj	*form;
+	FILE	*fp;
+
+	for_all_library_paths(p) {
+    		make_pathname(p->path, "colors.imf", "", spbuf);
+		fp = open_file(spbuf, "r");
+		if (fp != NULL) {
+			while ((form = read_form(fp, &startlineno, &endlineno)) != lispeof) {
+				if (append_imf_name(form, menu))
+				    ++num;
+			}
+			fclose(fp);
+			return TRUE;
+		}
+	}
+		run_warning("Could not find your colors.imf file");
+		return FALSE;		
+	}
+
+int
+append_terrain_names(MenuHandle menu)
+{
+	int	num = 0, startlineno = 1, endlineno = 1;
+	LibraryPath *p;
+	Obj	*form;
+	FILE	*fp;
+
+	for_all_library_paths(p) {
+    		make_pathname(p->path, "terrain.imf", "", spbuf);
+		fp = open_file(spbuf, "r");
+		if (fp != NULL) {
+			while ((form = read_form(fp, &startlineno, &endlineno)) != lispeof) {
+				if (append_imf_name(form, menu))
+				    ++num;
+			}
+			fclose(fp);
+			return TRUE;
+		}
+	}
+		run_warning("Could not find your terrain.imf file");
+	return FALSE;		
+}	
+
+int
+append_imf_name(Obj *form, MenuHandle menu)
+{
+	Str255 	lastname, pname;
+	char 		*name, cname[32];
+	Obj		*head;
+	
+	/* Ignore any non-lists. */
+	if (!consp(form))
+	    return FALSE;
+	head = car(form);
+	/* Ignore any non-imf forms. */
+	if (!match_keyword(head, K_IMF))
+	    return FALSE;
+	/* Ignore imf without a name. */
+	if (!stringp(cadr(form)))
+	    return FALSE;
+	name = c_string(cadr(form));
+	/* Ignore invalid imf names. */
+	if (!valid_imf_name(name))
+	    return FALSE;
+	/* Get the last item name in the menu. */
+	GetItem(menu, CountMItems(menu), lastname);
+	p2c(lastname, cname);
+	/* Skip if imf has the same name as last item. */
+	if (strcmp(name, cname) == NULL)
+	    return FALSE;
+	c2p(name, pname);
+	AppendMenu(menu, pname);
+	return TRUE;
+}
