@@ -1,5 +1,5 @@
 /* Main program of the Mac interface to Xconq.
-   Copyright (C) 1992-1998 Stanley T. Shebs.
+   Copyright (C) 1992-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,22 +7,16 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern int ok_to_exit;
-#include "kpublic.h"
 #include "macconq.h"
-extern void recalc_spacing(void);
-extern void invert_map(Map *map, int x, int y, int r);
-
-extern char *connection_method_name;
-extern int connection_method;
-extern int hosting;
-
-extern int numwindows;
-extern WindowPtr *winmenuwins;
 
 #include <errors.h>
 #include <time.h>
 
+/* Include Profile.h if profiling is enabled in CodeWarrior. */
+#if __profile__
+#include <Profiler.h>
+#endif
+
 #ifdef __MWERKS__
 #include <Sound.h>
 #endif
@@ -36,11 +30,19 @@ extern WindowPtr *winmenuwins;
 #define PREFSIZE 2048		/* Size of prefs buffer in lisp format. Must be bigger than the default */
 						/* BUFSIZE = 256 to make room for all the new preferences */
 
-extern
-#if 0
-inline
-#endif
-int handle_numeric_keypad(EventRecord *event);
+extern int ok_to_exit;
+extern short initialvrefnum;
+
+char *connection_method_name;
+
+extern void auto_pick_side_research(Side *side);
+extern void show_research_dialog();
+extern void draw_research_dialog(int force);
+extern int do_key_down_research(char key);
+extern void hit_research_dialog(int ditem);
+
+extern void recalc_spacing(void);
+extern void invert_map(Map *map, int x, int y, int r);
 
 /* Function prototypes. */
 
@@ -49,6 +51,8 @@ static void init_fonts(void);
 static void init_rects(void);
 static void init_ae(void);
 static void handle_event(EventRecord *event);
+static int handle_keystroke(EventRecord *event);
+static void drag_window(WindowPtr win, EventRecord *event);
 static pascal OSErr do_ae_open_application(AppleEvent *message, AppleEvent *reply, long refcon);
 static pascal OSErr do_ae_open_documents(AppleEvent *message, AppleEvent *reply, long refcon);
 static pascal OSErr do_ae_print_documents(AppleEvent *message, AppleEvent *reply, long refcon);
@@ -91,6 +95,10 @@ int foundimagesfile = FALSE;
 
 int sbarwid = 15;
 
+/* The width of a floating window scrollbar. */
+
+int floatsbarwid = 10;
+
 /* Info about various fonts we use. */
 
 short small_font_id;
@@ -126,11 +134,6 @@ Side *dside = NULL;
 
 Handle spare;
 
-/* This is true if savable prefs etc have been saved since being changed. */
-/* (always true for now, nothing being remembered) */
-
-int interfacestatesafe = TRUE;
-
 /* This is true when a single click suffices to move a unit. */
 
 int defaultmoveonclick = TRUE;
@@ -228,13 +231,24 @@ main()
 	init_library_path(NULL);
 	init_ae();
 	/* Acquire Mac-specific files (preferences and resources). */
-	/* Check if any screen has more than 256 colors and propose reduction. */	
-	check_screen_depths();
 	/* Load colors used in absence of saved preferences. */
 	set_default_colors();
 	get_files();
+	/* Check if any screen has more than 256 colors and propose reduction. */	
+	/* Don't check screen if this function has been turned off. */
+	if (alert_256_colors)
+		check_screen_depths();
 	/* A hack to ensure some memory available for error handling. */
 	spare = NewHandle(2000);
+
+#if __profile__
+
+	/* Prepare profiling for CodeWarrior. */
+	ProfilerInit(collectDetailed, bestTimeBase, 1250, 36);
+	ProfilerSetStatus(false);
+
+#endif
+
 	/* If no Apple Events, go to the splash screen now, otherwise we'll wait
 	   for an oapp/odoc/pdoc event to decide what to do. */
 	if (!hasAppleEvents) {
@@ -278,8 +292,6 @@ init_toolbox()
 {
 	SysEnvRec se;
 	
-// --- 68K DEFAULT STACK SIZE INCREASE ---------------------------
-
 #if  __MC68K__
 
 	/* Increase stack size from default 24K (ColorQD) to 64K by adding 40K. */
@@ -288,14 +300,13 @@ init_toolbox()
 	
 #endif
 
-// --------------------------------------------------------	
-
 	MaxApplZone();
 
 	InitGraf(&QD(thePort));
 	InitFonts();
 	FlushEvents(everyEvent, 0);
 	InitWindows();
+	InitFloats();
 	InitMenus();
 	TEInit();
 	InitDialogs(NULL);
@@ -464,7 +475,6 @@ get_files()
 {
 	int imagesfilenameres;
 	Str255 filename;
-	extern short initialvrefnum;
 
 	/* Capture the current vrefnum. */
 	GetVol(NULL, &initialvrefnum);
@@ -504,13 +514,13 @@ get_files()
 void
 event_loop()
 {
-	int done = FALSE;
-	Boolean gotevent;
-	Point mouse;
+	int 			done = FALSE;
+	Boolean		gotevent;
+	Point 		mouse;
 	EventRecord	event;
-	RgnHandle cursorRgn;
-	short itemhit;
-	DialogPtr dialog;
+	RgnHandle 		cursorRgn;
+	short 		itemhit;
+	DialogPtr 		dialog;
 
 	/* Figure out if the WaitNextEvent Trap is available. */
 	useWNE = (NGetTrapAddress(0x60, ToolTrap) != NGetTrapAddress(0x9f, ToolTrap));
@@ -518,50 +528,46 @@ event_loop()
 	cursorRgn = NewRgn();
 	/* Loop (almost) forever. */
 	while (!eventloopdone) {
+		/* Move here since we must call adjust_cursor at least once. Thus, we can 
+		avoid the second call below by always calling it here. */
+		get_global_mouse(&mouse);
+		adjust_cursor(mouse, cursorRgn);
 		/* Use WaitNextEvent if it is available, otherwise GetNextEvent. */
 		if (useWNE) {
-			get_global_mouse(&mouse);
-			adjust_cursor(mouse, cursorRgn);
 			gotevent = WaitNextEvent(everyEvent, &event, 0L, cursorRgn);
 		} else {
 			SystemTask();
 			gotevent = GetNextEvent(everyEvent, &event);
 		}
-		/* First decide if the event is for a dialog or is just any old event. */
-		if (FrontWindow() != nil && IsDialogEvent(&event)) {
-			/* Handle all the modeless dialogs here. */
-			/* Process any player setup window events that we want to handle
-			   specially (such as keystrokes). */
-			if (playersetupwin != nil && playersetupwin == FrontWindow())
-			  handle_player_setup_event(&event);
-			/* Avoid handling any dialog events to other than the windows that
-			   we know to be dialogs. */
-			if (!(instructionswin == FrontWindow()
-				  || playersetupwin == FrontWindow()))
-			  continue;  
-			if (DialogSelect(&event, &dialog, &itemhit)) {
-				if (dialog == instructionswin) {
-					hit_instructions_dialog(dialog, itemhit, &event);
-				}
-				/* Process player setup window events that the Dialog
-				   Manager handles (clicks on buttons for instance). */
+		/* Deal with keystrokes first since their handling now is the same 
+		for modeless dialogs and normal windows. */
+		if (event.what == keyDown || event.what == autoKey) {
+			if (handle_keystroke(&event))
+			    continue;
+		}
+		/* Then handle non-keystroke modeless dialog events. */
+		if (IsDialogEvent(&event) && DialogSelect(&event, &dialog, &itemhit)) {
 				if (dialog == playersetupwin) {
-					mouse = event.where;
-					SetPort(playersetupwin);
-					GlobalToLocal(&mouse);
-					if (hit_player_setup_dialog(itemhit, mouse))
-					  launch_game_2();
+				hit_player_setup_dialog(itemhit, mouse);
+			} else if (dialog == researchwin) {
+				hit_research_dialog(itemhit);
+			} else if (topFloat && unit_closeup_from_window(topFloat)) {
+				hit_closeup_dialog(topFloat, itemhit, mouse);
 				}
 				/* If this was something like a key or click event, but wasn't handled
-				   by the dialog, we're done and can go wait for the next event.
-				   Otherwise, for null events, fall through to the usual handling. */
+			   by the dialog, we're done and can go wait for the next event. Otherwise, 
+			   for null events, fall through to the usual handling. */
 				if (gotevent)
 				  continue;
 			}
-		}
+		
 		if (gotevent) {
+
+#if 0			/* Redundant, adjust_cursor is now always called above. */
+		
 			/* Make sure we have the right cursor before handling the event. */
 			adjust_cursor(event.where, cursorRgn);
+#endif
 			handle_event(&event);
 		} else if (connection_method > 0 && numremotewaiting > 0) {
 			receive_data(0);
@@ -570,10 +576,21 @@ event_loop()
 			/* DON'T automatically go to a watch cursor, since run_game often
 			   returns very quickly.  Instead, long-running subroutines should
 			   call back to put a watch cursor up. */
+#if __profile__
+	ProfilerSetStatus(true);
+#endif
+
 			run_local_ai(1, 20);
 			net_run_game(1);
+
+#if __profile__
+	ProfilerSetStatus(false);
+#endif
+
 			maybe_select_next_unit();
+#if 0
 			run_local_ai(2, 20);
+#endif
 			/* If the game ended, force various changes in interaction. */
 			if (endofgame)
 			  set_end_of_game_interaction_modes();
@@ -583,7 +600,7 @@ event_loop()
 			receive_data(0);
 		}
 		/* Blink TE cursors regularly. */
-		if (commandwin != nil && FrontWindow() == commandwin) {
+		if (commandwin != nil && topWindow == commandwin) {
 			TEIdle(command_text);
 		}
 		if (ok_to_exit && want_to_exit) {
@@ -592,6 +609,82 @@ event_loop()
 	}
 }
 
+/* This function returns TRUE for keyboard events that get handled and
+FALSE for those who should fall through to the dialog manager. */
+
+int
+handle_keystroke(EventRecord *event)
+{
+	char	charCode = event->message & charCodeMask;
+	char	keyCode = (event->message & keyCodeMask) >> 8;
+	int	handled = FALSE;
+	UnitCloseup *closeup;
+	List	*list;
+	
+	/* Menu key commands. */
+	if (event->modifiers & cmdKey
+	/* Don't interpret cmd + keypad as menu command. */
+	    && keyCode < 0x40) {
+		adjust_menus();
+		do_menu_command(MenuKey(charCode));
+		return TRUE;
+	}
+	/* Alt-w closes topFloat. */
+	if (charCode == '') {
+		close_window(topFloat);
+		return TRUE;
+	} 
+	/* Key commands for the player setup dialog. */ 
+	if (FrontWindow() == playersetupwin) {
+		handle_player_setup_event(event);
+		return TRUE;
+	}		
+	/* Key commands for modeless dialogs etc. */	
+	if (topFloat) {
+		closeup = unit_closeup_from_window(topFloat);
+		if (closeup) {
+			int result = do_key_down_closeup(closeup, keyCode);
+
+			/* We are done. Return TRUE to prevent further
+			handling by the dialog manager. */
+			if (result == DONE) {
+				return TRUE;
+			/* This was a key that should be handled by the
+			dialog manager. Return FALSE. */
+			} else if (result == TRUE) {
+				return FALSE;
+			/* The key returned FALSE and should therefore
+			be processed further below. */
+			}
+		} else if (topFloat == helpwin) {
+			handled = do_key_down_help(charCode);		
+		} else if (topFloat == researchwin) {
+			handled = do_key_down_research(charCode);		
+		}
+	}
+	/* Special use of the numeric keypad and arrow keys for 
+	scrolling of maps. */
+	if (!handled && (frontmap || worldmap)) {
+		handled = handle_numeric_keypad(event);
+	}
+	/* Key commands for normal windows. */
+	if (!handled && topWindow) {
+		list = list_from_window(topWindow);
+		if (list) {
+			handled = do_key_down_list(list, charCode);
+		} else	if (topWindow == constructionwin) {
+			handled = do_key_down_construction(charCode);
+		} else if (topWindow == commandwin) {
+			handled = do_key_down_command(charCode);
+		}
+	}
+	/* All other keys are interpreted as keyboard commands. */
+	if (!handled) {
+		do_keyboard_command(charCode);
+	}
+	return TRUE;
+}
+
 void
 get_global_mouse(Point *mouse)
 {
@@ -614,12 +707,16 @@ set_end_of_game_interaction_modes()
 
 Point lastmouse;
 
-char mouseoverbuf[100];
+char mouseoverbuf[BUFSIZE];
 
 /* Change the cursor to reflect the context. */
 
 CursHandle current_cursor;
 
+/* Delay before autoscrolling starts. */
+
+static int delay;
+
 void
 adjust_cursor(Point mouse, RgnHandle region)
 {
@@ -629,66 +726,116 @@ adjust_cursor(Point mouse, RgnHandle region)
 	Map *map;
 	GrafPtr oldport;
 
-	map = map_from_window(FrontWindow());
+	/* Work around of Appearance manager bug. Since clicks in the collapse box fails
+	to generate an application level event, and since the system fails to update dialogs
+	that are uncollapsed, it is necessary to force dialog updates in some other way. */
+	if (topFloat && IsDialog(topFloat) 
+	    && PtInRgn(mouse, ((WindowPeek) topFloat)->strucRgn) == TRUE
+	    && PtInRgn(mouse, ((WindowPeek) topFloat)->contRgn) == FALSE) {
+		update_window(topFloat);
+	}
+	map = frontmap;
 	if (map != NULL) {
 		GetPort(&oldport);
 		SetPort(map->window);
 		GlobalToLocal(&mouse);
-		if (mouse.h > conwid && mouse.h < map->window->portRect.right - sbarwid
-			&& mouse.v > map->toph && mouse.v < map->window->portRect.bottom - sbarwid) {
-			if (map_modal != NO_MODAL) {
-				switch (map_modal) {
-					case ATTACK_MODAL:
-					case FIRE_MODAL:
-					case FIRE_INTO_MODAL:
-						current_cursor = firecursor;
-						break;
-					case MOVE_TO_MODAL:
-					case SET_FORMATION_MODAL:
-					case ADD_TERRAIN_MODAL:
-					case REMOVE_TERRAIN_MODAL:
-					case DISTANCE_MODAL:
-					case ZOOM_MODAL:
-					case GENERIC_MODAL:
-						current_cursor = opencrosscursor;
-						break;
-					default:
-						run_error("unknown modal tool %d", map_modal);
-						break;
-				}
+
+		/* Don't do cursor adjustment for frontmap if we are in a floating window! */ 
+		if (PtInRgn(mouse, map->window->visRgn) != TRUE) {
+	  		SetCursor(&QD(arrow));
+			return;
+		}
+
+		if (mouse.h > map->conw 
+		    && mouse.v > map->toph 
+		    && mouse.h < map->window->portRect.right - map->sbarwid
+		    && mouse.v < map->window->portRect.bottom - map->sbarwid) {
+
+			if (mouse.h < map->conw + 40) {
+				current_cursor = leftcursor;				
 				usual = FALSE;
-#ifdef DESIGNERS
-			} else if (dside->designer && tooltype != notool) {
-				current_cursor = adjust_designer_cursor(mouse, region);  usual = FALSE;
-#endif DESIGNERS
-			} else if (map->moveonclick) {
-				if (map->numselections == 1
-				    && (unit = map->selections[0]) != NULL) {
-					/* Calculate the approx dir to here from selected unit. */
-					m_nearest_cell(map, mouse.h, mouse.v, &x, &y);
-					/* Note that we allow x,y here to be outside the world. */
-					if (mobile(unit->type)
-					    && (approxdir = approx_dir(x - unit->x, y - unit->y)) >= 0) {
-						current_cursor = movecursors[approxdir];  usual = FALSE;
+				/* Wait half second before autoscrolling. */
+				if (TickCount() > delay + 30)
+					beep();
+					scroll_map_window(map, -4, 0);
+			} else if (mouse.h > map->window->portRect.right - map->sbarwid - 40) {
+				current_cursor = rightcursor;
+				usual = FALSE;
+				/* Wait half second before autoscrolling. */
+				if (TickCount() > delay + 30)
+					beep();
+					scroll_map_window(map, 4, 0);
+			} else if (mouse.v < map->toph + 40) {
+				current_cursor = upcursor;
+				usual = FALSE;
+				/* Wait half second before autoscrolling. */
+				if (TickCount() > delay + 30)
+					beep();
+					scroll_map_window(map, 0, -4);
+			} else if (mouse.v > map->window->portRect.bottom - map->sbarwid - 40) {
+				current_cursor = downcursor;
+				usual = FALSE;
+				/* Wait half second before autoscrolling. */
+				if (TickCount() > delay + 30)
+					beep();
+					scroll_map_window(map, 0, 4);
+			} else	 {
+				/* Reset autoscrolling delay. */
+				delay = TickCount();		
+
+				if (map_modal != NO_MODAL) {
+					switch (map_modal) {
+						case ATTACK_MODAL:
+						case FIRE_MODAL:
+						case FIRE_INTO_MODAL:
+							current_cursor = firecursor;
+							break;
+						case MOVE_TO_MODAL:
+						case SET_FORMATION_MODAL:
+						case ADD_TERRAIN_MODAL:
+						case REMOVE_TERRAIN_MODAL:
+						case DISTANCE_MODAL:
+						case ZOOM_MODAL:
+						case GENERIC_MODAL:
+							current_cursor = opencrosscursor;
+							break;
+						default:
+							run_error("unknown modal tool %d", map_modal);
+							break;
+					}
+					usual = FALSE;
+	#ifdef DESIGNERS
+				} else if (dside->designer && tooltype != notool) {
+					current_cursor = adjust_designer_cursor(mouse, region);  usual = FALSE;
+	#endif DESIGNERS
+				} else if (map->moveonclick) {
+					if (map->numselections == 1
+					    && (unit = map->selections[0]) != NULL) {
+						/* Calculate the approx dir to here from selected unit. */
+						m_nearest_cell(map, mouse.h, mouse.v, &x, &y);
+						/* Note that we allow x,y here to be outside the world. */
+						if (mobile(unit->type)
+						    && (approxdir = approx_dir(x - unit->x, y - unit->y)) >= 0) {
+							current_cursor = movecursors[approxdir];  usual = FALSE;
+						} else {
+							current_cursor = nomovecursor;  usual = FALSE;
+						}
+					} else if (map->numselections > 1) {
+						current_cursor = allmovecursor;  usual = FALSE;
 					} else {
-						current_cursor = nomovecursor;  usual = FALSE;
+						/* (this is a little confusing here if no units are selected, since
+					   	will just be arrow cursor) */
 					}
-				} else if (map->numselections > 1) {
-					current_cursor = allmovecursor;  usual = FALSE;
-				} else if (map->autoselect) {
-					/* If no units are selected and autoselect is on, then select one unit.
-					   In theory this should never happen, but in practice it does... */
-					map->curunit = find_next_awake_mover(dside, NULL);
-					select_exactly_one_unit(map, map->curunit);
-				} else {
-					/* (this is a little confusing here if no units are selected, since
-				   	will just be arrow cursor) */
 				}
 			}
 			/* This isn't really "cursor adjustment", but this is the right place
 			   to do it - change the topline of the map to indicate what the cursor
 			   is over. */
-			if (map->toplineh > 0 && !EqualPt(mouse, lastmouse)) {
+			/* Scrolling, zooming or changing angle alters mouseover even though
+			   mouse == lastmouse. */
+			/* (should flag the change of state and or that with the !EqualPt,
+			   for efficiency) */
+			if (map->toplineh > 0 /* && !EqualPt(mouse, lastmouse) */ ) {
 				oneliner(dside, map->vp, mouse.h - map->conw, mouse.v - map->toph);
 				if (strcmp(tmpbuf, mouseoverbuf) != 0) {
 					strcpy(mouseoverbuf, tmpbuf);
@@ -698,6 +845,8 @@ adjust_cursor(Point mouse, RgnHandle region)
 				lastmouse = mouse;
 			}
 		} else {
+			/* Reset autoscrolling delay. */
+			delay = TickCount();
 			if (map->toplineh > 0) {
 				if (mouseover != NULL) {
 					mouseover = NULL;
@@ -726,14 +875,13 @@ void
 handle_event(EventRecord *event)
 {
 	short part, err;
-	WindowPtr win;
-	char key;
+	WindowPtr win, win2;
 	Point pnt;
 
 	switch (event->what) {
 		case mouseDown:
 			/* See if the click happened in a special part of the screen. */
-			part = FindWindow(event->where, &win);
+			part = MacFindWindow(event->where, &win);
 			switch (part) {
 				case inMenuBar:
 					adjust_menus();
@@ -743,12 +891,18 @@ handle_event(EventRecord *event)
 					SystemClick(event, win);
 					break;
 				case inContent:
-					if (win != FrontWindow()) {
+					if (win != topWindow && win != topFloat) {
 						/* Bring the clicked-on window to the front. */
-						SelectWindow(win);
+						SelectTheWindow(win);
 						update_window(win);
 						/* Fix the menu to match the new front window. */
 						adjust_menus();
+						/* Permit direct click action in non-top floating windows.
+						This is not exactly in line with Apple's interface guidelines
+						but it does make things easier when more than one float is up
+						since you don't have to worry about which one is in front. */
+						if (IsFloating(win))
+						    do_mouse_down(win, event);
 						/* We always want to discard the event now, since clicks in a
 						   windows are often irreversible actions. */
 					} else {
@@ -757,8 +911,7 @@ handle_event(EventRecord *event)
 					}
 					break;
 				case inDrag:
-					/* Standard drag behavior, no tricks necessary. */
-					DragWindow(win, event->where, &dragrect);
+					drag_window(win, event);
 					break;
 				case inGrow:
 					grow_window(win, event->where);
@@ -771,41 +924,21 @@ handle_event(EventRecord *event)
 					if (TrackGoAway(win, event->where))
 					  close_window(win);
 					break;
+				case inCollapseBox:
+					/* Uncollapsed dialogs don't get redrawn by the system. */ 
+					if (IsDialog(win)) {
+					    EraseRect(&win->portRect);
+					    force_update(win);
+					}
+					break;					
 			}
 			break;
+#if 0					/* Now handled elsewhere. */
 		case keyDown:
 		case autoKey:
-			/* Don't accept keystrokes if no windows up. */
-			if (!FrontWindow())
-			  break;
-
-			/* First handle the special use of the numeric keypad keys for offscreen scrolling */
-			/* Break if a numeric keypad key was hit to prevent reading of its charCode below */
-			if (handle_numeric_keypad(event))
-			  break;
-
-			key = event->message & charCodeMask;
-			/* Check for menukey equivalents. */
-			if (event->modifiers & cmdKey) {
-				if (event->what == keyDown) {
-					adjust_menus();
-					do_menu_command(MenuKey(key));
-				}
-			} else {
-				if (event->what == keyDown) {
-					int handled = FALSE;
-
-					/* Random keypress, interpret it. */
-					if (FrontWindow() == constructionwin)
-					  handled = do_key_down_construction(key);
-					if (FrontWindow() == commandwin)
-					  handled = do_key_down_command(key);
-					if (!handled)
-					  do_keyboard_command(key);
-				}
-			}
-
+			handle_keystroke(event);
 			break;
+#endif
 		case activateEvt:
 			activate_window((WindowPtr) event->message, event->modifiers & activeFlag);
 			break;
@@ -820,14 +953,24 @@ handle_event(EventRecord *event)
 				err = DIBadMount(pnt, event->message);
 			}
 			break;
-		case app4Evt:
+		case osEvt:
 		    /* Grab only a single byte. */
 			switch ((event->message >> 24) & 0xFF) {
+				/* Mouse moved evt - do nothing. */
 				case 0xfa:
 					break;
+				/* Suspend/resume evt. */
 				case 1:
+					/* Check for suspend (0) or resume (1). */
 					inbackground = !(event->message & 1);
-					activate_window(FrontWindow(), !inbackground);
+					/* Activate or deactivate the top window. */
+					ActivateWindow(topWindow, !inbackground);
+					/* The floats should also be activated/deactivated! */
+					for_all_windows(win) {
+						if (IsFloating(win) || IsDialog(win)) {
+							ActivateWindow(win, !inbackground);
+						}							
+					}
 					break;
 			}
 			break;
@@ -847,6 +990,14 @@ handle_event(EventRecord *event)
 #endif
 }
 
+void
+drag_window(WindowPtr win, EventRecord *event)
+{
+	DragTheWindow(win, event);
+	/* Necessary to redraw some modeless dialogs. */
+	update_all_visible_dialogs();
+}
+
 /* Handle window growing by mindlessly tracking via GrowWindow,
    then passing the chosen size to specific window resize handlers
    or else doing the generic resize. */
@@ -878,6 +1029,8 @@ grow_window(WindowPtr win, Point where)
 			grow_scores(winh, winv);
 		}
 		SetPort(oldport);
+		/* Necessary to redraw some modeless dialogs. */
+		update_all_visible_dialogs();
 	}
 }
 
@@ -907,10 +1060,12 @@ zoom_window(WindowPtr win, Point where, int part)
 		} else {
 			/* Generic window zooming. */
 			EraseRect(&win->portRect);
-			ZoomWindow(win, part, true);
+			ZoomWindow(win, part, false);
 			InvalRect(&win->portRect);
 		}
 		SetPort(oldport);
+		/* Necessary to redraw some modeless dialogs. */
+		update_all_visible_dialogs();
 	}
 }
 
@@ -929,17 +1084,14 @@ close_window(WindowPtr win)
 			destroy_list(list_from_window(win));
 		} else if (unit_closeup_from_window(win)) {
 			destroy_unit_closeup(unit_closeup_from_window(win));
-#ifdef DESIGNERS
-		} else if (win == designwin) {
-			/* Closing the design palette implies we're done designing. */
-			disable_designing();
-#endif /* DESIGNERS */
 		}
 		/* Remove the window from our sight, will provoke update events. */
-		HideWindow(win);
+		if (win)
+		HideTheWindow(win);
 		/* At least for now, don't actually dispose of the window. */
 	}
-	update_all_map_windows();
+	/* Necessary to redraw some modeless dialogs. */
+	update_all_visible_dialogs();
 }
 
 /* This just dispatches to the appropriate window handler. */
@@ -977,6 +1129,8 @@ do_mouse_down(WindowPtr window, EventRecord *event)
 			do_mouse_down_command(mouse, event->modifiers);
 		} else if (window == scoreswin) {
 			do_mouse_down_scores(mouse, event->modifiers);
+		} else if (window == instructionswin) {
+			do_mouse_down_instructions(mouse, event->modifiers);
 #ifdef DESIGNERS
 		} else if (window == designwin) {
 			do_mouse_down_design(mouse, event->modifiers);
@@ -1010,6 +1164,8 @@ activate_window(WindowPtr win, int activate)
 		activate_construction(activate);
 	} else if (win == helpwin) {
 		activate_help(activate);
+	} else if (win == instructionswin) {
+		activate_instructions(activate);
 	} else if (win == noticewin) {
 		activate_notice(activate);
 	} else if (win == commandwin) {
@@ -1048,6 +1204,9 @@ update_window(WindowPtr win)
 		  draw_unit_info(map);
 		/* Then copy map content from the gworld. */
 		copy_from_gworld(map, map->contentrect);	   
+		/* Don't draw scrollbars if they are zero width. */
+		if (map->sbarwid == 0)
+	             controls = FALSE;
 		growbox = TRUE;
 	} else if ((list = list_from_window(win)) != NULL) {
 		draw_list(list);
@@ -1066,6 +1225,9 @@ update_window(WindowPtr win)
 	} else if (win == helpwin) {
 		draw_help();
 		growbox = TRUE;
+	} else if (win == instructionswin) {
+		draw_instructions();
+		growbox = FALSE;
 	} else if (win == noticewin) {
 		draw_notice();
 		growbox = TRUE;
@@ -1075,6 +1237,11 @@ update_window(WindowPtr win)
 	} else if (win == scoreswin) {
 		draw_scores();
 		growbox = TRUE;
+	} else if (win == researchwin) {
+		/* Update events don't always work with dialogs, but setting the force 
+		flag forces redrawing. */
+		draw_research_dialog(true);
+		controls = FALSE;
 #ifdef DESIGNERS
 	} else if (win == designwin) {
 		draw_design_window();
@@ -1087,12 +1254,47 @@ update_window(WindowPtr win)
 		UpdateControls(win, win->visRgn);
 	}
 	if (growbox) {
-		DrawGrowIcon(win);
+		/* We want scrollbar lines only when there are scrollbars! */
+		if(controls)
+			DrawGrowIcon(win);
+		else 	draw_growicon_nolines(win);
 	}
 	EndUpdate(win);
 	SetPort(oldport);
 }
 
+/* Fixes a bug in the Toolbox which causes DrawGrowIcon to draw scrollbar
+lines even if there are no scrollbars in the window. */
+
+void 
+draw_growicon_nolines(WindowPtr win)
+{
+	Rect 		iconrect;
+	RgnHandle 	newclip;
+	RgnHandle 	oldclip;
+	
+	newclip = NewRgn();
+	oldclip = NewRgn();
+	GetClip(oldclip);
+
+	/* Set new clip rgn to GrowIcon. */
+	iconrect = win->portRect;
+	iconrect.left = iconrect.right-15; 
+	iconrect.top = iconrect.bottom-15;
+	SetRectRgn(newclip, iconrect.left, iconrect.top, 
+				       iconrect.right, iconrect.bottom);
+
+	/* Intersect it with old clip rgn. */
+	SectRgn(oldclip, newclip, newclip);
+
+	SetClip(newclip);
+	DrawGrowIcon(win);
+	SetClip(oldclip);
+	DisposeRgn(newclip);
+	DisposeRgn(oldclip);
+
+}
+
 static int last_tick_count = 0;
 
 void
@@ -1102,74 +1304,96 @@ maybe_select_next_unit()
 	Map *map;
 
 	if ((!beforestart && !endofgame)
-	    && (map = map_from_window(FrontWindow())) != NULL
+	    && (map = frontmap) != NULL
 	    && map->autoselect) {
-	    /* Hunt around for a reasonable "next unit" to select. */
-	    /* Scroll over to the current unit if appropriate. */
-	    if (map->curunit != NULL
-	    	&& in_play(map->curunit)
-			&& (map->curunit->act
-				&& map->curunit->act->acp > 0)  /* should be "above min"? */
-	    	&& (map->curunit->plan
-				&& !map->curunit->plan->asleep
-				&& !map->curunit->plan->reserve
-				&& !map->curunit->plan->delayed)
-	    	) {
-	    	if (map->scrolltocurunit) {
-				scroll_to_unit(map, map->curunit);
-				map->scrolltocurunit = FALSE;
+		    /* Hunt around for a reasonable "next unit" to select. */
+		    /* Scroll over to the current unit if appropriate. */
+		    if (map->curunit != NULL
+		    	&& in_play(map->curunit)
+			&& has_acp_left(map->curunit)
+		    	&& (map->curunit->plan
+					&& !map->curunit->plan->asleep
+					&& !map->curunit->plan->reserve
+					&& !map->curunit->plan->delayed)
+		    	) {
+			/* Make sure the current unit is selected if on autoselect. */
+		    	if (!unit_is_selected(map, map->curunit)) {
+		    		select_unit_on_map(map, map->curunit);
+		    	}
+		    	if (map->scrolltocurunit) {
+					scroll_to_unit(map, map->curunit);
+					map->scrolltocurunit = FALSE;
+				}
+		    	goto blink;
+		    }
+			unit = autonext_unit_inbox(dside, map->curunit, map->vp);
+			if (unit
+				&& unit->plan
+				&& !unit->plan->asleep
+				&& !unit->plan->reserve
+				&& !unit->plan->delayed
+				&& unit->plan->waitingfortasks
+				) {
+				map->curunit = unit;
+				select_exactly_one_unit(map, map->curunit);
+				goto blink;
 			}
-	    	goto blink;
-	    }
-		unit = autonext_unit_inbox(dside, map->curunit, map->vp);
-		if (unit
-			&& unit->plan
-			&& !unit->plan->asleep
-			&& !unit->plan->reserve
-			&& !unit->plan->delayed
-			&& unit->plan->waitingfortasks
-			) {
-			map->curunit = unit;
-			select_exactly_one_unit(map, map->curunit);
-			goto blink;
-		}
-		/* Look for the next unit. */
-		unit = find_next_awake_mover(dside, map->curunit);
-		if (unit
-			&& unit->plan
-			&& !unit->plan->asleep
-			&& !unit->plan->reserve
-			&& !unit->plan->delayed
-			&& unit->plan->waitingfortasks
-			) {
-			map->curunit = unit;
-			select_exactly_one_unit(map, map->curunit);
-			goto blink;
-		}
-		/* Start over from beginning of list. */
-		unit = find_next_awake_mover(dside, NULL);
-		if (unit
-			&& unit->plan
-			&& !unit->plan->asleep
-			&& !unit->plan->reserve
-			&& !unit->plan->delayed
-			&& unit->plan->waitingfortasks
-			) {
-			map->curunit = unit;
-			select_exactly_one_unit(map, map->curunit);
-			goto blink;
-		}
-	  blink:
-	    if (map->curunit != NULL) {
-	        int tick_count;
+			/* Look for the next unit. */
+			unit = find_next_awake_mover(dside, map->curunit);
+			if (unit
+				&& unit->plan
+				&& !unit->plan->asleep
+				&& !unit->plan->reserve
+				&& !unit->plan->delayed
+				&& unit->plan->waitingfortasks
+				) {
+				map->curunit = unit;
+				select_exactly_one_unit(map, map->curunit);
+				goto blink;
+			}
+			/* Start over from beginning of list. */
+			unit = find_next_awake_mover(dside, NULL);
+			if (unit
+				&& unit->plan
+				&& !unit->plan->asleep
+				&& !unit->plan->reserve
+				&& !unit->plan->delayed
+				&& unit->plan->waitingfortasks
+				) {
+				map->curunit = unit;
+				select_exactly_one_unit(map, map->curunit);
+				goto blink;
+			}
+		blink:
+		if (map->curunit != NULL) {
+			int tick_count = TickCount();
+			    
+			/* Use blinking current unit if asked to. */
+			if (map->blinking_curunit == TRUE
+			      && tick_count - last_tick_count >= 20) {
+				int curcp = map->curunit->cp;
 
-			tick_count = TickCount();
-	    	if (tick_count - last_tick_count >= 10) {
-	    		last_tick_count = tick_count;
-	    	    animation_pattern_state = (animation_pattern_state + 1) % 8;
-	    	    draw_selection_animation(map, map->curunit);
-	    	}
-	    }
+				last_tick_count = tick_count;
+				/* Make sure we have a nonzero state that can switch sign. */
+		    		if (animation_pattern_state == 0)
+				    animation_pattern_state = -1;
+				/* Blink. */
+				animation_pattern_state = -animation_pattern_state;	    	    
+				/* Set cp to bogus zero in order to force drawing as a gray
+				incomplete unit. */
+				if (animation_pattern_state < 0)
+				    map->curunit->cp = 0;
+				update_cell(map, map->curunit->x, map->curunit->y);
+				/* Restore correct cp. */
+				map->curunit->cp = curcp;
+			/* Else use animated selection box. */
+			} else if (map->blinking_curunit != TRUE
+				      && tick_count - last_tick_count >= 10) {
+					last_tick_count = tick_count;
+				    animation_pattern_state = (animation_pattern_state + 1) % 8;
+				    draw_selection_animation(map, map->curunit);
+			}
+		}
 	}
 }
 
@@ -1307,11 +1531,11 @@ connect_game_dialog()
 
 	hosting = FALSE;
 	connection_method_dialog();
-	open_remote_connection(connection_method_name);
+	open_remote_connection(connection_method_name, hosting);
 	if (connection_method > 0) {
 		rslt = send_join("mac");
 		if (!rslt) {
-			close_remote_connection();
+			close_remote_connection(0);
 			/* Use of connection_method this way is weak; means
 			   having to reselect everything up reconnection. */
 			connection_method = 0;
@@ -1605,32 +1829,57 @@ low_notify(Side *side, char *str)
 
 /* Kernel callback to update info about the given side. */
 
+void
+update_area_display(Side *side)
+{
+}
+
 void
 update_side_display(Side *side, Side *side2, int rightnow)
 {
 	GrafPtr oldport;
 	extern int gamenumsides;
 
-	if (active_display(side) && side2 != NULL) {
-		GetPort(&oldport);
-		if (gamewin != nil && ((WindowPeek) gamewin)->visible) {
+	if (active_display(side)) {
+		if (gamewin != nil
+		    && side2 != indepside 
+		    && ((WindowPeek) gamewin)->visible) {
 			if (gamenumsides == numsides) {
+				GetPort(&oldport);
 				SetPort(gamewin);
 				draw_side_status(side2);
+				SetPort(oldport);		
 			} else {
 				/* A side must have been added recently; redraw
 				   the entire window this time. */
 				if (eimages[side_number(side2)] == NULL)
-				  init_emblem_images(side2);
+				  init_emblem_images();
 				draw_game();
 			}
 		}
-		SetPort(oldport);
+		/* Handle the research window if necessary. */
+		if (numatypes > 0 && g_side_can_research()) {
+			/* Research is under human control and we need a new topic. */
+			if (!side_has_ai(dside) && dside->research_topic == NOADVANCE) {
+				 if (dside->autoresearch) {
+					auto_pick_side_research(dside);
+					if (researchwin)
+					    /* We call draw_research_dialog here passing false instead of 
+					    update_window, which would force redrawing. */
+					    draw_research_dialog(false);
+				/* Pick new topic manually. */
+				} else {
+					show_research_dialog();
+				}
+			/* Always update the window, but only if it exists. */
+			} else if (researchwin) {
+				draw_research_dialog(false);
+			}
+		}
 		if (side2 == dside && !side->ingame && side->status == 0 && wasingame) {
 			/* (should be able to quit from here?) */
 			CautionAlert(aOutOfGame, nil); 
 			wasingame = FALSE;
-			dside->may_set_see_all = TRUE;
 		}
 	}
 }
@@ -1661,13 +1910,12 @@ update_turn_display(Side *side, int rightnow)
 		}
 		/* This routine might have been called because the game is over. */
 		if (endofgame && !told_outcome && side == dside) {
-		    side->may_set_see_all = TRUE;
 		    if (!side->see_all) {
-		    	side->see_all = TRUE;
 		    	/* Can't do update forcing here because modal dialogs will
 		    	   come up before windows actually get to update. */
 		    	for_all_maps(map) {
 					map->see_all = TRUE;
+					map->vp->show_all = TRUE;
 					draw_map(map);
 				}
 		    }
@@ -1854,17 +2102,17 @@ update_fire_into_display(Side *side, Unit *unit, int x2, int y2, int z2, int m,
 void
 update_unit_display(Side *side, Unit *unit, int rightnow)
 {
-	UnitCloseup *unitcloseup;
+	UnitCloseup *unitcloseup = find_unit_closeup(unit);
 
 	if (active_display(side) && unit != NULL) {
 		update_unit_in_maps(unit);
 		if (1 /* unit visible to side in any way */ && inside_area(unit->x, unit->y)) {
-			update_cell_display(side, unit->x, unit->y, TRUE);
+			update_cell_display(side, unit->x, unit->y, UPDATE_ALWAYS);
 		}
 		update_unit_in_lists(unit);
-		if ((unitcloseup = find_unit_closeup(unit)) != NULL
-		    && 1 /* window is visible */) {
-			draw_unit_closeup(unitcloseup);
+		if (unitcloseup
+		    && ((WindowPeek) (unitcloseup->window))->visible) {
+			update_window(unitcloseup->window);
 		}
 		if (unit->side != NULL && unit->act != NULL) {
 			update_side_display(side, unit->side, rightnow);
@@ -1885,13 +2133,14 @@ update_unit_acp_display(Side *side, Unit *unit, int rightnow)
 		update_unit_in_maps(unit);
 #if 0  /* maybe add later, maybe not - acp change not usually visible tho */
 		if (1 /* unit visible to side in any way */ && inside_area(unit->x, unit->y)) {
-			update_cell_display(side, unit->x, unit->y, TRUE);
+			update_cell_display(side, unit->x, unit->y, UPDATE_ALWAYS);
 		}
 #endif
 		update_unit_in_lists(unit);
 		unitcloseup = find_unit_closeup(unit);
-		if (unitcloseup != NULL && (((WindowPeek) (unitcloseup->window))->visible)) {
-			draw_unit_closeup(unitcloseup);
+		if (unitcloseup
+		    && ((WindowPeek) (unitcloseup->window))->visible) {
+			update_window(unitcloseup->window);
 		}
 	}
 }
@@ -2242,13 +2491,8 @@ play_movies(SideMask sidemask)
 					  continue;
 					blasttype = (movies[j].itype == movie_miss ? 0 :
 								 (movies[j].itype == movie_hit ? 1 : 2));
-					for_all_maps(map) {
-						if (curport != map->window) {
-							SetPort(map->window);
-							curport = map->window;
-						}
-						clear_unit_blast(map, unit, blasttype);
-					}
+					for_all_maps(map)
+					  update_cell(map, unit->x, unit->y);
 					break;
 				default:
 					break;
@@ -2299,6 +2543,11 @@ stagger_window(WindowPtr win, int *lasthp, int *lastvp)
 
 	if (*lasthp > 0 && *lastvp > 0) {
 		while (1) {
+			/* Prevent excessive staggering of new windows. */
+			if (!position_already_used(*lasthp, *lastvp)) {
+				h = *lasthp;  v = *lastvp;
+				break;
+			}
 			h = *lasthp + 20;  v = *lastvp + 20;
 			/* Let windows go partly off the screen, but keep at least the top
 			   40x40 pixels visible. */
@@ -2314,10 +2563,15 @@ stagger_window(WindowPtr win, int *lasthp, int *lastvp)
 			} else if (!position_already_used(h, v)) {
 				break;
 			}
+			/* Moving this inside the while loop fixed infinite loop hangs with certain values. */
+			*lasthp = h;  *lastvp = v;
 		}
 		MoveWindow(win, h, v, FALSE);
-		*lasthp = h;  *lastvp = v;
 	} else {
+		/* Do move to accomodate Apple Platinum frames (would be more correct 
+		to use strucRgn->rgnBBox here, but it is not yet defined at this point). */
+		MoveWindow(win, 5, GetMBarHeight() + 21, FALSE);
+
 		/* Don't move the first window, but do record its position. */
 		GetPort(&oldport);
 		SetPort(win);
@@ -2418,7 +2672,9 @@ set_standard_state(WindowPtr win, int fullw, int fullh)
 		if (bestscreen == GetMainDevice()) {
 			gdrect.top += GetMBarHeight();
 		}
-		InsetRect(&gdrect, 3, 3);
+		/* Modified for Apple Platinum. */
+		InsetRect(&gdrect, 5, 5);
+		gdrect.top -= 5;
 		gdrect.top += wintitlehgt;
 		screenw = gdrect.right - gdrect.left;  screenh = gdrect.bottom - gdrect.top;
 		if (winrect.left + fullw <= gdrect.right
@@ -2508,8 +2764,8 @@ force_update(WindowPtr win)
 	map = map_from_window(win);
 	if (map) {
 		/* Do this stuff first so that the user has something to look at. */
-		set_map_scrollbars(map);
 		set_content_rect(map);
+		set_map_scrollbars(map);
 
 		/* Draw the map decor. */
 		if (map->conw > 0)
@@ -2521,9 +2777,10 @@ force_update(WindowPtr win)
 
  		update_gworld(map);
 	} else {
-		/* For non-maps, erase and invalidate everything. */
-		EraseRect(&win->portRect);
+		/* For non-maps invalidate and update the window. Don't EraseRect
+		since it messes up the modeless dialogs. */
 		InvalRect(&win->portRect);
+		update_window(win);
 	}
 	SetPort(oldport);
 }
@@ -2543,7 +2800,7 @@ force_overall_update()
 	force_update(commandwin);
 	force_update(scoreswin);
 	for_all_maps(map) {
-		force_update(map->window);
+		force_map_update(map);
 	}
 	for_all_lists(list) {
 		force_update(list->window);
@@ -2591,7 +2848,18 @@ set_game_file_type(char *name)
 void
 exit_macconq()
 {
-	close_remote_connection();
+	close_remote_connection(0);
+
+#if __profile__
+
+	/* Make sure we use the Xconq directory. */
+	SetVol(NULL, initialvrefnum);
+	/* Dump any unsaved profiler output for CodeWarrior. */
+	ProfilerDump("\pXconq.Profile");
+	ProfilerTerm();
+
+#endif
+
 	ExitToShell();
 }
 
@@ -2640,3 +2908,12 @@ __ttyname(long fd)
 
 #endif /* __MWERKS__ */
 
+void
+add_remote_locally(int rid, char *str)
+{
+}
+
+void
+send_chat(int rid, char *str)
+{
+}
