@@ -1,5 +1,5 @@
 /* Definitions of all the resources for the Mac interface to Xconq.
-   Copyright (C) 1992-1997 Stanley T. Shebs.
+   Copyright (C) 1992-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -8,6 +8,10 @@ any later version.  See the file COPYING.  */
 
 #define XconqSignature 'XCNQ'
 
+#define OkButton 1
+#define CancelButton 2
+#define HelpButton 3
+
 /* The menu bar. */
 
 #define mbMain 128
@@ -48,15 +52,14 @@ any later version.  See the file COPYING.  */
 #define mViewGrids 216
 #define mViewColors 217
 #define mViewBorders 218
+#define mPlanTypePopup 240
 
 #define mPreferencesTopics 700
 #define mSmallFont 701
 #define mSmallSize 702
 #define mLargeFont 703
 #define mLargeSize 704
-
 #define mSideColorPopup 705
-
 #define mSideColorUse 706
 
 #define miNotInUse 1
@@ -67,6 +70,11 @@ any later version.  See the file COPYING.  */
 #define mSidesPopup 707
 #define mFeatPopup 708
 
+#define mBuildPopup 710
+#define mResearchPopup 711
+
+#define mAdvanceAchieved 713
+
 /* Menu items. */
 
 #define miAppleAbout 1
@@ -75,9 +83,9 @@ any later version.  See the file COPYING.  */
 
 #define miFileNew 1
 #define miFileOpen 2
-/* 3 */
-#define miFileConnect 4
-/* 5 */
+#define miFileConnect 3
+/* 4 */
+#define miFileClose 5
 #define miFileSave 6
 #define miFileSaveAs 7
 /* 8 */
@@ -112,7 +120,7 @@ any later version.  See the file COPYING.  */
 #define miFindSelected 9
 
 #define miPlayCloseup 1
-/* 2 */
+#define miPlayCityDialog 2
 #define miPlayMove 3
 #define miPlayReturn 4
 #define miPlaySetFormation 5
@@ -136,8 +144,8 @@ any later version.  See the file COPYING.  */
 #define miMoreEmbark 3
 #define miMoreDisembark 4
 #define miMoreDrop 5
-#define miMoreGiveUnit 6
-#define miMoreDetach 7
+/* 6 */				/* Can't have submenu at position 5 or 6 for some weird reason! */
+#define miMoreDetach 7	/* Problem now fixed. See macmenus.c for details. */
 #define miMoreDisband 8
 /* 9 */
 #define miMoreAddTerrain 10
@@ -147,6 +155,7 @@ any later version.  See the file COPYING.  */
 #define miMoreAIControl 14
 /* 15 */
 #define miMoreRename 16
+#define miMoreGiveUnit 17
 
 #define miSideCloseup 1
 #define miSideFinishedTurn 2
@@ -158,11 +167,11 @@ any later version.  See the file COPYING.  */
 #define miSideSound 8
 /* 9 */
 /* 10 is AI Type submenu */
-/* 11 */
-#define miSideDoctrine 12
-/* 13 */
-#define miSideSendMessage 14
-/* 15 */
+#define miSideAutoResearch 11
+/* 12 */
+#define miSideDoctrine 13
+/* 14 */
+#define miSideSendMessage 15
 #define miSideRename 16
 
 #define miWindowsGame 1
@@ -170,15 +179,16 @@ any later version.  See the file COPYING.  */
 #define miWindowsHistory 3
 #define miWindowsConstruction 4
 #define miWindowsCommand 5
-#define miWindowsAgreements 6
+#define miWindowsResearch 6
 #define miWindowsScores 7
-/* 8 */
-#define miWindowsNewMap 9
-#define miWindowsNewList 10
+#define miWindowsDesign 8
+#define miWindowsAgreements 9
+#define miWindowsWorldMap 10
 /* 11 */
-#define miWindowsWorldMap 12
-/* 13 */
-#define miWindowsFirst 14
+#define miWindowsNewMap 12
+#define miWindowsNewList 13
+/* 14 */
+#define miWindowsFirst 15
 
 #define miViewCloser 1
 #define miViewFarther 2
@@ -195,12 +205,14 @@ any later version.  See the file COPYING.  */
 #define miViewDaylight 12
 #define miViewCoverage 13
 /* 14 */
-
 #define miViewText 15
 #define miViewGrids 16
 #define miViewColors 17
 #define miViewBorders 18
-#define miViewOffscreen 19
+/* 19 */
+#define miViewOffscreen 20
+#define miViewMeridians 21
+#define miViewWorldMap 22
 
 #define miViewByType 1
 #define miViewByName 2
@@ -215,6 +227,7 @@ any later version.  See the file COPYING.  */
 /* 11 */
 #define miViewIconSize 12
 #define miListSideColors 13
+#define miListDrawEmblems 14
 
 #define miAngle15 1
 #define miAngle30 2
@@ -229,7 +242,9 @@ any later version.  See the file COPYING.  */
 #define miPlanTypeDefensive 3
 #define miPlanTypeExploratory 4
 #define miPlanTypeOffensive 5
-#define miPlanTypeRandom 6
+#define miPlanTypeColonizing 6
+#define miPlanTypeImproving 7
+#define miPlanTypeRandom 8
 
 #define miPreferencesTopicsFiles 1
 #define miPreferencesTopicsMap 2
@@ -239,18 +254,22 @@ any later version.  See the file COPYING.  */
 #define miTextFeatureNames 2
 #define miTextOptimizeFonts 3
 #define miTextEraseNames 4
+#define miTextDrawEmblems 5
 
 #define miViewGrid 1
-#define miDrawLatLong 2
-#define miViewTopline 3
-#define miViewTopunit 4
-#define miViewOtherMaps 5
+#define miViewTopline 2
+#define miViewTopunit 3
+#define miViewScrollBars 4
+#define miViewControlPanel 5
+#define miViewOtherMaps 6
 
 #define miSideColors 1
 #define miIconMasks 2
 #define miBoxMasks 3
 #define miTextMasks 4
 #define miSolidColorTerrain 5
+#define miBlinkingCurunit 6
+#define miSelectionMasks 7
 
 #define miFeatureBorders 1
 #define miShoreLines 2
@@ -312,6 +331,7 @@ any later version.  See the file COPYING.  */
 #define diPlayerSetupRemote 9
 #define diPlayerSetupExchange 10
 #define diPlayerSetupAdvantageRange 14
+#define diPlayerSetupConfigure 15
 
 #define dVariants 131
 
@@ -321,14 +341,21 @@ any later version.  See the file COPYING.  */
 #define diVariantsWorldSeen 4
 #define diVariantsSeeAll 5
 #define diVariantsSequential 6
-#define diVariantsFirstCheckBox 7
+#define diVariantsPeople 7
+#define diVariantsEconomy 8
+#define diVariantsSupply 9
+#define diVariantsIndepsAI 10
+#define diVariantsIndepsBuild 11
+#define diVariantsIndepsResearch 12
+#define diVariantsIndepsTreasury 13
+#define diVariantsFirstCheckBox 14
 /* and next 6 also */
-#define diVariantsFirstSlider 14
+#define diVariantsFirstSlider 20
 /* and next 2 also */
-#define diVariantsWorldSize 17
-#define diVariantsRealTime 18
-#define diVariantsMoreVariants 19
-#define diVariantsHelp 20
+#define diVariantsWorldSize 23
+#define diVariantsRealTime 24
+#define diVariantsMoreVariants 25
+#define diVariantsHelp 26
 
 #define dProgress 132
 
@@ -368,6 +395,19 @@ any later version.  See the file COPYING.  */
 #define diMoreVariantsOK 1
 #define diMoreVariantsCancel 2
 
+#define dScreenDepth 137
+
+#define diScreenDepthOK 1
+#define diScreenDepthCancel 2
+
+#define dConfigureSide 138
+
+#define diConfigureSideOK 1
+#define diConfigureSideCancel 2
+#define diConfigureSideBuild 3
+#define diConfigureSideResearch 4
+#define diConfigureSideTreasury 5
+
 #define dPreferences 700
 
 #define diPrefsOK 1
@@ -395,7 +435,7 @@ any later version.  See the file COPYING.  */
 #define diPrefsNames (diPrefsCommon + 2)
 #define diPrefsFeatureNames (diPrefsCommon + 3)
 #define diPrefsFeatureBorders (diPrefsCommon + 4)
-#define diPrefsLatLong (diPrefsCommon + 5)
+#define diPrefsMeridians (diPrefsCommon + 5)
 #define diPrefsIncrText (diPrefsCommon + 6)
 #define diPrefsIncr (diPrefsCommon + 7)
 #define diPrefsSideColors (diPrefsCommon + 8)
@@ -412,6 +452,9 @@ any later version.  See the file COPYING.  */
 #define diPrefsBoxMasks (diPrefsCommon + 19)
 #define diPrefsTextMasks (diPrefsCommon + 20)
 #define diPrefsMagPopup (diPrefsCommon + 26)
+#define diPrefsAIRegions (diPrefsCommon + 27)
+#define diPrefsScrollBars (diPrefsCommon + 28)
+#define diPrefsControlPanel (diPrefsCommon + 29)
 
 #define dsPreferencesFonts 703
 
@@ -436,6 +479,7 @@ any later version.  See the file COPYING.  */
 #define diPrefsUseVertiBuff (diPrefsCommon + 4)
 #define diPrefsVertiBuff (diPrefsCommon + 5)
 #define diPrefsVPixel (diPrefsCommon + 6)
+#define diPrefs256colors (diPrefsCommon + 10)
 
 #define dsPreferencesImageSets 705
 
@@ -444,6 +488,11 @@ any later version.  See the file COPYING.  */
 #define diPrefsColorIconsText (diPrefsCommon + 3)
 #define diPrefsUnifiedCicns (diPrefsCommon + 4)
 #define diPrefsUnifiedText (diPrefsCommon + 5)
+#define diPrefsDrawEmblems (diPrefsCommon + 6)
+#define diPrefsBlinkingUnit (diPrefsCommon + 8)
+#define diPrefsAnimatedBox (diPrefsCommon + 9)
+#define diPrefsSelectionMasks (diPrefsCommon + 10)
+#define diPrefsSelectionBoxes (diPrefsCommon + 11)
 
 #define dsPreferencesColorSchemes 706
 
@@ -519,11 +568,6 @@ any later version.  See the file COPYING.  */
 #define diFeatureColorPopup 4
 #define diFeatureColorName 5
 
-#define dScreenDepth 137
-
-#define diScreenDepthOK 1
-#define diScreenDepthCancel 2
-
 #define dConnectionMethod 170
 
 #define diConnectionMethodOK 1
@@ -733,6 +777,105 @@ any later version.  See the file COPYING.  */
 #define diGameOverQuit 1
 #define diGameOverContinue 2
 
+#define dCity 803
+
+#define diCityMatPanel 4
+#define diCityFacPanel 5
+#define diCityOccPanel 6
+
+#define diCityPlanPopup 7
+#define diCityAdvancePopup 8
+#define diCityResearchStatus 9
+#define diCityBuildPopup 10
+#define diCityBuildStatus 11
+
+#define diCityAICheck 12
+#define diCityPlanCheck 13
+#define diCityAdvanceCheck 14
+#define diCityBuildCheck 15
+#define diCityBuildRepeat 16
+#define diCityBuildEdit 17
+
+#define diCityName 18
+#define diCitySize 19
+#define diCityStats 20
+
+#define diCityMatBase 16
+#define diCityOccBase 41
+
+#define diCityIcon 93
+#define diCityClickText 94
+#define diCityBuildTimes 97
+#define diCityMap 98
+
+#define dAdvance 804
+
+#define diAdvanceMadScientist 3
+#define diAdvanceBusyScientist 4
+#define diAdvanceIdleScientist 5
+#define diAdvanceMainText 6
+#define diAdvancePopup 7
+#define diAdvanceStatus 8
+#define diAdvanceCheck 9
+
+#define dGlobalAdvance 805
+
+#define diGlobalMadScientist 3
+#define diGlobalBusyScientist 4
+#define diGlobalIdleScientist 5
+#define diGlobalMainText 6
+#define diGlobalIdleMenu 7		/* First popup menu (Idle units). */
+#define diGlobalIdleNum 17		/* Its associated number of cities. */
+#define diGlobalIdleStatus 27	/* Completion status of research. */
+#define diGlobalIdleText 37		/* Its associated text. */
+#define diGlobalMoreNote 47	
+#define diGlobalDoneMenu 50
+#define diGlobalIdleCheck 53
+#define diGlobalAllCheck 64
+
+#define dMeridians 806
+
+#define diMeridiansOK 1
+#define diMeridiansCancel 2
+#define diMeridiansOnOff 3
+#define diMeridiansInterval 4
+
+#define dBuild 807
+
+#define diBuildMadScientist 3
+#define diBuildBusyScientist 4
+#define diBuildIdleScientist 5
+#define diBuildMainText 6
+#define diBuildPopup 7
+#define diBuildStatus 8
+#define diBuildEdit 9
+#define diBuildText 10
+#define diBuildCheck 11
+
+#define dPlan 808
+
+#define diPlanMainText 4
+#define diPlanPopup 5
+#define diPlanCheck 6
+
+#define dWorldMap 809
+
+#define diWorldMapOK 1
+
+#define dCloseup 810
+
+#define dSideResearch 811
+
+#define diSideResearchClose 1
+#define diSideResearchHelp 2
+#define diSideResearchPopup 3
+#define diSideResearchCheck 4
+#define diSideResearchText1 5
+#define diSideResearchText2 6
+#define diSideResearchWizard 7
+
+#define MAXPROJMENUS 9		/* Max number of project menus (excluding Idle units) and associated items. */
+
 /* Alerts. */
 
 #define aInitWarning 1000
@@ -810,6 +953,9 @@ any later version.  See the file COPYING.  */
 #define wNotice 136
 #define wCommand 137
 #define wScores 138
+#define wFloatFixed 139
+#define wFloatResize 140
+#define wInstructions 141
 
 /* Controls. */
 
@@ -817,11 +963,13 @@ any later version.  See the file COPYING.  */
 #define cResearchButton 129
 #define cRunLengthPopup 210
 
+#define cCloseButton 130
 #define cTopicsButton 131
 #define cHelpButton 132
 #define cPrevButton 133
 #define cNextButton 134
 #define cBackButton 135
+#define cDevelopButton 136
 
 #define cFeatureAddButton 401
 #define cFeatureRemoveButton 402
@@ -861,6 +1009,11 @@ any later version.  See the file COPYING.  */
 #define cSendPacket 300
 #define cReceivePacket 301
 
+#define cLeft 401
+#define cRight 402
+#define cDown 403
+#define cUp 404
+
 /* Patterns. */
 
 #define pMarchingAnts 128
