@@ -7,7 +7,6 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-#include "kpublic.h"
 #include "macconq.h"
 
 /* Needed for changing screen depths and handling the control strip */
@@ -16,8 +15,6 @@ any later version.  See the file COPYING.  */
 
 #ifdef CONTROL_STRIP
 
-// --- NEW STUFF MOVED HERE FROM APPLESTUFF.H ------------------
-
 /*	Apple control strip routine descriptors follow below. */
 /*	Must be included in PPC projects that use the control strip. *
 /*	See http://devworld.apple.com/dev/qa/ops/ops15.html. */
@@ -29,13 +26,16 @@ any later version.  See the file COPYING.  */
 
 #include <MixedMode.h>
 #include <OSUtils.h>
+
 pascal Boolean SBIsControlStripVisible ( void );
 pascal void SBShowHideControlStrip(Boolean showIt);
 
 /*  SBIsControlStripVisible is a Pascal routine, */
 /*  dispatched from the selector in D0, returning */
 /*  a Boolean result */
-pascal Boolean SBIsControlStripVisible ( void )
+
+pascal Boolean
+SBIsControlStripVisible(void)
 {
     enum
         {
@@ -49,7 +49,8 @@ pascal Boolean SBIsControlStripVisible ( void )
             uppSBIsControlStripVisibleInfo, 0x00);
 }
 
-pascal void SBShowHideControlStrip(Boolean showIt)
+pascal void
+SBShowHideControlStrip(Boolean showIt)
 {
     enum
         {
@@ -69,16 +70,11 @@ pascal void SBShowHideControlStrip(Boolean showIt)
 #include <ControlStrip.h>
 #endif /*  GENERATINGCFM */
 
-// --- END NEW STUFF -------------------------------------------------------
-
 #endif /* CONTROL_STRIP */
 
-// --- USED TO DEBUG OFFSCREEN GRAPHICS ----------------------------------------
-
 static Rect debugRect = {0};
-static int scrolling = FALSE;
 
-// --- FUNCTION PROTOTYPES  --------------------------------------------------
+static int scrolling = FALSE;
 
 static void reduce_screen_depth(GDHandle gdev);
 static void auto_scroll_left(Map *map);
@@ -90,12 +86,16 @@ static void auto_scroll_down_right(Map *map);
 static void auto_scroll_up_left(Map *map);
 static void auto_scroll_up_right(Map *map);
 
-// --- NEW FUNCTION ------------------------------------------------------------
+extern void scroll_list_using_key(List *list, char key);
 
+static void auto_scroll_left_arrow(Map *map);
+static void auto_scroll_right_arrow(Map *map);
+static void auto_scroll_up_arrow(Map *map);
+static void auto_scroll_down_arrow(Map *map);
+static void rotate_angle_up(Map *map);
+static void rotate_angle_down(Map *map);
 static void copy_clipped_to_map(Map *map, Rect sourcerect, Rect destrect);
 
-// --- FUNCTIONS USED TO SET THE SCREEN DEPTH ----------------------------------------
-
 /* Check if any device has more than 256 colors */
 
 void
@@ -174,8 +174,6 @@ reduce_screen_depth(GDHandle gdev)
 	DisposeDialog(win);
 }
 
-// ---FUNCTIONS USED DIRECTLY IN OFFSCREEN GRAPHICS ----------------------------------
-
 /* Scrolls the map window dx, dy steps within its offscreen gworld. This function should always 
    be called whenever the viewport is moved. If necessary, get the new focus point, reset to the old
    focus, and then let scroll_map_window do the job. This ensures that the gworld always is used. */
@@ -183,176 +181,285 @@ reduce_screen_depth(GDHandle gdev)
 void 
 scroll_map_window(Map *map, int dx, int dy)
 {
-	int 		newsx, newsy, scaled_sx, scaled_sy, dummy, redraw = FALSE;
-	Rect		destRect, tmprect, portrect = map->gworldPortPtr->portRect;
+	int 		mainw, mainh, newsx, newsy, scaled_sx, scaled_sy, dummy, redraw = FALSE;
+	Rect		destRect, mapRect;
+	Point		mapUL, mapDR;
 	GrafPtr	oldport;
-	
+	WindowPtr	win;
+
 	GetPort(&oldport);
-	/* Set current port to the map window */
+	/* Set current port to the map window. */
 	SetPort((GrafPtr)map->windowPortPtr);
 
-	if (DebugG) {
-		/* Erase the old debugRect */
+  	if (DebugG) {
+		/* Erase the old debugRect. */
 		ForeColor(redColor);
 		PenMode(patXor);
 		FrameRect(&debugRect);
 		ForeColor(blackColor);
 		PenMode(normal);
-		scrolling = TRUE;	/* Turns off debugRect while scrolling */
+		scrolling = TRUE;	/* Turn off debugRect while scrolling. */
 	}
 
-	/* Erase other-map boxes in other windows */
+	/* Erase other-map boxes in other windows. */
 	draw_related_maps(map);
 
-	/* Find the new position */
+	/* Find the new position. */
 	newsx = map->vp->sx + dx;
 	newsy = map->vp->sy + dy;
 	
-	/* Handle xwrapped world */
 	if (area.xwrap) {	
-		/* Scrolling past the eastern edge of the world */
-		if (newsx > map->vp->sxmax) newsx -= map->vp->sxmax;
-		/* Scrolling past the western edge of the world */
-		else if (newsx < 0) newsx += map->vp->sxmax;
-	/* Handle non-xwrapped world */
+		/* Handle xwrapped world. */
+		while (newsx >= map->vp->sxmax)
+		  newsx -= map->vp->sxmax;
+		while (newsx <= 0)
+		  newsx += map->vp->sxmax;
 	} else {		
-		/* Restrict newsx to stay between the map edges */
+		/* Handle non-xwrapped world. Restrict newsx to stay between the map edges. */
 		newsx = limitn(map->vp->sxmin, newsx, map->vp->sxmax);
-		/* Adjust dx to reflect this restriction */
+		/* Adjust dx to reflect this restriction. */
 		dx = newsx - map->vp->sx;
 	}
 
-	/* Scale the scrollbar to circumvent the 32K limit */
-	/* (see set_map_scrollbars for details) */
+	/* Scale the scrollbar to circumvent the 32K limit (see 
+	set_map_scrollbars for details). */
 	dummy = map->vp->sxmax;
 	scaled_sx = newsx;
 	while (dummy > 32000) {
 		dummy /= 2;
 		scaled_sx /= 2;
 	}		
-	/* Use the scaled value to set the new thumb position */
+	/* Use the scaled value to set the new thumb position. */
 	SetCtlValue(map->hscrollbar, scaled_sx);
 
-	/* Restrict newsy to stay between the map edges */
+	/* Restrict newsy to stay between the map edges. */
 	newsy = limitn(map->vp->symin, newsy, map->vp->symax);
-	/* Adjust dy to reflect this restriction */
+	/* Adjust dy to reflect this restriction. */
 	dy = newsy - map->vp->sy;
 
-	/* Scale the scrollbar to circumvent the 32K limit */
-	/* (see set_map_scrollbars for details) */
+	/* Scale the scrollbar to circumvent the 32K limit (see 
+	set_map_scrollbars for details). */
 	dummy = map->vp->symax;
 	scaled_sy = newsy;
 	while (dummy > 32000) {
 		dummy /= 2;
 		scaled_sy /= 2;
 	}		
-	/* Use the scaled value to set the new thumb position */
+	/* Use the scaled value to set the new thumb position. */
 	SetCtlValue(map->vscrollbar, scaled_sy);
 
-	/* Move the viewport */
+	/* Move the viewport. */
 	map->vp->sx = newsx;
 	map->vp->sy = newsy;
 	set_content_rect(map);
-
-	/* Handle dx for wrapped gworlds.  Never update. */
-	if (area.xwrap && map->vp->sxmax <= portrect.right - portrect.left) {
+	
+	/* Handle dx. Don't update if the map is wrapped. */
+	if (map->xwrap) {
 		redraw = FALSE; 
 	} else if ((dx > 0 && map->offsetx + dx > map->bufx)
 			   || (dx < 0 && map->offsetx + dx < -map->bufx)) {
 		/* Handles dx in other cases.  Update if stepping outside. */
 		redraw = TRUE;
 	}		
-	/* Always update if dy take us outside the gworld */
+	/* Always update if dy take us outside the gworld. */
 	if ((dy > 0 && map->offsety + dy > map->bufy)
 		|| (dy < 0 && map->offsety + dy < -map->bufy)) {
 		redraw = TRUE;
 	}		
 
 	if (redraw) {
-		/* Redraw gworld and copy contentrect to the window */
+		/* Redraw gworld and copy contentrect to the window. */
 		update_gworld(map);
-		/* Redraw other-map boxes and restore port before exit */
+		/* Redraw other-map boxes and restore port before exit. */
 		draw_related_maps(map);
 		SetPort(oldport);
 		if (DebugG) {
-			SetRect(&debugRect, 0, 0, 0, 0);		/* Prevents drawing of junk next time */
-			scrolling = FALSE;				/* Turns on debugRect after scrolling */
+			SetRect(&debugRect, 0, 0, 0, 0);		/* Prevents drawing of junk next time. */
+			scrolling = FALSE;				/* Turn on debugRect after scrolling. */
 		}
 		return;
 	}	   
 	  
-	/* Scroll the already drawn region */
+	/* Scroll the already drawn region. */
 	ScrollRect(&map->contentrect, -dx, -dy, NULL);
-	/* Update offset trackers */
+	/* Update offset trackers. */
 	map->offsetx += dx;
 	map->offsety += dy;
-	/* Wrap offsetx if necessary */
+	/* Wrap offsetx if necessary. */
 	map->offsetx %= map->vp->sxmax;
 
-	/* Calculate update rects and update each one in turn from the gworld */
+	/* Find the corners of the map contentrect in global coordinates. */
+	mapUL.h = map->contentrect.left;
+	mapUL.v = map->contentrect.top;
+	mapDR.h = map->contentrect.right;
+	mapDR.v = map->contentrect.bottom;
+	LocalToGlobal(&mapUL);			
+	LocalToGlobal(&mapDR);			
 
-	if (dx > 0) {				/* Going Right */
-		SetRect(&destRect, 
-				map->conw + map->vp->pxw - dx, 
-				map->toph, 
-				map->conw + map->vp->pxw, 
-				map->toph + map->vp->pxh);
+	/* Clip it to main screen minus menubar to fix graphics drag at edges. */
+	get_main_screen_size(&mainw, &mainh);
+	mapUL.h = max(0, mapUL.h);
+	mapUL.v = max(GetMBarHeight(), mapUL.v);
+	mapDR.h = min(mapDR.h, mainw);
+	mapDR.v = min(mapDR.v, mainh);
+		
+	/* Convert clipped contentrect back to local coordinates. */
+	GlobalToLocal(&mapUL);
+	GlobalToLocal(&mapDR);
+	SetRect(&mapRect, mapUL.h, mapUL.v, mapDR.h, mapDR.v);
 
+	/* Do horizontal scrolling. */
+	destRect = mapRect;
+
+	if (dx > 0) {							/* Going Right. */
+		destRect.left = destRect.right - dx; 
 		copy_from_gworld(map, destRect);
-	
-	} else if (dx < 0) {			/* Going Left */
-		SetRect(&destRect, 
-				map->conw, 
-				map->toph, 
-				map->conw - dx, 
-				map->toph + map->vp->pxh);
-
+	} else if (dx < 0) {						/* Going Left. */
+		destRect.right = destRect.left - dx; 
 		copy_from_gworld(map, destRect);
 	}
 
-	if (dy > 0) {				/* Going Down */
-		SetRect(&destRect, 
-			/* Skip area already updated by Going Right */
-			max(map->conw,   map->conw - dx), 
-		   	         map->toph + map->vp->pxh - dy, 
-			/* Skip area already updated by Going Left */
-			min(map->conw + map->vp->pxw, map->conw + map->vp->pxw - dx), 
-				map->toph + map->vp->pxh);
+	/* Do vertical scrolling. */
+	destRect = mapRect;
 
+	/* Skip overlap already updated by Going Right or Going Left. */
+	destRect.left = max(destRect.left, destRect.left - dx); 
+	destRect.right = min(destRect.right, destRect.right - dx);		
+
+	if (dy > 0) {							/* Going Down. */
+		destRect.top = destRect.bottom - dy;
 		copy_from_gworld(map, destRect);
-
-	} else if (dy < 0) {			/* Going Up */
-		SetRect(&destRect, 
-			/* Skip area already updated by Going Right */
-			 max(map->conw,   map->conw - dx), 
-		   	         map->toph, 
-			/* Skip area already updated by Going Left */
-			 min(map->conw + map->vp->pxw, map->conw + map->vp->pxw - dx), 
-				map->toph - dy);
-
+	} else if (dy < 0) {						/* Going Up. */
+		destRect.bottom = destRect.top - dy;
 		copy_from_gworld(map, destRect);
 	}
 	
-	/* Redraw other-map boxes and restore the old port */
+	/* This handles situations where we have hidden the scrollbars and the grow box forms
+	a small protruding corner that causes a graphics drag. */
+	
+	if (!map->sbarwid) {
+
+		Rect		growRect;
+
+		/* Calculate the grow box rect. */
+		growRect = map->contentrect;
+		growRect.left = growRect.right - (map != worldmap ? sbarwid : floatsbarwid);
+		growRect.top = growRect.bottom - (map != worldmap ? sbarwid : floatsbarwid);
+
+		/* Clip to (map and) main screen content. Proceed if an overlap was found. */
+		if (SectRect(&growRect, &mapRect, &growRect) == TRUE) {
+		
+			/* Do horizontal scrolling. */
+			if (dx > 0) {						/* Going Right. */
+				destRect = growRect;
+				destRect.right = destRect.left;
+				destRect.left -= dx;
+				destRect.top -= dy;
+				copy_from_gworld(map, destRect);
+			}
+
+			/* Do vertical scrolling. */
+			if (dy > 0) {						/* Going Down. */
+				destRect = growRect;
+				destRect.bottom = destRect.top;
+				destRect.top -= dy;
+				/* Add small square missed by Going Right. */	
+				destRect.left -= dx;							
+				copy_from_gworld(map, destRect);
+			}
+		}
+	}
+	
+	/* This updates areas that scroll into view from behind floating windows. Note that 
+	because of the inverted geometry, going-down and going-up ADDS an extra square at
+	the end of destRect instead of skipping one (min and max are reversed). */
+
+	/* Note: this code also handles non-floating windows in front of the scrolled window
+	correctly. Can happen if the map is scrolled in the background. */
+
+	for_all_windows(win) {
+
+		WindowPeek 	winPeek;
+		Rect			winRect;		
+		Point			winUL;
+		Point			winDR;
+					
+		/* Stop when we reach the map window. */
+		if (win == map->window)
+			break;
+
+		/* Find the corners of the window in global coordinates. */
+		winPeek = (WindowPeek) win;
+		winRect = (*(winPeek->strucRgn))->rgnBBox;			
+		
+		/* Transform the result into local coordinates. */
+		winUL.h = winRect.left;
+		winUL.v = winRect.top;
+		winDR.h = winRect.right;
+		winDR.v = winRect.bottom;
+		GlobalToLocal(&winUL);			
+		GlobalToLocal(&winDR);			
+		SetRect(&winRect, winUL.h, winUL.v, winDR.h, winDR.v);
+
+		/* Clip window rect to the map (and main screen) content. Skip 
+		to the next window if no overlap was found. */
+		if (SectRect(&winRect, &mapRect, &winRect) != TRUE)
+		    continue;
+
+		/* Do horizontal scrolling. */
+		destRect = winRect;
+
+		if (dx > 0) {						/* Going Right. */
+			destRect.right = destRect.left;
+			destRect.left -= dx;
+			copy_from_gworld(map, destRect);
+		} else if (dx < 0) {					/* Going Left. */
+			destRect.left = destRect.right;
+			destRect.right -= dx;
+			copy_from_gworld(map, destRect);
+		}
+
+		/* Do vertical scrolling. */
+		destRect = winRect;
+
+		/* Add small square missed by Going Right or Going Left. */
+		destRect.left = min(destRect.left, destRect.left - dx); 
+		destRect.right = max(destRect.right, destRect.right - dx);		
+
+		if (dy > 0) {						/* Going Down. */
+			destRect.bottom = destRect.top;
+			destRect.top -= dy;
+			copy_from_gworld(map, destRect);
+		} else if (dy < 0) {					/* Going Up. */
+			destRect.top = destRect.bottom;
+			destRect.bottom -= dy;
+			copy_from_gworld(map, destRect);
+		}
+	}
+
+	/* Redraw other-map boxes and restore the old port. */
 	draw_related_maps(map);
 	SetPort(oldport);
 
 	if (DebugG) {
-		SetRect(&debugRect, 0, 0, 0, 0);		/* Prevents drawing of junk next time */
-		scrolling = FALSE;				/* Turns on debugRect after scrolling */
+		SetRect(&debugRect, 0, 0, 0, 0);		/* Prevents drawing of junk next time. */
+		scrolling = FALSE;				/* Turn on debugRect after scrolling. */
+				
 	}
 }
 
 /* Copies destrect from its source in the offscreen gworld. 
-     Special code for handling wrapped cylindrical gworlds. */
+   Special code for handling wrapped cylindrical gworlds. */
 
 void 
 copy_from_gworld(Map *map, Rect destrect)
 {
-	Rect 	portrect, sourcerect;
-	
+	Rect 	portrect= map->gworldPortPtr->portRect;
+	Rect	sourcerect = destrect;
+	int 	sxmax = map->vp->sxmax;
+		
+	/* Erase the old debugRect. */
 	if (DebugG &! scrolling) {
-		/* Erase the old debugRect */
 		ForeColor(redColor);
 		PenMode(patXor);
 		FrameRect(&debugRect);
@@ -360,93 +467,72 @@ copy_from_gworld(Map *map, Rect destrect)
 		PenMode(normal);
 	}
 
-	portrect = map->gworldPortPtr->portRect;
-	sourcerect = destrect;
-	
-	/* Adjust source position for offset, scrolling buffers and decor */
+	/* Adjust source position for offset, scrolling buffers and decor. */
 	OffsetRect(&sourcerect, map->offsetx + map->bufx - map->conw, 
 			   		   map->offsety + map->bufy - map->toph);
 
-	/* Handle wrapped cylindrical gworlds (complicated) */
-	if (area.xwrap && map->vp->sxmax <= portrect.right - portrect.left) {
+	/* Make sure sourcerect is at least partially within the gworld. */
+	while (area.xwrap && sourcerect.right < 0)
+		OffsetRect(&sourcerect, sxmax, 0);
+	while (area.xwrap && sourcerect.left > sxmax)
+		OffsetRect(&sourcerect, -sxmax, 0);
+
+	/* Handle wrapped map (complicated). */
+	if (map->xwrap) {
 		Rect westsource = sourcerect;
 		Rect eastsource = sourcerect;
 		Rect midsource = sourcerect;
 		Rect westdest = destrect;
 		Rect eastdest = destrect;
 		Rect middest = destrect;
-		int rest;
 				
-		/* Handle region inside gworld (unnecessary if scrolling)*/
+		/* Handle inside area (unnecessary if scrolling).*/
 		if (!scrolling) {
 			SectRect(&sourcerect, &portrect, &midsource);
 			middest.left += midsource.left - sourcerect.left;				
 			middest.right += midsource.right - sourcerect.right;				
-			/* Make several tandem copies if necessary */
-			rest = map->vp->pxw - middest.left;
-			while (rest > -map->vp->sxmax) {
+			/* Make several tandem copies if necessary. */
+			while (map->vp->pxw + sxmax > middest.left) {
 				copy_clipped_to_map(map, midsource, middest);
-				/* Move destination one full gworld width east */
-				OffsetRect(&middest, map->vp->sxmax, 0);
-				rest -= map->vp->sxmax;
+				OffsetRect(&middest, sxmax, 0);
 			} 
 		}
 				
-		/* Handle region beyond western edge of gworld */
-		if (sourcerect.left < portrect.left) {
-			westsource.right = portrect.left;
-			westdest.right   += portrect.left - sourcerect.right;
-			OffsetRect(&westsource, map->vp->sxmax, 0);
-			/* Make several tandem copies if necessary */
-			rest = map->vp->pxw - westdest.left;
-			while (rest > -map->vp->sxmax) {
-				/* Only make westmost copy if scrolling */
-				if (!scrolling || rest == map->vp->pxw - westdest.left)
-					copy_clipped_to_map(map, westsource, westdest);
-				/* Move destination one full gworld width east */
-				OffsetRect(&westdest, map->vp->sxmax, 0);
-				rest -= map->vp->sxmax;
+		/* Handle area beyond western edge. */
+		if (sourcerect.left < 0) {
+			westsource.right = 0;
+			westdest.right -= sourcerect.right;
+			OffsetRect(&westsource, sxmax, 0);
+			/* Make only westmost copy if scrolling. */
+			if (scrolling)
+				copy_clipped_to_map(map, westsource, westdest);
+			/* Else make several tandem copies if necessary. */
+			else while (map->vp->pxw + sxmax > westdest.left) {
+				copy_clipped_to_map(map, westsource, westdest);
+				OffsetRect(&westdest, sxmax, 0);
 			}
 		}
 
-		/* Handle region beyond eastern edge of gworld */
+		/* Handle area beyond eastern edge. */
 		if (sourcerect.right > portrect.right) {
-			/* Extend/truncate source to right end of gworld */
 			eastsource.left = portrect.right;
-			/* Extend/truncate dest by the same amount */
 			eastdest.left += portrect.right - sourcerect.left;
-			/* Repeat as long as source protrudes beyond gworld */
-			while (eastsource.right > eastsource.left) {
-				Rect tmpsource, tmpdest = eastdest;
-				/* Shift source one full area width to the left */
-				OffsetRect(&eastsource, -map->vp->sxmax, 0);
-				/* Copy to screen only when we reach destrect */
-				if (destrect.left - eastdest.left < map->vp->sxmax) {
-					/* Copy one full gworld content each time */
-					SectRect(&eastsource, &portrect, &tmpsource);
-					tmpdest.right -= eastsource.right - tmpsource.right; 
-					/* Make several tandem copies if necessary */
-					rest = map->vp->pxw - tmpdest.left;
-					while (rest > -map->vp->sxmax) {
-						/* Only make eastmost copy if scrolling */
-						if (!scrolling || rest < map->vp->sxmax)
-							copy_clipped_to_map(map, tmpsource, tmpdest);
-						/* Move destination one full gworld width east */
-						OffsetRect(&tmpdest, map->vp->sxmax, 0);
-						rest -= map->vp->sxmax;
-					} 
-				}
-				/* Truncate both rects by one area width */
-				eastsource.left 	+= map->vp->sxmax;
-				eastdest.left	+= map->vp->sxmax;
+			OffsetRect(&eastsource, -sxmax, 0);
+			/* Make only eastmost copy if scrolling. */
+			if (scrolling)
+				copy_clipped_to_map(map, eastsource, eastdest);
+			/* Else make several tandem copies if necessary. */
+			else while (map->vp->pxw + sxmax > eastdest.left) {
+				copy_clipped_to_map(map, eastsource, eastdest);
+				OffsetRect(&eastdest, sxmax, 0);
 			}
 		}
 
-	/* Handle all other cases (simple) */
+	/* Handle all other cases (simple). */
 	} else copy_clipped_to_map(map, sourcerect, destrect);
 	
+	/* Draw the new debugRect. */
 	if (DebugG &! scrolling) {
-		/* Draw the new debugRect */
 		debugRect = destrect;					
 		ForeColor(redColor);
 		PenMode(patXor);
@@ -456,28 +542,40 @@ copy_from_gworld(Map *map, Rect destrect)
 	}
 }
 
-/* Core function in offscren graphics that does the actual copying to the screen. Both source */
-/* and destination rects are first adjusted so that copying is clipped to the map contentrect. */ 
+/* Core function in offscreen graphics that does the actual copying to the screen. Both source
+   and destination rects are first adjusted so that copying is clipped to the map contentrect. */ 
 
 void
 copy_clipped_to_map(Map *map, Rect sourcerect, Rect destrect)
 {
-	/* Clip left edge to contentrect */
+	/* Much of this clipping could be eliminated when certain bugs are gone. */
+
+	/* Clip left edge to gworld. */
+	if (sourcerect.left < 0) {
+		destrect.left  -= sourcerect.left;
+		sourcerect.left = 0;
+	}	
+	/* Clip right edge to gworld. */
+	if (sourcerect.right > map->gworldPortPtr->portRect.right) {
+		destrect.right -= sourcerect.right - map->gworldPortPtr->portRect.right;
+		sourcerect.right = map->gworldPortPtr->portRect.right;
+	}
+	/* Clip left edge to contentrect. */
 	if (destrect.right > map->contentrect.right) {
 		sourcerect.right -= destrect.right - map->contentrect.right;
 		destrect.right = map->contentrect.right;
 	}
-	/* Clip right edge to contentrect */
+	/* Clip right edge to contentrect. */
 	if (destrect.left < map->contentrect.left) {
 		sourcerect.left += map->contentrect.left - destrect.left;
 		destrect.left = map->contentrect.left;
 	}
-	/* Clip top edge to contentrect */
+	/* Clip top edge to contentrect. */
 	if (destrect.top < map->contentrect.top) {
 		sourcerect.top += map->contentrect.top - destrect.top;
 		destrect.top = map->contentrect.top;
 	}
-	/* Clip bottom edge to contentrect */
+	/* Clip bottom edge to contentrect. */
 	if (destrect.bottom > map->contentrect.bottom) {
 		sourcerect.bottom -= destrect.bottom - map->contentrect.bottom;
 		destrect.bottom = map->contentrect.bottom;
@@ -537,7 +635,10 @@ update_resized_map(Map *map)
 
 		/* Paint the update region blue (looks nice) */
 		BackPat(QDPat(black));
-		if (hasColorQD) ForeColor(cyanColor);
+
+		if (hasColorQD)
+		  RGBForeColor(&backcolor);
+
 		EraseRect(&map->contentrect);
 		ForeColor(blackColor);
 		EndUpdate(map->window);
@@ -649,6 +750,12 @@ update_gworld(Map *map)
 	/* Set window displacement trackers to zero */
 	map->offsetx = map->offsety = 0;
 
+	/* Cache the wrapping status. */
+	if (area.xwrap && map->vp->sxmax <= GWportRect.right - GWportRect.left)
+	  map->xwrap = TRUE;
+	else
+	  map->xwrap = FALSE;
+
 	/* Do all drawing in the background */
 	ValidRect(&map->contentrect);
 	BeginUpdate(map->window);
@@ -690,12 +797,13 @@ cell_is_in_gworld(Map *map, int x, int y)
 		return FALSE;
 	if (sy >  map->vp->pxh + 2 * map->bufy) 
 		return FALSE;
-	if (area.xwrap && map->vp->sxmax <= portrect.right - portrect.left)
-		return TRUE;
+
+	if (map->xwrap)
+	  return TRUE;
 	else if (sx + map->vp->hw + map->bufx < 0) 
-		return FALSE;
+	  return FALSE;
 	else if (sx >  map->vp->pxw + 2 * map->bufx) 
-		return FALSE;
+	  return FALSE;
 	return TRUE;
 }
 
@@ -709,20 +817,31 @@ handle_numeric_keypad(EventRecord *event)
 {
 	int	pxh, pxw;
 	Map *map;
-	char key;
+	char	key;
 
-	/* Get the front map */
-	map = front_map();
+	/* Get the worldmap if alt key was down, else get the front map. */
+	if (event->modifiers & optionKey)
+		map = worldmap;
+	else	map = frontmap;
 
-	/* Check keys by keyCode instead of charCode to identify numeric keypad events */
+	/* Use keyCode instead of charCode to distinguish numeric keypad keys. */
 	key = (event->message & keyCodeMask) >> 8;
 
-	/* Return FALSE if this is not a numeric keypad key (or if it is an arrow key on an antique Mac) */
-	if (key < 0x41 || key > 0x5C || key == 0x42 || key == 0x46 || key == 0x48 || key == 0x4D)
+	/* Return FALSE if this is not a numeric keypad key (or if it is an arrow 
+	key on an antique Mac) */
+	if (key < 0x41
+		|| (key > 0x5C && key < 0x7B)
+		|| key == 0x42
+		|| key == 0x46
+		|| key == 0x47
+		|| key == 0x48
+		|| key == 0x4C
+		|| key == 0x4D)
+
 	  return FALSE;
 
 	/* Return TRUE and BEEP if the front window is not a map window */
-	if (!map || map->window != FrontWindow()) {
+	if (!map) {
 		beep(); 
 		return TRUE;
 	}
@@ -746,8 +865,14 @@ handle_numeric_keypad(EventRecord *event)
 		if (key == 0x53) { scroll_map_window(map, -pxw, pxh); return TRUE; }	/* Keypad 1 */
 		if (key == 0x55) { scroll_map_window(map, pxw, pxh); return TRUE; }		/* Keypad 3 */					
 
-	/* Else either auto-scroll as long as the key is down, or change the center, scale or view angle  */
+		if (key == 0x7B) { scroll_map_window(map, -pxw, 0); return TRUE; }		/* Left Arrow*/
+		if (key == 0x7C) { scroll_map_window(map, pxw, 0); return TRUE; }		/* Right Arrow */
+		if (key == 0x7E) { scroll_map_window(map, 0, -pxh); return TRUE; }		/* Up Arrow */
+		if (key == 0x7D) {  scroll_map_window(map, 0, pxh); return TRUE; }		/* Down Arrow */
+
 	} else {
+		/* Else either auto-scroll as long as the key is down, or change the center,
+		   scale or view angle.  */
 		if (key == 0x56) { auto_scroll_left(map); return TRUE; }		/* Keypad 4 */
 		if (key == 0x58) { auto_scroll_right(map); return TRUE; }		/* Keypad 6 */
 		if (key == 0x5B) { auto_scroll_up(map); return TRUE; }		/* Keypad 8 */
@@ -757,25 +882,22 @@ handle_numeric_keypad(EventRecord *event)
 		if (key == 0x5C) { auto_scroll_up_right(map); return TRUE; }	/* Keypad 9 */
 		if (key == 0x53) { auto_scroll_down_left(map); return TRUE; }	/* Keypad 1 */
 		if (key == 0x55) { auto_scroll_down_right(map); return TRUE; }	/* Keypad 3 */
-		if (key == 0x57) { do_recenter(); return TRUE; }				/* Keypad 5 */
-		if (key == 0x45) { do_mac_zoom_in(); return TRUE; }			/* Keypad + */
-		if (key == 0x4E) { do_mac_zoom_out(); return TRUE; }			/* Keypad - */
-		if (key == 0x43) { do_mac_set_map_angle(); return TRUE; }		/* Keypad * */
 
-		if (Debug || DebugG || DebugM) {
-			/* Take a snapshot of the gworld */
-			if (key == 0x47) {
-				copy_from_gworld(map, map->contentrect); 		/* Keypad CLEAR */
-				return TRUE;
-			}
-			/* Enter the debugger */
-			if (key == 0x4C) {Debugger(); return TRUE; }			/* Keypad ENTER */
-		}
+		if (key == 0x7B) { auto_scroll_left_arrow(map); return TRUE; }		/* Left Arrow*/
+		if (key == 0x7C) { auto_scroll_right_arrow(map); return TRUE; }		/* Right Arrow */
+		if (key == 0x7E) { auto_scroll_up_arrow(map); return TRUE; }		/* Up Arrow */
+		if (key == 0x7D) { auto_scroll_down_arrow(map); return TRUE; }		/* Down Arrow */
+
+		if (key == 0x57) { do_recenter(dside); return TRUE; }			/* Keypad 5 */
+		if (key == 0x45) { magnify_map(map, 1); return TRUE; }		/* Keypad + */
+		if (key == 0x4E) { magnify_map(map, -1); return TRUE; }		/* Keypad - */
+
+		if (key == 0x43) { rotate_angle_up(map); return TRUE; }		/* Keypad * */
+		if (key == 0x4B) { rotate_angle_down(map); return TRUE; }		/* Keypad / */
 	}
 	return FALSE;	/* Default FALSE permits normal use of unassigned numeric keypad keys */
 }
 
-
 /* The following auto_scroll functions bypass the main event loop for increased scrolling speeed. */
 
 /* Scroll left as long as key is pressed */
@@ -788,7 +910,7 @@ auto_scroll_left(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, -4, 0);
-		GetKeys((long *) keyMap);
+		GetKeys((unsigned long *) keyMap);
 		keyDown = keyMap[0x56 >> 3] >> (0x56 & 7) & 1;	/* keypad 4 */
 	}
 }
@@ -803,7 +925,7 @@ auto_scroll_right(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 4, 0);
-		GetKeys((long *) keyMap);
+		GetKeys((unsigned long *) keyMap);
 		keyDown = keyMap[0x58 >> 3] >> (0x58 & 7) & 1;	/* keypad 6 */
 	}
 }
@@ -818,7 +940,7 @@ auto_scroll_up(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 0, -4);
-		GetKeys((long *) keyMap);
+		GetKeys((unsigned long *) keyMap);
 		keyDown = keyMap[0x5B >> 3] >> (0x5B & 7) & 1;	/* keypad 8 */
 	}
 }
@@ -833,7 +955,7 @@ auto_scroll_down(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 0, 4);
-		GetKeys((long *) keyMap);
+		GetKeys((unsigned long *) keyMap);
 		keyDown = keyMap[0x54 >> 3] >> (0x54 & 7) & 1;	/* keypad 2 */
 	}
 }
@@ -848,7 +970,7 @@ auto_scroll_down_left(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, -4, 4);
-		GetKeys((long *) keyMap);
+		GetKeys((unsigned long *) keyMap);
 		keyDown = keyMap[0x53 >> 3] >> (0x53 & 7) & 1;	/* keypad 1 */
 	}
 }
@@ -863,7 +985,7 @@ auto_scroll_down_right(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 4, 4);
-		GetKeys((long *) keyMap);
+		GetKeys((unsigned long *) keyMap);
 		keyDown = keyMap[0x55 >> 3] >> (0x55 & 7) & 1;	/* keypad 3 */
 	}
 }
@@ -878,7 +1000,7 @@ auto_scroll_up_left(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, -4, -4);
-		GetKeys((long *) keyMap);
+		GetKeys((unsigned long *) keyMap);
 		keyDown = keyMap[0x59 >> 3] >> (0x59 & 7) & 1;	/* keypad 7 */
 	}
 }
@@ -893,11 +1015,93 @@ auto_scroll_up_right(Map *map)
 
 	while (keyDown) {
 		scroll_map_window(map, 4, -4);
-		GetKeys((long *) keyMap);
+		GetKeys((unsigned long *) keyMap);
 		keyDown = keyMap[0x5C >> 3] >> (0x5C & 7) & 1;	/* keypad 9 */
 	}
 }
 
+/* Scroll left as long as left arrow key is pressed */
+
+void
+auto_scroll_left_arrow(Map *map)
+{
+	unsigned char keyMap[16];
+	short keyDown = TRUE;
+
+	while (keyDown) {
+		scroll_map_window(map, -4, 0);
+		GetKeys((unsigned long *) keyMap);
+		keyDown = keyMap[0x7B >> 3] >> (0x7B & 7) & 1;	/* left arrow */
+	}
+}
+
+/* Scroll right as long as right arrow key is pressed. */
+
+void
+auto_scroll_right_arrow(Map *map)
+{
+	unsigned char keyMap[16];
+	short keyDown = TRUE;
+
+	while (keyDown) {
+		scroll_map_window(map, 4, 0);
+		GetKeys((unsigned long *) keyMap);
+		keyDown = keyMap[0x7C >> 3] >> (0x7C & 7) & 1;	/* right arrow */
+	}
+}
+
+/* Scroll up as long as up arrow key is pressed. */
+
+void
+auto_scroll_up_arrow(Map *map)
+{
+	unsigned char keyMap[16];
+	short keyDown = TRUE;
+
+	while (keyDown) {
+		scroll_map_window(map, 0, -4);
+		GetKeys((unsigned long *) keyMap);
+		keyDown = keyMap[0x7E >> 3] >> (0x7E & 7) & 1;	/* keypad 8 */
+	}
+}
+
+/* Scroll down as long as down key is pressed. */
+
+void
+auto_scroll_down_arrow(Map *map)
+{
+	unsigned char keyMap[16];
+	short keyDown = TRUE;
+
+	while (keyDown) {
+		scroll_map_window(map, 0, 4);
+		GetKeys((unsigned long *) keyMap);
+		keyDown = keyMap[0x7D >> 3] >> (0x7D & 7) & 1;	/* keypad 2 */
+	}
+}
+
+void
+rotate_angle_up(Map *map)
+{
+	if (map->vp->angle == 90)
+	  set_map_angle(map, 15);
+	else if (map->vp->angle == 15)
+	  set_map_angle(map, 30);
+	else
+	  set_map_angle(map, 90);
+}
+
+void
+rotate_angle_down(Map *map)
+{
+	if (map->vp->angle == 90)
+	  set_map_angle(map, 30);
+	else if (map->vp->angle == 30)
+	  set_map_angle(map, 15);
+	else
+	  set_map_angle(map, 90);
+}
+
 /* Dialog that sets the size of the offscreen scrolling buffers. */
 
 void
@@ -909,8 +1113,11 @@ set_offscreen_buffers(Map *map)
 	Rect itemrect;
 	Str255 tmpstr;
 	DialogPtr win;
+	GrafPtr oldport;
 
 	win = GetNewDialog(dOffscreen, NULL, (DialogPtr) -1);
+	GetPort(&oldport);	
+	SetPort(win);
 
 	GetDItem(win, diOffscreenHorizBuff, &itemtype, &itemhandle, &itemrect);
 	NumToString(map->max_bufx, tmpstr);
@@ -943,7 +1150,7 @@ set_offscreen_buffers(Map *map)
 				map->max_bufy = max(*itemnum, 0);
 				
 				/* Redraw the gworld (and the map) */
-				update_gworld(map);
+				force_map_update(map);
 				done = TRUE;
 				break;
 
@@ -957,16 +1164,75 @@ set_offscreen_buffers(Map *map)
 		}
 	}
 	DisposeDialog(win);
-
-// --- FASTER UPDATES ---------------------------------------
-
+	SetPort(oldport);
 	update_all_map_windows();
-	
-// -------------------------------------------------------
-
 }
 
-// --- NEW FUNCTIONS FOR FASTER UPDATE SUPPORT ---------------------------------
+/* Dialog that sets the size of the meridian interval for a map. */
+/* Clone of set_offscreen_buffers, should be moved elsewhere. */
+
+void
+set_meridians(Map *map)
+{
+	short ditem, itemtype, done = FALSE;
+	long *itemnum = 0;
+	Handle itemhandle;  
+	Rect itemrect;
+	Str255 tmpstr;
+	DialogPtr win;
+	GrafPtr oldport;
+
+	win = GetNewDialog(dMeridians, NULL, (DialogPtr) -1);
+	GetPort(&oldport);	
+	SetPort(win);
+
+	GetDItem(win, diMeridiansOnOff, &itemtype, &itemhandle, &itemrect);
+	SetCtlValue((ControlHandle) itemhandle, map->vp->draw_meridians);
+
+	GetDItem(win, diMeridiansInterval, &itemtype, &itemhandle, &itemrect);
+	NumToString(map->vp->meridian_interval, tmpstr);
+	SetIText(itemhandle, tmpstr);
+
+	ShowWindow(win);
+	while (!done) {
+
+		SetCursor(&QD(arrow));
+		draw_default_button(win, diMeridiansOK);
+		ModalDialog(NULL, &ditem);
+		switch (ditem) {
+
+			case diMeridiansOnOff:
+				GetDItem(win, ditem, &itemtype, &itemhandle, &itemrect);
+				SetCtlValue((ControlHandle) itemhandle, !GetCtlValue((ControlHandle) itemhandle));
+				break;
+
+			case diMeridiansOK:
+				GetDItem(win, diMeridiansOnOff, &itemtype, &itemhandle, &itemrect);
+				map->vp->draw_meridians = GetCtlValue((ControlHandle) itemhandle);
+				GetDItem(win, diMeridiansInterval, &itemtype, &itemhandle, &itemrect);
+	        			GetIText(itemhandle,tmpstr);
+				StringToNum(tmpstr, itemnum);
+				/* Dont allow negative values */
+				map->vp->meridian_interval = max(*itemnum, 0);
+
+				/* Redraw the gworld (and the map) */
+				force_map_update(map);
+				done = TRUE;
+				break;
+
+			case diMeridiansCancel:
+
+				done = TRUE;
+				break;
+
+			default:
+				break;
+		}
+	}
+	DisposeDialog(win);
+	SetPort(oldport);
+	update_all_map_windows();
+}
 
 void
 update_all_windows(void)
@@ -1010,3 +1276,25 @@ update_all_visible_map_windows(void)
 	}
 }
 
+void
+update_all_dialogs(void)
+{
+	WindowPtr win;
+	
+	for_all_windows(win) {
+		if (((WindowPeek) win)->windowKind == dialogKind)
+			update_window(win);
+	}
+}
+
+void
+update_all_visible_dialogs(void)
+{
+	WindowPtr win;
+	
+	for_all_windows(win) {
+		if (((WindowPeek) win)->windowKind == dialogKind
+		    && ((WindowPeek) win)->visible)
+			update_window(win);
+	}
+}
