@@ -77,7 +77,7 @@ create_list()
 
 	DGprintf("Creating a list\n");
 	/* Default to listing every side and indeps too. */
-	list->sides == ALLSIDES;
+	list->sides = ALLSIDES;
 	list->completed_units = TRUE;
 	list->incomplete_units = TRUE;
 	for (i = 0; i < MAXSORTKEYS; ++i)
@@ -86,6 +86,8 @@ create_list()
 	list->mainsortmi = miViewBySide;
 	list->largeicons = FALSE; /* (should be a preference) */
 	list->sidecolors = default_sidecolors;	/* same default as for maps */
+	list->drawsizes = default_drawsizes;	/* same default as for maps */
+	list->draw_emblems = default_draw_emblems;	/* same default as for maps */
 	/* (should share with toggle routine?) */
 	list->entryspacing = (list->largeicons ? largeentryspacing : smallentryspacing);
 	list->firstvisible = 0;
@@ -94,9 +96,11 @@ create_list()
 	list->next = listlist;
 	listlist = list;
 	/* Make a window for the list, give it scrollbars. */
-	list->window = GetNewWindow(wList, nil, (WindowPtr) -1L);
+	if (hasColorQD)
+		list->window = GetNewCWindow(wList, nil, NULL);
+	else	list->window = GetNewWindow(wList, nil, NULL);
 	stagger_window(list->window, &lastlisth, &lastlistv);
-	ShowWindow(list->window);
+	SelectTheWindow(list->window);
 	SetPort(list->window);
 	TextFont(small_font_id);
 	TextSize(small_font_size);
@@ -117,7 +121,9 @@ create_list()
 			 					  0, 0, 100, scrollBarProc, 0L);
 	/* Now set the scrollbars to their *real* limits. */
 	set_list_scrollbars(list);
-	sprintf(spbuf, "List %d", listnum++);
+	/* Needed for list number cleanup. */
+	list->id = listnum++;
+	sprintf(spbuf, "List %d", list->id);
 	add_window_menu_item(spbuf, list->window);
 }
 
@@ -246,7 +252,7 @@ draw_list(List *list)
 	/* Set up clipping for the contents of the list. */
 	cliprect = listwin->portRect;
 	cliprect.right -= sbarwid;
-	tmprect.bottom -= sbarwid;
+	cliprect.bottom -= sbarwid;
 	ClipRect(&cliprect);
 	draw_list_contents(list);
 	SetClip(tmprgn);
@@ -270,9 +276,10 @@ draw_list_contents(List *list)
 	for (line = list->firstvisible; line <= list->lastvisible; ++line) {
 		draw_unit_list_entry(list, line, FALSE);
 	}
-	HiliteControl(list->vscrollbar, ((numvisunits < list->numunits) ? 0 : 255));
+	/* The HiliteControl calls at this point caused a display bug. They were not
+	necessary since UpdateControls is smart enough to figure out if a scrollbar
+	should be displayed as active or not depending on the window size. */
 	viswidth = winrect.right - winrect.left - sbarwid;
-	HiliteControl(list->hscrollbar, ((viswidth < maxlistwidth) ? 0 : 255));
 }
 
 void
@@ -297,6 +304,9 @@ draw_list_headings(List *list)
 	Line(winrect.right, 0);
 	MoveTo(0, listtoph - 1);
 	Line(winrect.right, 0);
+	/* Draw a dividing line between the fixed and mobile fields. */
+	MoveTo(fixedfieldwidth - 15, 0);
+	LineTo(fixedfieldwidth - 15, listtoph - 1);
 	/* (should underline sort keys with varying line heaviness) */
 	/* We have to do MoveTo everywhere because DrawString moves the pen. */
 	/* First draw headings for fields not affected by horizontal scrolling. */
@@ -404,8 +414,15 @@ draw_unit_list_entry(List *list, int n, int clearfirst)
 	}
 	/* Draw an icon with side emblem for this unit. */
 	draw_unit_image(list->window, curactcolw + 2, y + 1,
-					(list->largeicons ? 32 : 16), (list->largeicons ? 32 : 16),
-					u, side_number(unit->side), !completed(unit), TRUE);
+					(list->largeicons ? 32 : 16), (list->largeicons ? 32 : 16), 
+					u, side_number(unit->side), list->sidecolors, 0, !completed(unit), list->draw_emblems);
+
+	/* Draw unit size on top of advanced unit. */
+	if (u_advanced(unit->type) && list->drawsizes) {
+		draw_unit_size(unit, list->window, 
+			curactcolw + 2, y + 1, (list->largeicons ? 32 : 16), (list->largeicons ? 32 : 16));
+	}
+
 	/* Write the name or ordinal number. */
 	name_or_number(unit, spbuf);
 	spbuf[15] = '\0';  /* (should be clipping by pixels) */
@@ -473,7 +490,7 @@ draw_unit_list_entry(List *list, int n, int clearfirst)
 	/* Draw the task. */
 	MoveTo(x, texty);
 	if (unit->plan && unit->plan->tasks) {
-		task_desc(spbuf, unit->side, unit->plan->tasks);
+		task_desc(spbuf, unit->side, unit, unit->plan->tasks);
 	} else {
 		strcpy(spbuf, "  -  ");
 	}
@@ -495,6 +512,13 @@ draw_unit_list_entry(List *list, int n, int clearfirst)
 	cliprect = list->window->portRect;
 	cliprect.right -= sbarwid;
 	ClipRect(&cliprect);
+
+	/* Draw a dividing line between the fixed and mobile fields. */
+	/* It may seem excessive to draw the whole line for each entry but 
+	otherwise the line may end before the bottom of the window content. */
+	MoveTo(fixedfieldwidth - 15, listtoph);
+	LineTo(fixedfieldwidth - 15, cliprect.bottom - sbarwid);
+
 	/* Highlight this entry if it was selected. */
 	if (list->contents->units[n].flag) {
 		InvertRect(&entryrect);
@@ -525,7 +549,7 @@ zoom_list(List *list, int part)
 		set_standard_state(listwin, maxlistwidth, maxh);
 	}
 	EraseRect(&listwin->portRect);
-	ZoomWindow(listwin, part, true);
+	ZoomWindow(listwin, part, false);
 	adjust_list_decor(list);
 }
 
@@ -539,9 +563,9 @@ adjust_list_decor(List *list)
 	w = list->window->portRect.right - list->window->portRect.left;
 	h = list->window->portRect.bottom - list->window->portRect.top;
 	MoveControl(list->vscrollbar, w - sbarwid, listtoph - 1);
-	SizeControl(list->vscrollbar, sbarwid + 1, h - listtoph - sbarwid + 1);
+	SizeControl(list->vscrollbar, sbarwid + 1, h - listtoph - sbarwid + 2);
 	MoveControl(list->hscrollbar, fixedfieldwidth - 1, h - sbarwid);
-	SizeControl(list->hscrollbar, w - fixedfieldwidth - sbarwid + 1, sbarwid + 1);
+	SizeControl(list->hscrollbar, w - fixedfieldwidth - sbarwid + 2, sbarwid + 1);
 	set_list_scrollbars(list);
 }
 
@@ -587,8 +611,6 @@ list_vscroll_fn(ControlHandle control, short code)
 	SetRect(&tmprect, 0, listtoph, listwin->portRect.right - sbarwid, listtoph + pagesize * curlist->entryspacing);
 	ScrollRect(&tmprect, 0, (oldvalue - curvalue) * curlist->entryspacing, tmprgn);
 	InvalRgn(tmprgn);
-	/* We'll need to redraw the headings line. */
-	draw_list_headings(curlist);
 	/* Do the update now, because we won't get back to the main event loop
 	   until the mouse button is released. */
 	update_window(curlist->window);
@@ -606,6 +628,7 @@ list_hscroll_fn(ControlHandle control, short code)
 	oldvalue = GetCtlValue(control);
 	minvalue = GetCtlMin(control);
 	maxvalue = GetCtlMax(control);
+
 	pagesize = curlist->lastvisfield - curlist->firstvisfield;
 	switch (code) {
 		case inPageDown:
@@ -628,6 +651,23 @@ list_hscroll_fn(ControlHandle control, short code)
 	curlist->firstvisfield = curvalue;
 	curlist->lastvisfield = curlist->firstvisfield + pagesize;
 	SetCtlValue(control, curvalue);
+
+#if 1
+
+/* 	New code that fixes horisontal scrolling bug. */
+
+	/* Return to avoid flicker if stuck at the end of the scrollbar. */
+	if (curvalue == oldvalue) return;
+	/* Update everything but the fixed field. */
+	InvalRect(&listwin->portRect);
+	SetRect(&tmprect, 0, 0, fixedfieldwidth, listwin->portRect.bottom);
+	ValidRect(&tmprect);
+	BeginUpdate(listwin);		
+	draw_list(curlist);
+	EndUpdate(listwin);
+
+#else
+
 	/* Scroll the already-drawn bits. */
 	tmprgn = NewRgn();
 	SetRect(&tmprect, 0, listtoph, listwin->portRect.right - sbarwid, listwin->portRect.bottom - sbarwid);
@@ -639,8 +679,53 @@ list_hscroll_fn(ControlHandle control, short code)
 	   until the mouse button is released. */
 	update_window(curlist->window);
 	DisposeRgn(tmprgn);
+
+#endif
+
 }
 
+int
+do_key_down_list(List *list, char key)
+{
+	int		handled = TRUE;
+	GrafPtr	oldport;
+
+	GetPort(&oldport);
+	SetPort(list->window);
+	curlist = list;
+
+	switch (key) {
+		case 27:		/* Escape key. */
+			close_window(curlist->window);
+			break;
+		case 0x1F:		/* Down Arrow. */
+			list_vscroll_fn(curlist->vscrollbar, inDownButton);
+			break;
+		case 0x1E:		/* Up Arrow. */
+			list_vscroll_fn(curlist->vscrollbar, inUpButton);
+			break;
+		case 0x1D:		/* Right Arrow. */
+			list_hscroll_fn(curlist->hscrollbar, inDownButton);
+			break;
+		case 0x1C:		/* Left Arrow. */
+			list_hscroll_fn(curlist->hscrollbar, inUpButton);
+			break;
+		case 0x0C:		/* Page Down. */
+			list_vscroll_fn(curlist->vscrollbar, inPageDown);
+			break;
+		case 0x0B:		/* Page Up. */
+			list_vscroll_fn(curlist->vscrollbar, inPageUp);
+			break;
+		default:
+			/* The key was not handled here. */
+			handled = FALSE;
+			break;
+	}
+	SetPort(oldport);
+	return handled;
+}
+
+
 /* Handle a mouse down in the list.  Grafport already set, mouse coords are local. */
 
 /* (mouse downs should select/deselect list elements) */
@@ -755,6 +840,13 @@ toggle_list_sidecolors(List *list)
 	force_update(list->window);
 }
 
+void
+toggle_list_draw_emblems(List *list)
+{
+	list->draw_emblems = !list->draw_emblems;
+	force_update(list->window);
+}
+
 void
 update_unit_in_lists(Unit *unit)
 {
@@ -863,30 +955,14 @@ activate_list(List *list, int activate)
 {
 	Rect growRect;
 
+	GrafPtr oldport;
+
 	if (activate) {
 		HiliteControl(list->vscrollbar, 0);
 		HiliteControl(list->hscrollbar, 0);
-#if 0
-		/* the controls must be redrawn on activation: */
-		(*(list->vscrollbar))->contrlVis = 255;
-		(*(list->hscrollbar))->contrlVis = 255;
-		InvalRect(&(*(list->vscrollbar))->contrlRect);
-		InvalRect(&(*(list->hscrollbar))->contrlRect);
-#endif
-		/* The growbox needs to be redrawn on activation. */
-		growRect = list->window->portRect;
-		/* adjust for the scrollbars */
-		growRect.top = growRect.bottom - sbarwid;
-		growRect.left = growRect.right - sbarwid;
-		InvalRect(&growRect);
 	} else {
-		/* The scrollbars must be hidden on deactivation. */
 		HiliteControl(list->vscrollbar, 255);
 		HiliteControl(list->hscrollbar, 255);
-/*		HideControl(list->vscrollbar);
-		HideControl(list->hscrollbar); */
-		/* The growbox should be changed immediately on deactivation. */
-		DrawGrowIcon(list->window);
 	}
 }
 
@@ -916,8 +992,16 @@ destroy_list(List *list)
 			if (list2->next == list) {
 				list2->next = list->next;
 			}
+			if (list2->id > list->id) {
+				remove_window_menu_item(list2->window);
+				--list2->id;
+				sprintf(spbuf, "List %d", list2->id);
+				add_window_menu_item(spbuf, list2->window);
+			}
 		}
 	}
+	--listnum;
 	/* (should destroy substructs) */
 	free(list);
 }
+
