@@ -1,10 +1,4 @@
 /* 	Mouse-driven user interaction for the Mac interface to Xconq.
-
-	This file now ONLY contains user interaction code for the mouse
-	such as clicking, dragging, selecting, scrolling and toggling items.
-     	It has therefore been renamed from macmap2.c to macmouse.c.    
-	All other map-related functions have been moved to macmap.c.
-     
 	Copyright (C) 1992-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
@@ -12,27 +6,25 @@ it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
+/* This file now ONLY contains user interaction code for the mouse
+   such as clicking, dragging, selecting, scrolling and toggling items. */
+     
 #include "conq.h"
-#include "kpublic.h"
 #include "macconq.h"
 
-#undef DEF_CMD
-#define DEF_CMD(letter,name,args,FN,help) extern void FN(void);
+extern int side_owns_occupant PARAMS ((Side *side, Unit *unit));
 
-#include "cmd.def"
-#include "maccmd.def"
+extern void set_position_modally(void);
+extern void do_closeup_mi(void);
 
 static int selrect;
 static int downx, downy, downdir;
 static void drag_for_distance(Map *map, int h0, int v0);
 
-extern void set_position_modally(void);
-extern int side_owns_occupant PARAMS ((Side *side, Unit *unit));
-extern int mapnum;
-extern int nummaps;
-
 int topunithgt;
+
 Map *curmap;
+
 ControlActionUPP map_scroll_proc;
 
 /* This scroll proc is shared by both the horizontal and vertical scrollbars. */
@@ -104,7 +96,7 @@ scroll_best_map_to_unit(Unit *unit, int bringtofront)
 		SetPort(oldport);
 	}
 	if (bringtofront) {
-		SelectWindow(bestmap->window);
+		SelectTheWindow(bestmap->window);
 		update_window(bestmap->window);
 		adjust_menus();
 	}
@@ -203,7 +195,7 @@ do_mouse_down_map(Map *map, Point mouse, int mods)
 				part = TrackControl(control, mouse, map_scroll_proc);
 				break;
 		}
-	} else if (mouse.h <= conwid) {
+	} else if (mouse.h <= map->conw) {
 		/* Interpret as a control panel hit. */
 		do_mouse_down_map_control_panel(map, mouse.h, mouse.v, mods);
 	} else {
@@ -217,10 +209,12 @@ do_mouse_down_map_control_panel(Map *map, int h, int v, int mods)
 	int winh = map->window->portRect.bottom - map->window->portRect.top;
 
 	/* (should better organize tests here) */
-	if (between(winh - 2 * sbarwid, v, winh)) {
-		switch ((winh - v) / sbarwid) {
+
+	/* Use 15 instead of map->sbarwid here, since the latter may be zero. */
+	if (between(winh - 2 * 15, v, winh)) {
+		switch ((winh - v) / 15) {
 			case 0:
-				magnify_map(map, ((h < conwid / 2) ? -1 : 1));
+				magnify_map(map, ((h < map->conw / 2) ? -1 : 1));
 				break;
 #if 0
 			/* This was too confusing here, so it's now flushed.  However, the
@@ -235,28 +229,28 @@ do_mouse_down_map_control_panel(Map *map, int h, int v, int mods)
 	} else if ((v - 32) < 5 * 15) {
 		switch ((v - 32) / 15) {
 			case 0:
-				if (h < conwid / 2) {
+				if (h < map->conw / 2) {
 					select_previous_awake_mover(map);
 				} else {
 					select_next_awake_mover(map);
 				}
 				break;
 			case 1:
-				if (h < conwid / 2) {
+				if (h < map->conw / 2) {
 					select_previous_mover(map);
 				} else {
 					select_next_mover(map);
 				}
 				break;
 			case 2:
-				if (h < conwid / 2) {
+				if (h < map->conw / 2) {
 					select_previous_actor(map);
 				} else {
 					select_next_actor(map);
 				}
 				break;
 			case 3:
-				if (h < conwid / 2) {
+				if (h < map->conw / 2) {
 					select_previous_unit(map);
 				} else {
 					select_next_unit(map);
@@ -294,8 +288,13 @@ do_mouse_down_map_control_panel(Map *map, int h, int v, int mods)
 				toggle_map_ai(map);
 				break;
 			case 6:
-				if (dside->may_set_see_all) {
+				if (dside->may_set_show_all) {
 					map->see_all = !map->see_all;
+					map->vp->show_all = map->see_all;
+					/* This is necessary in order to enable closeups that have no map 
+					or vp associated with them. Also needed to permit the designer to 
+					select an enemy unit. */
+					dside->see_all = map->see_all;
 					force_map_update(map);
 				}
 				break;
@@ -366,12 +365,11 @@ set_map_mag(Map *map, int newpower)
 		m_center_on_focus(map);
 		set_map_scrollbars(map);
 
-		/* Update the gworld (and the map) */
-		update_gworld(map);
+		/* Update the entire map */
+		force_map_update(map);
 
 		/* Redraw other-map boxes in other windows */
 		draw_related_maps(map);
-		
 	}
 }
 
@@ -386,16 +384,25 @@ toggle_map_grid(Map *map)
 void
 toggle_map_topline(Map *map)
 {
-	Rect tmprect;
-	RgnHandle tmprgn;
-	GrafPtr oldport;
 	int oldtoph = map->toph;
+	GrafPtr oldport;
 
+	/* Setting the port fixed a display bug. */
+	GetPort(&oldport);
+	SetPort(map->window);
 	map->draw_topline = !map->draw_topline;
 	map->toplineh = (map->draw_topline ? tophgt : 0);
 	/* Erase other-map boxes in other windows. */
 	draw_related_maps(map);
 	map->toph = map->toplineh + map->topunith;
+
+#if 1
+
+	/* This is a temp fix. It should be possible to get the code below to work! */
+	force_map_update(map);
+
+#else
+
 	/* Update without scrolling if we are expanding and the buffer is too small. */ 
 	if (map->bufy - map->offsety < oldtoph - map->toph) {
 		set_content_rect(map);
@@ -407,18 +414,23 @@ toggle_map_topline(Map *map)
 		  draw_unit_info(map);
 	/* Always scroll the map if we are shrinking, or if the buffer is big enough. */
 	} else update_resized_map(map);
+
+#endif
+
 	/* Redraw other-map boxes in other windows. */
 	draw_related_maps(map);
+	SetPort(oldport);
 }
 
 void
 toggle_map_topunit(Map *map)
 {
-	int oldtoph;
-	Rect tmprect;
-	RgnHandle tmprgn;
 	GrafPtr oldport;
+	int oldtoph;
 
+	/* Setting the port fixed a display bug. */
+	GetPort(&oldport);
+	SetPort(map->window);
 	oldtoph = map->toph;
 	map->draw_topunit = !map->draw_topunit;
 	map->topunith = (map->draw_topunit ? topunithgt : 0);
@@ -438,6 +450,41 @@ toggle_map_topunit(Map *map)
 	} else update_resized_map(map);
 	/* Redraw other-map boxes in other windows. */
 	draw_related_maps(map);
+	SetPort(oldport);
+}
+
+void
+toggle_map_scrollbars(Map *map)
+{
+	/* Erase other-map boxes in other windows. */
+	draw_related_maps(map);
+
+	if (map->sbarwid)
+		map->sbarwid = 0;
+	else if (map == worldmap)
+		map->sbarwid = floatsbarwid;
+	else	map->sbarwid = sbarwid;
+	
+	adjust_map_decor(map);
+	force_map_update(map);
+
+	/* Redraw other-map boxes in other windows. */
+	draw_related_maps(map);
+}
+
+void
+toggle_map_cpanel(Map *map)
+{
+	/* Erase other-map boxes in other windows. */
+	draw_related_maps(map);
+
+	/* Toggle the panel. */
+	map->conw = (map->conw > 0 ? 0 : conwid);
+	adjust_map_decor(map);
+	force_map_update(map);
+
+	/* Redraw other-map boxes in other windows. */
+	draw_related_maps(map);
 }
 
 void
@@ -559,12 +606,8 @@ void
 toggle_map_plans(Map *map)
 {
 	map->vp->draw_plans = !map->vp->draw_plans;
-	if (map->numselections > 0) {
-		force_map_update(map);
-	} else {
-		/* (should be a force update on control panel alone) */
-		draw_control_panel(map);
-	}
+
+	force_map_update(map);
 }
 
 void
@@ -583,6 +626,13 @@ toggle_map_sidecolors(Map *map)
 	force_map_update(map);
 }
 
+void
+toggle_map_draw_emblems(Map *map)
+{
+	map->draw_emblems = !map->draw_emblems;
+	force_map_update(map);
+}
+
 void
 toggle_iconmasks(Map *map)
 {
@@ -640,9 +690,9 @@ toggle_optimize_fonts(Map *map)
 }
 
 void
-toggle_draw_latlong(Map *map)
+toggle_draw_meridians(Map *map)
 {
-	map->vp->draw_latlong = !map->vp->draw_latlong;
+	map->vp->draw_meridians = !map->vp->draw_meridians;
 	force_map_update(map);
 }
 
@@ -653,6 +703,20 @@ toggle_solid_color_terrain(Map *map)
 	force_map_update(map);
 }
 
+void
+toggle_blinking_curunit(Map *map)
+{
+	map->blinking_curunit = !map->blinking_curunit;
+	force_map_update(map);
+}
+
+void
+toggle_selectionmasks(Map *map)
+{
+	map->selectionmasks = !map->selectionmasks;
+	force_map_update(map);
+}
+
 void
 toggle_erase_names(Map *map)
 {
@@ -672,23 +736,40 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 	if (inside_area(downx, downy)) {
 		map->vp->vcx = downx;  map->vp->vcy = downy;
 	}
+	/* First handle refocus clicks in world map. */
+	if (map == worldmap) {
+		WindowPtr	 win;
+		Map *nextmap;
+
+		/* Find the next frontmost map. */
+		for_all_windows(win) {
+			if (map_from_window(win) != NULL
+			     && win != FrontWindow()) {
+			       	nextmap = map_from_window(win);
+				/* Scroll next map to the new focus. */
+				set_map_focus(nextmap, downx, downy);		
+				break;
+			}
+		}
+		return;
+	}		
 	if (map_modal != NO_MODAL) {
 		switch (map_modal) {
 			case ATTACK_MODAL:
 				set_position_modally();
-				do_attack();
+				do_attack(dside);
 				break;
 			case FIRE_MODAL:
 				set_position_modally();
-				do_fire();
+				do_fire(dside);
 				break;
 			case FIRE_INTO_MODAL:
 				set_position_modally();
-				do_fire_into();
+				do_fire_into(dside);
 				break;
 			case SET_FORMATION_MODAL:
 				set_position_modally();
-				do_set_formation();
+				do_set_formation(dside);
 				break;
 			case MOVE_TO_MODAL:
 				set_position_modally();
@@ -696,11 +777,11 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 				break;
 			case ADD_TERRAIN_MODAL:
 				set_position_modally();
-				do_add_terrain();
+				do_add_terrain(dside);
 				break;
 			case REMOVE_TERRAIN_MODAL:
 				set_position_modally();
-				do_remove_terrain();
+				do_remove_terrain(dside);
 				break;
 			case DISTANCE_MODAL:
 				drag_for_distance(map, h, v);
@@ -722,12 +803,14 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 		if (map->moveonclick && map->autoselect) {
 			unselect_all(map);
 			m_nearest_unit(map, h, v, &unit);
-			if (unit != NULL && (side_controls_unit(dside, unit) || endofgame)) {
-				/* The nearest unit will become the "current unit". */
+			/* Always select the unit if it exists and we may examine it. */
+			if (side_sees_unit(dside, unit)) {
 				map->curunit = unit;
 				select_unit_on_map(map, unit);
 				draw_selections(map);
-				move_on_drag(map, unit, mods);
+				/* But only move on drag if we control it. */
+				if (side_controls_unit(dside, unit))
+					move_on_drag(map, unit, mods);
 			} else {
 				select_all_dragged_over(map, h, v, mods);
 				/* Pick the first of the multiple selection as the "current unit". */
@@ -738,7 +821,9 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 		} else {
 			anysuccess = FALSE;
 			for (i = 0; i < map->numselections; ++i) {
-				if ((unit = map->selections[i]) != NULL) {
+				/* Only move those selected units that we control. */ 
+				unit = map->selections[i];
+				if (side_controls_unit(dside, unit)) {
 					rslt = move_the_selected_unit(map, unit, h, v);
 					if (rslt)
 					  anysuccess = TRUE;
@@ -749,7 +834,9 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 		}
 	} else if (mods & optionKey) {
 		for (i = 0; i < map->numselections; ++i) {
-			if ((unit = map->selections[i]) != NULL) {
+			/* Only fire those selected units that we control. */ 
+			unit = map->selections[i];
+			if (side_controls_unit(dside, unit)) {
 				fire_the_selected_unit(map, unit, h, v);
 			}
 		}
@@ -759,7 +846,7 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 			/* Invert the selection status of the unit. */
 			if (unit_is_selected(map, unit)) {
 				unselect_unit_on_map(map, unit);
-				erase_selection(map, unit);
+				update_cell(map, unit->x, unit->y);
 			} else {
 				select_unit_on_map(map, unit);
 				draw_selections_at(map, unit->x, unit->y);
@@ -768,31 +855,34 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 		} else {
 			select_all_dragged_over(map, h, v, mods);
 		}
-
-	/* Focus on clicked point and select closest own unit if control key is down. */
 	} else if (mods & controlKey) {
+		m_nearest_unit(map, h, v, &unit);
+		/* Open closeup if clicking on a unit that we may examine. */
+		if (map->see_all || side_sees_unit(dside, unit)) {
+			show_unit_closeup(unit);
+		/* Else focus on clicked spot and select closest own unit. */
+		} else {
+			long	unitdist, mindist = 32000;
 
-		long unitdist, mindist = 32000;
-
-		/* Focus on clicked point */
-		set_focus(map, downx, downy);
-		/* Find nearest own unit and select it */
-	    	for (i = 0; i < dside->actionvector->numunits; ++i) {
-	    		unit = (dside->actionvector->units)[i].unit;
-   			 if (could_be_next_unit(unit) && side_controls_unit(dside, unit)) {
-				/* Use least square method */
-				unitdist = (downx - unit->x) * (downx - unit->x) + 
-					        (downy - unit->y) * (downy - unit->y);
-				if (unitdist < mindist) {
-					mindist = unitdist;
-					unselect_all(map);
-					map->curunit = unit;
-				}
+			/* Focus on the clicked point. */
+			set_map_focus(map, downx, downy);
+			for_all_side_units(dside, unit) {
+   				if (side_controls_unit(dside, unit)) {
+					/* Use least square method. */
+					unitdist = (downx - unit->x) * (downx - unit->x) + 
+					        	        (downy - unit->y) * (downy - unit->y);
+					if (unitdist < mindist) {
+						mindist = unitdist;
+						map->curunit = unit;
+					}
+		    		}
 		    	}
-	    	}
-		select_unit_on_map(map, map->curunit);
-		draw_selections(map);
-
+			if (map->curunit) {
+				unselect_all(map);
+				select_unit_on_map(map, map->curunit);
+				draw_selections(map);
+			}
+		}
 	} else {
 		/* Interpret an unmodified mouse down. */
 #ifdef DESIGNERS
@@ -804,7 +894,9 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 			/* Usually will only be one to move, but be general anyway. */
 			anysuccess = FALSE;
 			for (i = 0; i < map->numselections; ++i) {
-				if ((unit = map->selections[i]) != NULL) {
+				/* Only move those selected units that we control. */ 
+				unit = map->selections[i];
+				if (side_controls_unit(dside, unit)) {
 					rslt = move_the_selected_unit(map, unit, h, v);
 					if (rslt)
 					  anysuccess = TRUE;
@@ -816,12 +908,19 @@ do_mouse_down_map_content(Map *map, int h, int v, int mods)
 		} else {
 			unselect_all(map);
 			m_nearest_unit(map, h, v, &unit);
-			if (unit != NULL && (side_controls_unit(dside, unit) || endofgame)) {
+			/* Always select the unit if it exists and we may examine it. */
+			if (side_sees_unit(dside, unit)) {
+				map->curunit = unit;
 				select_unit_on_map(map, unit);
 				draw_selections(map);
-				move_on_drag(map, unit, mods);
+				/* But only move on drag if we control it. */
+				if (side_controls_unit(dside, unit))
+					move_on_drag(map, unit, mods);
 			} else {
 				select_all_dragged_over(map, h, v, mods);
+				/* Pick the first of the selection as the current unit. */
+				if (map->numselections > 0)
+					map->curunit = map->selections[0];
 			}
 		}
 	}
@@ -1001,13 +1100,16 @@ drag_for_distance(Map *map, int h0, int v0)
 void
 unselect_all(Map *map)
 {
-	int num = map->numselections;
+	Unit *unit;
 
-	if (map->numselections > 0) {
-		erase_selections(map);
-		map->numselections = 0;
-		draw_unit_info(map);
+	while (map->numselections > 0) {
+		/* Start from top of selection list. */
+		unit = map->selections[map->numselections - 1]; 	
+		/* Remove unit from selection list before updating its cell. */
+		--map->numselections;
+		update_cell(map, unit->x, unit->y);
 	}
+	draw_unit_info(map);
 }
 
 /* Add the given unit to the array of units selected in the given map.  If we need
@@ -1077,17 +1179,16 @@ select_all_units_in_rect(Map *map, Rect *rectptr)
 	/* Now look at all the plausible units and see if any's image intersects the rect. */
 	for_all_units(unit) {
 		if (in_play(unit)
-			&& (side_controls_unit(dside, unit) || endofgame)
-			&& (rectissmall || unit->transport == NULL)) {
+		     && side_sees_unit(dside, unit)
+		     && (rectissmall || unit->transport == NULL)) {
 			m_xform_unit_self(map, unit, &sx, &sy, &sw, &sh);
 			SetRect(&unitrect, sx, sy, sx + sw, sy + sh);
 			if (SectRect(&unitrect, rectptr, &tmprect)) {
 				select_unit_on_map(map, unit);
-				/* (could do setport etc once...) */
-				draw_selected_unit_setport(map, unit);
 			}
 		}
 	}
+	draw_selections(map);
 }
 
 /* This translates the user's "go to here" into appropriate tasks and/or actions. */
@@ -1221,7 +1322,7 @@ select_another(Map *map, Unit *(*fn)(Side *side, Unit *unit))
 		scroll_to_unit(map, thisunit);
 		/* (should not be done this way, but how else?) */
 		if (map->autoselect
-			&& (thisunit->act && thisunit->act->acp > 0)
+			&& has_acp_left(thisunit)
 			&& (thisunit->plan && !thisunit->plan->asleep && !thisunit->plan->reserve && !thisunit->plan->delayed)) {
 			map->curunit = thisunit;
 		}
