@@ -34,37 +34,37 @@ for i in $list ; do
 	echo "" >> $logname
 	echo ">>> Running skelconq on: " $i "<<<" >> $slogname
 	../kernel/skelconq -f $i -L $srcdir/../lib <$srcdir/savetst1.inp >> $logname
-	if [ -f ~/.xconq/save.xconq ] ; then
-		mv ~/.xconq/save.xconq save1.xconq
+	if [ -f ~/.xconq/save.xcq ] ; then
+		mv ~/.xconq/save.xcq save1.xcq
 		echo ">>> Restoring skelconq; <<<" >> $logname
-		../kernel/skelconq -f save1.xconq -noai <$srcdir/savetst2.inp >> $logname
-		if [ -f ~/.xconq/save.xconq ] ; then
+		../kernel/skelconq -f save1.xcq -noai <$srcdir/savetst2.inp >> $logname
+		if [ -f ~/.xconq/save.xcq ] ; then
 			sed -e '/run-serial-number/d' \
 			    -e '/elapsed-real-time/d' \
 			    -e '/ai-type-name/d' \
 			    -e '/game-saved/d' \
 			    -e '/log-started/d' \
 			    -e '/game-restarted/d' \
-			    save1.xconq >save1f.xconq
-			mv ~/.xconq/save.xconq save2.xconq
+			    save1.xcq >save1f.xcq
+			mv ~/.xconq/save.xcq save2.xcq
 			sed -e '/run-serial-number/d' \
 			    -e '/elapsed-real-time/d' \
 			    -e '/ai-type-name/d' \
 			    -e '/game-saved/d' \
 			    -e '/log-started/d' \
 			    -e '/game-restarted/d' \
-			    save2.xconq >save2f.xconq
-			if cmp save1f.xconq save2f.xconq >/dev/null ; then
+			    save2.xcq >save2f.xcq
+			if cmp save1f.xcq save2f.xcq >/dev/null ; then
 				echo ">>> Savefiles match <<<" >> $logname
 			else
-				echo ">>> diff -u save1f.xconq save2f.xconq <<<" >> $logname
-				diff -u save1f.xconq save2f.xconq >> $logname
-				echo ">>> diff -u0 save1f.xconq save2f.xconq <<<" >> $slogname
-				diff -u0 save1f.xconq save2f.xconq | grep -v '^@' >> $slogname
-				echo `diff -u0 save1f.xconq save2f.xconq | grep -v '^[0-9]' | wc -l` lines of difference
+				echo ">>> diff -u save1f.xcq save2f.xcq <<<" >> $logname
+				diff -u save1f.xcq save2f.xcq >> $logname
+				echo ">>> diff -u0 save1f.xcq save2f.xcq <<<" >> $slogname
+				diff -u0 save1f.xcq save2f.xcq | grep -v '^@' >> $slogname
+				echo `diff -u0 save1f.xcq save2f.xcq | grep -v '^[0-9]' | wc -l` lines of difference
 				gamename=`echo $i | sed -e 's,.*/lib/,,'`
-				mv save1f.xconq save1f.$gamename.xconq
-				mv save2f.xconq save2f.$gamename.xconq
+				mv save1f.xcq save1f.$gamename.xcq
+				mv save2f.xcq save2f.$gamename.xcq
 			fi
 		else
 			echo Second savefile missing!
