@@ -26,8 +26,6 @@ Display *tmp_display;
 
 Window tmp_root_window;
 
-extern char *imflib;
-
 static Pixmap x11_load_bitmap PARAMS ((Display *dpy, Window rootwin,
 				       char *name, char *ext, int *w, int *h));
 static void x11_make_color_pixmap PARAMS ((Display *dpy, Window rootwin,
@@ -161,13 +159,6 @@ int *wp, *hp;
 	    *wp = w;  *hp = h;
 	    return rslt;
 	}
-	make_pathname(imflib, name, ext, sbuf);
-	if (XReadBitmapFile(dpy, rootwin, sbuf,
-			    &w, &h, &rslt, &hotx, &hoty) == BitmapSuccess) {
-	    DGprintf("Loaded bitmap \"%s\"\n", sbuf);
-	    *wp = w;  *hp = h;
-	    return rslt;
-	}
     }
     return None;
 }
